# -*- coding: utf-8 -*-
""" Основной модуль программы NodeVolt, который запускает графический интерфейс.
"""

# TODO: + Сделать автоматическую простановку реального напряжения и угла поворота фазы на основе данных трансформаторов.
# TODO: Скрыть реальное напряжение и фазу реального напряжения у всех элементов.
# TODO: Добавить учет рабочей температуры элементов.
# TODO: Добавить возможность создаватать режимы работы схемы.
# TODO: Работа с однофазной схемой.
# TODO: Работа с трехфазной не симметричной схемой.
# TODO: Создать вольтодобавочный трансформатор.
# TODO: Элементы по схеме должны перемещаться с привязкой к сетке.
# TODO: Добавить возможность экспорта таблиц элементов и узлов вместе с результатами расчета.
# TODO: Добавить возможность экспорта результатов расчета в PDF.
# TODO: Добавить возможность преобразовывать элементы из одного вида в другой.

import cPickle
import os
import sys
import time
from PyQt4 import QtGui, QtCore
import classes
import graphics
from gui import node_volt_ui, scheme_settings_ui, settings_ui, start_window_ui, progress_window_ui, style
import gui_classes
import language
import models
import node_volt_calc_plus as nv_plus
import node_volt_calc as nv
import optimization
import prog_settings

#from meliae import scanner
#scanner.dump_all_objects('mem_profile')

# Расположение вкладок "Схема", "Элементы", "Узлы", "Результаты".
TAB_BRANCHES = {'scheme': 0, 'vetvi': 1, 'usli': 2, 'results': 3, 'optimization_results': 4}


def module_path():
    """ Функция, определяющая полный путь к программе."""
    if hasattr(sys, "frozen"):
        return os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))
    return os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))


try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return unicode(app.translate(context, text, disambig, _encoding))
except AttributeError:
    def _translate(context, text, disambig):
        return unicode(app.translate(context, text, disambig))


def message(mes_type, title=u'', mes_list=[], parent=None):
    """ Функция, отображающая окно сообщения из списка сообщений.

    Функция получает тип сообщения (информация или предупреждение), заголовок окна и список сообщений и
    показывает на экране окно с этими сообщениями.

    Args:
        mes_type: предупреждение (warning) или информация (information).
        title: заголовок окна сообщения.
        mes_list: список сообщений (unicode строки).
        parent: родительский объект.
    """
    # Строка сообщений, которая отображается в окне.
    mes = u''
    # Создание строки сообщения из списка сообщений.
    for err in mes_list:
        mes += u'%s\n' % err
    # В зависимости от типа сообщения выводится на экран тот или иной вид окна.
    if mes_type == 'warning':
        QtGui.QMessageBox.warning(parent, title, mes, QtGui.QMessageBox.Ok)
    elif mes_type == 'information':
        QtGui.QMessageBox.information(parent, title, mes, QtGui.QMessageBox.Ok)


class NewElementsTable(QtGui.QListView):
    """ Класс, который описывает работу списков добавления новых элементов.
    """

    def __init__(self, elements_group_name, new_elements_obj=gui_classes.NewElements(), parent=None):
        QtGui.QListView.__init__(self, parent)
        self.elements_group_name = elements_group_name
        self.new_elements_obj = new_elements_obj
        self.elements_group_obj = None
        self.elements_group_list_model = None
        self.read_elements_group()
        # Установка размеров списка.
        width = 400
        if self.elements_group_obj and self.elements_group_obj is not None:
            font = self.font()
            fm = QtGui.QFontMetricsF(font)
            values_list = [self.elements_group_obj.child(i).value()
                           for i in range(self.elements_group_obj.child_count())]
            max_len_name = ''
            max_len = 0
            for value in values_list:
                if len(value) > max_len:
                    max_len = len(value)
                    max_len_name = value
            width = fm.width(max_len_name) + 100
            height = min(400, len(values_list) * 52)
        self.setMinimumSize(QtCore.QSize(width, height))
        self.setMaximumSize(QtCore.QSize(400, 400))

        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
        self.hide_list()
        self.connect_actions()
        self.setStyleSheet(style.NEW_ELEMENTS_LIST_STYLE_SHEET)

    def connect_actions(self):
        self.clicked.connect(self.add_element)

    def add_element(self):
        """ Метод, вызываемый при выборе элемента из списка."""
        # Скрыть список добавления новых элементов.
        self.hide_list()
        # Список выбранных элементов.
        selection = self.selectedIndexes()
        # Метод работает только, если выбран один элемент.
        if len(selection) == 1:
            # Порядковый номер выбранного элемента.
            row = selection[0].row()
            # Если задана модель списка добавления новых элементов.
            if self.elements_group_list_model is not None:
                # Поиск объекта нового элемента.
                element_obj = self.elements_group_obj.child(row)
                # Если родитель данного класса - основное окно программы.
                if isinstance(self.parent(), QtGui.QMainWindow):
                    # Вызов метода добавления нового элемента на схему.
                    self.parent().add_new_element(self.elements_group_name, element_obj)

    def read_elements_group(self):
        # Чтение параметров группы элементов.
        self.elements_group_obj = self.new_elements_obj.find_child(self.elements_group_name)
        if self.elements_group_obj:
            self.elements_group_list_model = models.NewElementsTableModel(self.elements_group_obj)
            self.setModel(self.elements_group_list_model)
            self.setIconSize(QtCore.QSize(50, 50))
        else:
            self.elements_group_list_model = None
            self.setModel(None)

    def show_list_in_cursor(self):
        """Показать список на месте расположения курсора."""
        if self.isVisible() and self.isEnabled():
            self.hide_list()
        else:
            cursor_position = QtGui.QCursor.pos()
            if isinstance(self.parent(), QtGui.QMainWindow):
                x = cursor_position.x() - self.parent().pos().x()
                y = cursor_position.y() - self.parent().pos().y()
                cursor_position = QtCore.QPoint(x, y)
                self.parent().hide_all_new_elements_lists()
            self.setVisible(True)
            self.setEnabled(True)
            self.move(cursor_position)

    def hide_list(self):
        """ Скрыть список."""
        self.setVisible(False)
        self.setEnabled(False)


class Settings(nv.Settings):
    """ Класс для хранения настроек программы.

    В этом классе дополнительно описаны настройки граффического интерфейса и
    методы для сохранения и чтения настроек из файла.
    """

    def __init__(self):
        nv.Settings.__init__(self)
        self.tree_properties_column_widths = [200,  100]
        self.language = 'ru - Russian'
        self.geometry = None
        self.splitter_scheme = None

    def read_gui_settings(self):
        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")
        qt_settings.beginGroup('gui')
        # Считываение ширин столбцов таблицы параметров.
        column_widths = qt_settings.value('tree_properties_column_widths', [200,  100]).toList()
        self.tree_properties_column_widths = [200,  100]
        for i, column_width in enumerate(column_widths):
            column_width = int(column_width.toInt()[0])
            self.tree_properties_column_widths[i] = column_width
        # Считывание языка.
        self.language = qt_settings.value('language', 'ru - Russian').toString()
        # Считывание геометрии основного окна.
        self.geometry = qt_settings.value('geometry', None).toByteArray()
        # Считывание положения разделителя между схемой и таблицей параметров на вкладке "Схема".
        self.splitter_scheme = qt_settings.value('splitter_scheme', None).toByteArray()
        qt_settings.endGroup()

    def write_gui_settings(self):
        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")
        qt_settings.beginGroup('gui')
        # Запись ширин столбцов таблицы параметров.
        for i, column_width in enumerate(self.tree_properties_column_widths):
            if column_width < 10:
                self.tree_properties_column_widths[i] = 10
        qt_settings.setValue('tree_properties_column_widths', self.tree_properties_column_widths)
        # Запись языка.
        qt_settings.setValue('language', self.language)
        # Запись геометрии основного окна.
        qt_settings.setValue('geometry', self.geometry)
        # Запись положения разделителя между схемой и таблицей параметров на вкладке "Схема".
        qt_settings.setValue('splitter_scheme', self.splitter_scheme)
        qt_settings.endGroup()

    def read_recent_files(self):
        # Счтиывание последних открытых файлов.
        recent_files = []
        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")
        for i in range(0, 5):
            if not qt_settings.contains(u'recent_files/file_%s' % i):
                break
            recent_files.append(unicode(qt_settings.value(u'recent_files/file_%s' % i).toString()))
        return recent_files

    def add_recent_file(self, new_recent_file=u''):
        recent_files = self.read_recent_files()

        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")

        if new_recent_file in recent_files:
            recent_files.remove(new_recent_file)
        recent_files.insert(0, new_recent_file)
        recent_files = recent_files[:5]
        for i, recent_file in enumerate(recent_files):
            qt_settings.setValue(u'recent_files/file_%s' % i, recent_file)

    def read_settings(self):
        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")
        qt_settings.beginGroup('main')
        self.delta_norm = round(qt_settings.value('delta_norm', 0.001).toFloat()[0], 7)
        self.max_shod = qt_settings.value('max_shod', 2000).toInt()[0]
        self.round_n = qt_settings.value('round_n', 3).toInt()[0]
        self.file_point = qt_settings.value('file_point', ',').toString()
        qt_settings.endGroup()

    def write_settings(self):
        qt_settings = QtCore.QSettings(QtCore.QSettings.IniFormat, QtCore.QSettings.UserScope,
                                       "NodeVolt", "node_volt")
        qt_settings.beginGroup('main')
        qt_settings.setValue('delta_norm', self.delta_norm)
        qt_settings.setValue('max_shod', self.max_shod)
        qt_settings.setValue('round_n', self.round_n)
        qt_settings.setValue('file_point', self.file_point)
        qt_settings.endGroup()


class StartWindow(QtGui.QDialog, start_window_ui.Ui_dlg_start_window):
    """ Класс, который описывает начальное окно программы, содержащее кнопки "Создать схему" и "Открыть схему".
    """

    def __init__(self, parent=None):
        super(StartWindow, self).__init__(parent)
        self.setupUi(self)
        self.setWindowFlags(QtCore.Qt.FramelessWindowHint)
        self.connect_actions()
        # Создание стиля.
        self.create_style()

    def create_style(self):
        # Создание стиля.
        self.setStyleSheet(style.START_STYLE_SHEET)
        self.btn_close.setStyleSheet(style.CLOSE_BTN_STYLE_SHEET)

    def main(self, settings_class=Settings()):
        recent_files = settings_class.read_recent_files()
        if recent_files:
            for recent_file in recent_files:
                self.list_recent_files.addItem(u'%s [%s]' % (os.path.basename(recent_file), recent_file))
        result = self.exec_()
        if result == 3:
            recent_file = unicode(self.list_recent_files.selectedItems()[0].text())
            recent_file = recent_file.split(' [')[1].replace(']', '')
            return 3, recent_file
        else:
            return result, u''

    def connect_actions(self):
        self.btn_new_scheme.clicked.connect(self.new_scheme)
        self.btn_open_scheme.clicked.connect(self.open_scheme)
        self.list_recent_files.clicked.connect(self.open_recent_file)
        self.btn_close.clicked.connect(lambda: self.done(0))

    def new_scheme(self):
        self.done(1)

    def open_scheme(self):
        self.done(2)

    def open_recent_file(self):
        self.done(3)

    def keyPressEvent(self, event):
        event_key = event.key()
        if event_key == QtCore.Qt.Key_Escape:
            self.done(0)


class ProgressWindow(QtGui.QDialog, progress_window_ui.Ui_progress_window):
    """ Класс, который описывает окно, появляющееся при нажатии кнопки "Расчет" или "Оптимизация".

    В этом окне отображаются предупрежения, ошибки и другая информация.
    """

    def __init__(self, parent=None):
        super(ProgressWindow, self).__init__(parent)
        self.setupUi(self)
        self.connect_actions()
        # Создание стиля.
        self.create_style()

    def create_style(self):
        # Создание стиля.
        self.setStyleSheet(style.MAIN_STYLE_SHEET)

    def main(self):
        return self.exec_()

    def connect_actions(self):
        self.btn_close.clicked.connect(self.done)


class ProgSettings(QtGui.QDialog, settings_ui.Ui_dlg_settings):
    """ Класс, который описывает окно настроек программы."""

    def __init__(self, parent=None):
        super(ProgSettings, self).__init__(parent)
        self.setupUi(self)
        # Создание стиля.
        self.create_style()

    def create_style(self):
        # Создание стиля.
        self.setStyleSheet(style.MAIN_STYLE_SHEET)

    def main(self, settings_obj, file_points, round_ns, delta_norms):
        # Список разелителей десятичных дробей.
        self.combobox_file_point.addItems(file_points)
        for i in range(len(file_points)):
            if settings_obj.file_point == file_points[i]:
                self.combobox_file_point.setCurrentIndex(i)
        # Список показателей округления результатов расчета.
        self.combobox_round_n.addItems(round_ns)
        for i in range(len(round_ns)):
            if settings_obj.round_n == int(round_ns[i]):
                self.combobox_round_n.setCurrentIndex(i)
        # Список показателей сходимости.
        self.combobox_delta_norm.addItems(delta_norms)
        for i in range(len(delta_norms)):
            if settings_obj.delta_norm == float(delta_norms[i]):
                self.combobox_delta_norm.setCurrentIndex(i)
        # Максимальное количество итераций.
        self.lineedit_max_shod.setText(str(settings_obj.max_shod))
        # Список языков.
        self.combobox_language.addItems(language.LANGUAGES)
        for i in range(len(language.LANGUAGES)):
            if settings_obj.language == language.LANGUAGES[i]:
                self.combobox_language.setCurrentIndex(i)

        self.exec_()


class SchemeSettings(QtGui.QDialog, scheme_settings_ui.Ui_dlg_scheme_settings):
    """ Класс, который описывает окно настроек схемы."""

    def __init__(self, parent=None, cancel_enable=True):
        super(SchemeSettings, self).__init__(parent)
        self.setupUi(self)
        # Создание стиля.
        self.create_style()
        self.combobox_unit_reactive_power.setEnabled(False)
        self.combobox_unit_full_power.setEnabled(False)
        if not cancel_enable:
            self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Ok)

    def connect_actions(self):
        self.combobox_unit_active_power.currentIndexChanged.connect(self.unit_active_power_change)

    def unit_active_power_change(self):
        # Функция, вызываемая при изменении единицы измерения активной мощности.
        # Изменяются также единицы измерение реактивной и полной мощностей.
        properties = language.Properties()
        units = {unit: k for k, unit in properties.unit_active_power.iteritems()}
        unit = unicode(self.combobox_unit_active_power.currentText())
        if unit in units:
            unit_k = units[unit]
            if unit_k in properties.unit_reactive_power:
                # Изменение единицы измерения реактивной мощности.
                units_reactive_power = properties.unit_reactive_power.values()
                unit_reactive_power = properties.unit_reactive_power[unit_k]
                i = units_reactive_power.index(unit_reactive_power)
                self.combobox_unit_reactive_power.clear()
                self.combobox_unit_reactive_power.addItems(units_reactive_power)
                self.combobox_unit_reactive_power.setCurrentIndex(i)
            if unit_k in properties.unit_full_power:
                # Изменение единицы измерения полной мощности.
                units_full_power = properties.unit_full_power.values()
                unit_full_power = properties.unit_full_power[unit_k]
                i = units_full_power.index(unit_full_power)
                self.combobox_unit_full_power.clear()
                self.combobox_unit_full_power.addItems(units_full_power)
                self.combobox_unit_full_power.setCurrentIndex(i)

    def create_style(self):
        # Создание стиля.
        self.setStyleSheet(style.MAIN_STYLE_SHEET)

    def main(self, node_scheme):
        # Настройка выбора типа схемы.
        properties = language.Properties()
        s_types = [properties.scheme_types[i][0] for i in range(len(properties.scheme_types))]
        self.combobox_scheme_type.addItems(s_types)
        for i in range(len(properties.scheme_types)):
            if node_scheme.scheme_type == properties.scheme_types[i][1]:
                self.combobox_scheme_type.setCurrentIndex(i)
                break
        # Базисное напряжение.
        self.lineedit_base_u.setText(unicode(node_scheme.base_u_input))
        # Единицы измерения.
        units = properties.unit_voltage.values()
        self.combobox_unit_voltage.addItems(units)
        if node_scheme.unit_k_voltage in properties.unit_voltage:
            unit = properties.unit_voltage[node_scheme.unit_k_voltage]
            i = units.index(unit)
            self.combobox_unit_voltage.setCurrentIndex(i)

        units = properties.unit_current.values()
        self.combobox_unit_current.addItems(units)
        if node_scheme.unit_k_current in properties.unit_current:
            unit = properties.unit_current[node_scheme.unit_k_current]
            i = units.index(unit)
            self.combobox_unit_current.setCurrentIndex(i)

        units = properties.unit_active_power.values()
        self.combobox_unit_active_power.addItems(units)
        if node_scheme.unit_k_active_power in properties.unit_active_power:
            unit = properties.unit_active_power[node_scheme.unit_k_active_power]
            i = units.index(unit)
            self.combobox_unit_active_power.setCurrentIndex(i)

        units = properties.unit_reactive_power.values()
        self.combobox_unit_reactive_power.addItems(units)
        if node_scheme.unit_k_reactive_power in properties.unit_reactive_power:
            unit = properties.unit_reactive_power[node_scheme.unit_k_reactive_power]
            i = units.index(unit)
            self.combobox_unit_reactive_power.setCurrentIndex(i)

        units = properties.unit_full_power.values()
        self.combobox_unit_full_power.addItems(units)
        if node_scheme.unit_k_full_power in properties.unit_full_power:
            unit = properties.unit_full_power[node_scheme.unit_k_full_power]
            i = units.index(unit)
            self.combobox_unit_full_power.setCurrentIndex(i)
        # Настройки расчета.
        self.checkbox_use_temperature_coefficient.setChecked(node_scheme.use_temperature_coefficient)
        # Настройки вывода результатов расчета.
        self.checkbox_print_current.setChecked(node_scheme.print_vetvi_current)
        self.checkbox_print_power.setChecked(node_scheme.print_power)
        self.checkbox_print_vetvi_power.setChecked(node_scheme.print_vetvi_power)
        self.checkbox_print_vetvi_voltage.setChecked(node_scheme.print_vetvi_voltage)
        self.checkbox_print_usli_voltage.setChecked(node_scheme.print_usli_voltage)

        self.connect_actions()
        self.exec_()


class NodeVolt(QtGui.QMainWindow, node_volt_ui.Ui_MainWindow):
    """ Класс основного окна программы."""
    
    def __init__(self, settings_obj=Settings(), parent=None):
        super(NodeVolt, self).__init__(parent)
        errors = language.Errors()
        self.setupUi(self)
        self.settings = settings_obj
        # Буфер обмена элементов и узлов.
        self.vetvi_copied = []
        self.usli_copied = []
        # Модели таблиц элементов и узлов.
        self.scheme_vetvi_model = None
        self.scheme_usli_model = None
        # Модель таблицы результатов расчета.
        self.results_table_model = None
        # Модели таблиц вариантов схемы, полученных при оптимизации.
        self.optimization_table_normal_model = None
        self.optimization_table_overdrive_model = None
        self.optimization_table_bad_model = None
        # Объект схемы.
        self.node_scheme = nv_plus.NodeScheme(self.settings)
        # Считывание файла с новыми элементами.
        new_elements_obj = gui_classes.NewElements()
        result = new_elements_obj.read_elements_file(module_path())
        if result[0]:
            message(parent=self, mes_type='warning', title=errors.titles['error open new elements file'],
                    mes_list=result[0])
        # Создание таблиц добавления новых элементов.
        self.simple_elements_list = NewElementsTable('simple_elements', new_elements_obj, self)
        self.connections_list = NewElementsTable('connections', new_elements_obj, self)
        self.generators_list = NewElementsTable('generators', new_elements_obj, self)
        self.load_list = NewElementsTable('load', new_elements_obj, self)
        self.transformers_list = NewElementsTable('transformers', new_elements_obj, self)
        self.switchers_list = NewElementsTable('switchers', new_elements_obj, self)
        # Модель графического отображения схемы.
        self.scheme_scene = graphics.SchemeView(self.tree_properties, self.statusbar, new_elements_obj,
                                                module_path(), self, self.graphics_scheme)
        # Создание сочетаний клавиш.
        self.shortcut_open = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+O"), self)
        self.shortcut_save = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+S"), self)
        self.shortcut_copy = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+C"), self)
        self.shortcut_past = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+V"), self)
        self.shortcut_select_all = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+A"), self)
        self.connect_actions()

    def connect_actions(self):
        """ Соединение действий нажатия на кнопки программы или сочетания клавиш с методами."""
        # Нажатие кнопки "Создать".
        self.btn_create_new_scheme.clicked.connect(self.create_scheme)
        # Нажатие кнопки "Открыть".
        self.btn_open_scheme.clicked.connect(self.open_scheme)
        # Нажатие кнопки "Сохранить" и "Сохранить как".
        self.btn_save_scheme.clicked.connect(self.save_scheme)
        self.btn_save_as_scheme.clicked.connect(self.save_as_scheme)
        # Нажатие кнопки "Расчет".
        self.btn_calculate.clicked.connect(self.calculate_scheme)
        # Нажатие кнопки "Оптимизация".
        self.btn_optimization.clicked.connect(self.optimize_scheme)
        # Нажатие кнопки "Экспорт CSV".
        self.btn_export_csv.clicked.connect(self.export_results_csv)
        # Нажатие кнопки "Удалить".
        self.btn_remove_element.clicked.connect(self.remove_element)
        # Нажатие кнопки "Добавить узел".
        self.btn_add_usel.clicked.connect(self.add_new_usel)
        # Нажатие кнопки "Копировать".
        self.btn_copy.clicked.connect(self.copy)
        # Нажатие кнопки "Вставить".
        self.btn_past.clicked.connect(self.past)
        # Нажатие кнопки "Настройки схемы".
        self.btn_scheme_settings.clicked.connect(lambda: self.scheme_settings(cancel_enable=True))
        # Нажатие кнопки "Настройки программы".
        self.btn_settings.clicked.connect(self.prog_settings)
        # Нажатие кнопки перемещения элемента или узла вверх по списку.
        self.btn_move_up.clicked.connect(self.move_up)
        # Нажатие кнопки перемещения элемента или узла вниз по списку.
        self.btn_move_down.clicked.connect(self.move_down)
        # Нажатие кнопки поворота элемента или узла влево.
        self.btn_rotate_left.clicked.connect(self.rotate_left)
        # Нажатие кнопки поворота элемента или узла вправо.
        self.btn_rotate_right.clicked.connect(self.rotate_right)
        # Изменение вкладки "Схема", "Элементы", "Узлы", "Результаты".
        self.tab_main.currentChanged.connect(self.main_tab_change)
        # Нажатие кнопок добавления новых элементов.
        self.btn_add_simple_element.clicked.connect(self.simple_elements_list.show_list_in_cursor)
        self.btn_add_connection.clicked.connect(self.connections_list.show_list_in_cursor)
        self.btn_add_switcher.clicked.connect(self.switchers_list.show_list_in_cursor)
        self.btn_add_generator.clicked.connect(self.generators_list.show_list_in_cursor)
        self.btn_add_load.clicked.connect(self.load_list.show_list_in_cursor)
        self.btn_add_transformer.clicked.connect(self.transformers_list.show_list_in_cursor)
        # Сочетание клавиш Ctrl+O.
        self.shortcut_open.activated.connect(self.open_scheme)
        # Сочетание клавиш Ctrl+S.
        self.shortcut_save.activated.connect(self.save_scheme)
        # Сочетание клавиш Ctrl+C.
        self.shortcut_copy.activated.connect(self.copy)
        # Сочетание клавиш Ctrl+V.
        self.shortcut_past.activated.connect(self.past)
        # Сочетание клавиш Ctrl+A.
        self.shortcut_select_all.activated.connect(self.select_all_in_shceme)
        # Изменение размеров столбцов дерева параметров.
        self.tree_properties.header().sectionResized.connect(self.tree_properties_resize)
        # Выбор схемы во вкладке "Результаты оптимизации".
        self.table_normal_schemes.clicked.connect(lambda: self.select_optimized_scheme(normal=True, bad=False))
        self.table_overdrive_schemes.clicked.connect(lambda: self.select_optimized_scheme(normal=False, bad=False))
        self.table_bad_schemes.clicked.connect(lambda: self.select_optimized_scheme(normal=False, bad=True))

    def tree_properties_resize(self):
        """Метод, вызываемый при изменении размеров столбцов дерева параметров."""
        if self.tree_properties.model() is not None:
            column_0_width = self.tree_properties.columnWidth(0)
            if self.settings.tree_properties_column_widths[0] != column_0_width:
                column_1_width = self.tree_properties.columnWidth(1)
                if self.settings.tree_properties_column_widths[1] != column_1_width:
                    if column_0_width != 100 and column_1_width != 100:
                        self.settings.tree_properties_column_widths[0] = column_0_width
                        self.settings.tree_properties_column_widths[1] = column_1_width
                        sum_col_width = column_0_width + column_1_width
                        widget_width = self.tree_properties.size().width()
                        if sum_col_width > widget_width:
                            self.settings.tree_properties_column_widths[1] = widget_width - column_0_width - 15
                            self.tree_properties.setColumnWidth(1, self.settings.tree_properties_column_widths[1])

    def hide_all_new_elements_lists(self):
        """ Метод, который скрывает все списки добавления новых элементов."""
        self.simple_elements_list.hide_list()
        self.connections_list.hide_list()
        self.generators_list.hide_list()
        self.load_list.hide_list()
        self.transformers_list.hide_list()
        self.switchers_list.hide_list()

    def select_optimized_scheme(self, normal=True, bad=False):
        """ Метод, вызываемый при выборе варианта схемы в таблицах результатов оптимизации."""
        if normal and not bad and self.optimization_table_normal_model is not None:
            selection = self.table_normal_schemes.selectedIndexes()
            row = selection[0].row()
            list_node_schemes = self.optimization_table_normal_model.list_node_schemes
            if row < len(list_node_schemes):
                node_scheme = list_node_schemes[row][1]
                if isinstance(node_scheme, nv_plus.NodeScheme):
                    self.node_scheme = node_scheme
                    self.table_overdrive_schemes.clearSelection()
                    self.table_bad_schemes.clearSelection()
        elif not normal and not bad and self.optimization_table_overdrive_model is not None:
            selection = self.table_overdrive_schemes.selectedIndexes()
            row = selection[0].row()
            list_node_schemes = self.optimization_table_overdrive_model.list_node_schemes
            if row < len(list_node_schemes):
                node_scheme = list_node_schemes[row][1]
                if isinstance(node_scheme, nv_plus.NodeScheme):
                    self.node_scheme = node_scheme
                    self.table_normal_schemes.clearSelection()
                    self.table_bad_schemes.clearSelection()
        elif not normal and bad and self.optimization_table_bad_model is not None:
            selection = self.table_bad_schemes.selectedIndexes()
            row = selection[0].row()
            list_node_schemes = self.optimization_table_bad_model.list_node_schemes
            if row < len(list_node_schemes):
                node_scheme = list_node_schemes[row][1]
                if isinstance(node_scheme, nv_plus.NodeScheme):
                    self.node_scheme = node_scheme
                    self.table_normal_schemes.clearSelection()
                    self.table_overdrive_schemes.clearSelection()

    def key_esc(self):
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            if self.scheme_scene is not None:
                self.scheme_scene.key_esc()
        self.hide_all_new_elements_lists()

    def select_all_in_shceme(self):
        """ Метод, вызываемы при нажатии сочетания клавиш Ctrl+A, который выбирает все элементы на схеме."""
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            if self.scheme_scene is not None:
                self.scheme_scene.select_all()

    def main_tab_change(self):
        """ Метод, вызываемый при изменении текущей вкладки "Схема", "Элементы", "Узлы" и др."""
        # Обновление моделей таблиц элементов и узлов.
        self.scheme_vetvi_model = models.VetviModel(self.node_scheme, self.table_vetvi)
        self.scheme_usli_model = models.UsliModel(self.node_scheme, self.table_usli)
        # Скрыть все списки добавления новых элементов.
        self.hide_all_new_elements_lists()

        def set_add_elements_visible(visible=True):
            """ Показать или скрыть кнопки добавления новых элементов или узлов."""
            self.btn_add_usel.setVisible(visible)
            self.btn_add_connection.setVisible(visible)
            self.btn_add_generator.setVisible(visible)
            self.btn_add_load.setVisible(visible)
            self.btn_add_simple_element.setVisible(visible)
            self.btn_add_switcher.setVisible(visible)
            self.btn_add_transformer.setVisible(visible)

        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            # Вкладка графического вида схемы.
            # Кнопки перемещения элементов - скрыть.
            self.btn_move_down.setVisible(False)
            self.btn_move_up.setVisible(False)
            # Кнопки поворота - показать.
            self.btn_rotate_right.setVisible(True)
            self.btn_rotate_left.setVisible(True)
            # Кнопки добавления элементов и узлов - показать.
            set_add_elements_visible(True)
            # Кнопки копирования, удаления и вставки - показать.
            self.btn_copy.setVisible(True)
            self.btn_past.setVisible(True)
            self.btn_remove_element.setVisible(True)
            # Создание графической сцены.
            self.scheme_scene.set_scheme_model(self.node_scheme)
            self.graphics_scheme.setScene(self.scheme_scene)
            # Очистка дерева параметров.
            self.tree_properties.setModel(None)
        elif self.tab_main.currentIndex() == TAB_BRANCHES['vetvi']:
            # Вкладка списка элементов.
            input_parameters = prog_settings.VetvInputParameters()
            # Кнопки перемещения - показать.
            self.btn_move_down.setVisible(True)
            self.btn_move_up.setVisible(True)
            # Кнопки поворота - скрыть.
            self.btn_rotate_right.setVisible(False)
            self.btn_rotate_left.setVisible(False)
            # Кнопки удаления и добавления элементов и узлов - скрыть.
            set_add_elements_visible(False)
            # Кнопки копирования, удаления и вставки - показать.
            self.btn_copy.setVisible(True)
            self.btn_past.setVisible(True)
            self.btn_remove_element.setVisible(True)
            # Обновление модели таблицы элементов.
            self.table_vetvi.setModel(self.scheme_vetvi_model)
            # Установка настроек таблицы элементов.
            for i in range(len(input_parameters.widths)):
                self.table_vetvi.setColumnWidth(i, input_parameters.widths[i])
        elif self.tab_main.currentIndex() == TAB_BRANCHES['usli']:
            # Вкладка списка узлов.
            # Кнопки перемещения - показать.
            self.btn_move_down.setVisible(True)
            self.btn_move_up.setVisible(True)
            # Кнопки поворота - скрыть.
            self.btn_rotate_right.setVisible(False)
            self.btn_rotate_left.setVisible(False)
            # Кнопки удаления и добавления элементов и узлов - скрыть.
            set_add_elements_visible(False)
            # Кнопки копирования, удаления и вставки - показать.
            self.btn_copy.setVisible(True)
            self.btn_past.setVisible(True)
            self.btn_remove_element.setVisible(True)
            # Обновление модели таблицы узлов.
            self.table_usli.setModel(self.scheme_usli_model)
            # Установка настроек таблицы узлов
            usel_input_parameters = prog_settings.UselInputParameters()
            for i in range(len(usel_input_parameters.widths)):
                self.table_usli.setColumnWidth(i, usel_input_parameters.widths[i])
        elif self.tab_main.currentIndex() == TAB_BRANCHES['results']:
            # Вкладка результатов расчета.
            # Кнопки перемещения - скрыть.
            self.btn_move_down.setVisible(False)
            self.btn_move_up.setVisible(False)
            # Кнопки поворота - скрыть.
            self.btn_rotate_right.setVisible(False)
            self.btn_rotate_left.setVisible(False)
            # Кнопки удаления и добавления элементов и узлов - скрыть.
            set_add_elements_visible(False)
            # Кнопки копирования, удаления и вставки - скрыть.
            self.btn_copy.setVisible(False)
            self.btn_past.setVisible(False)
            self.btn_remove_element.setVisible(False)
            if self.node_scheme.vetvi.children_list():
                # Отображение результатов предыдущих расчетов, если они есть.
                results_table = nv_plus.results_data(node_scheme=self.node_scheme,
                                                     file_name='', round_n=self.settings.round_n,
                                                     file_point=self.settings.file_point)[3]
                self.results_table_model = models.TableResultsModel(results_table.rows, self.table_results)
                self.table_results.setModel(self.results_table_model)
                # Изменение ширины столбцов.
                index = self.results_table_model.index(0, 0)
                self.table_results.setCurrentIndex(index)
                self.table_results.setColumnWidth(0, 260)
                for i in range(1, self.results_table_model.columnCount(None)):
                    self.table_results.setColumnWidth(i, 90)
            else:
                self.table_results.setModel(None)
        elif self.tab_main.currentIndex() == TAB_BRANCHES['optimization_results']:
            # Вкладка результатов оптимизации.
            # Кнопки перемещения - скрыть.
            self.btn_move_down.setVisible(False)
            self.btn_move_up.setVisible(False)
            # Кнопки поворота - скрыть.
            self.btn_rotate_right.setVisible(False)
            self.btn_rotate_left.setVisible(False)
            # Кнопки удаления и добавления элементов и узлов - скрыть.
            set_add_elements_visible(False)
            # Кнопки копирования, удаления и вставки - скрыть.
            self.btn_copy.setVisible(False)
            self.btn_past.setVisible(False)
            self.btn_remove_element.setVisible(False)

    def keyPressEvent(self, event):
        event_key = event.key()
        if event_key == QtCore.Qt.Key_Delete:
            self.remove_element()
        elif event_key == QtCore.Qt.Key_Escape:
            self.key_esc()

    def past(self):
        """ Метод, вызываемый при нажатии кнопки "Вставить" или сочетания клавиш Ctrl+V."""
        if not self.node_scheme:
            return
        words = language.Other()
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            # Вкладка "Схема".
            if self.vetvi_copied:
                # Вставка скопированных элементов.
                for vetv in self.vetvi_copied:
                    vetv = cPickle.loads(vetv)
                    vetv_name = u'%s_%s' % (vetv.name(), words.lang_element_node_copy)
                    for j in range(1, 101):
                        if vetv_name not in self.node_scheme.vetvi.children_list():
                            vetv.set_name(vetv_name)
                            position = self.node_scheme.vetvi.child_count()
                            self.node_scheme.vetvi.insert_child(vetv, position)
                            self.scheme_scene.add_new_element(vetv)
                            break
                        else:
                            vetv_name = u'%s_%s_%s' % (vetv.name(), words.lang_element_node_copy, j)
                # Очистка списка скопированных элементов.
                self.vetvi_copied = []
            if self.usli_copied:
                # Вставка скопированных узлов.
                for usel in self.usli_copied:
                    usel = cPickle.loads(usel)
                    usel_name = u'%s_%s' % (usel.name(), words.lang_element_node_copy)
                    for j in range(1, 101):
                        if usel_name not in self.node_scheme.usli.children_list():
                            usel.set_name(usel_name)
                            position = self.node_scheme.usli.child_count()
                            self.node_scheme.usli.insert_child(usel, position)
                            self.scheme_scene.add_new_usel(usel)
                            break
                        else:
                            usel_name = u'%s_%s_%s' % (usel.name(), words.lang_element_node_copy, j)
                # Очистка списка скопированных узлов.
                self.usli_copied = []
        if self.tab_main.currentIndex() == TAB_BRANCHES['vetvi'] and self.vetvi_copied:
            # Вкладка "Элементы".
            position = self.table_vetvi.currentIndex().row() + 1
            if not position:
                position = self.node_scheme.vetvi.child_count() + 1
            i = 0
            for vetv in self.vetvi_copied:
                vetv = cPickle.loads(vetv)
                vetv_name = u'%s_%s' % (vetv.name(), words.lang_element_node_copy)
                for j in range(1, 101):
                    if vetv_name not in self.node_scheme.vetvi.children_list():
                        vetv.set_name(vetv_name)
                        self.node_scheme.vetvi.insert_child(vetv, position + i)
                        break
                    else:
                        vetv_name = u'%s_%s_%s' % (vetv.name(), words.lang_element_node_copy, j)
                i += 1
            # Очистка списка скопированных элементов.
            self.vetvi_copied = []
            # Создание модели схемы.
            self.scheme_vetvi_model = models.VetviModel(self.node_scheme, self.table_vetvi)
            self.table_vetvi.setModel(self.scheme_vetvi_model)
        if self.tab_main.currentIndex() == TAB_BRANCHES['usli'] and self.usli_copied:
            # Вкладка "Узлы".
            position = self.table_usli.currentIndex().row() + 1
            if not position:
                position = self.node_scheme.usli.child_count() + 1
            i = 0
            for usel in self.usli_copied:
                usel = cPickle.loads(usel)
                usel_name = u'%s_%s' % (usel.name(), words.lang_element_node_copy)
                for j in range(1, 101):
                    if usel_name not in self.node_scheme.usli.children_list():
                        usel.set_name(usel_name)
                        self.node_scheme.usli.insert_child(usel, position + i)
                        break
                    else:
                        usel_name = u'%s_%s_%s' % (usel.name(), words.lang_element_node_copy, j)
                i += 1
            # Очистка списка скопированных узлов.
            self.usli_copied = []
            # Создание модели схемы.
            self.scheme_usli_model = models.UsliModel(self.node_scheme, self.table_usli)
            self.table_usli.setModel(self.scheme_usli_model)

    def copy(self):
        """ Метод, вызываемый при нажатии кнопки "Копировать" или сочетания клавиш Ctrl+C."""
        if not self.node_scheme:
            return

        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            # Вкладка "Схема".
            selected_items = self.scheme_scene.selectedItems()
            if selected_items:
                # Очистка списков скопированных элементов и узлов.
                self.usli_copied = []
                self.vetvi_copied = []
                # Заполнение списков скопированных элементов и узлов.
                for selected_item in selected_items:
                    if selected_item.element.type() == 'usel':
                        usel_obj = selected_item.element.self_copy()
                        usel_copy = cPickle.dumps(usel_obj)
                        self.usli_copied.append(usel_copy)
                    elif selected_item.element.type() == 'vetv':
                        vetv_obj = selected_item.element.self_copy()
                        input_parameters = vetv_obj.input_parameters()
                        input_parameters.begin().set_value('')
                        input_parameters.end().set_value('')
                        input_parameters.end_2().set_value('')
                        vetv_copy = cPickle.dumps(vetv_obj)
                        self.vetvi_copied.append(vetv_copy)
        if self.tab_main.currentIndex() == TAB_BRANCHES['vetvi']:
            # Вкладка "Элементы".
            # Определение строк выделенных ячеек.
            indexes = self.table_vetvi.selectionModel().selectedIndexes()
            if indexes:
                # Очистка списков скопированных элементов и узлов.
                self.usli_copied = []
                self.vetvi_copied = []
                rows = sorted(list(set([index.row() for index in indexes])))
                # Заполнение списка скопированных элементов.
                for row in rows:
                    vetv_copy = cPickle.dumps(self.node_scheme.vetvi.child(row).self_copy())
                    self.vetvi_copied.append(vetv_copy)
        if self.tab_main.currentIndex() == TAB_BRANCHES['usli']:
            # Вкладка "Узлы".
            #  Определение строк выделенных ячеек.
            indexes = self.table_usli.selectionModel().selectedIndexes()
            if indexes:
                # Очистка списков скопированных элементов и узлов.
                self.usli_copied = []
                self.vetvi_copied = []
                rows = sorted(list(set([index.row() for index in indexes])))
                # Заполнение списка скопированных узлов.
                for row in rows:
                    usel_copy = cPickle.dumps(self.node_scheme.usli.child(row).self_copy())
                    self.usli_copied.append(usel_copy)

    def move_up(self):
        """ Метод, вызываемый при нажатии кнопки "Переместить вверх".

        Перемещает элемент или узел вверх по списку. Работает только во вкладках "Элементы" и "Узлы".
        """
        if not self.node_scheme:
            return

        if self.tab_main.currentIndex() == TAB_BRANCHES['vetvi']:
            # Вкладка "Элементы".
            # Определение строк выделенных ячеек.
            indexes = self.table_vetvi.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                # Если в выделении нет первой строки.
                if rows[0] != 0:
                    for row in rows:
                        vetv = self.node_scheme.vetvi.child(row)
                        self.node_scheme.vetvi.remove_child(row)
                        self.node_scheme.vetvi.insert_child(vetv, row-1)
                    # Обновление модели таблицы элементов.
                    self.scheme_vetvi_model = models.VetviModel(self.node_scheme, self.table_vetvi)
                    self.table_vetvi.setModel(self.scheme_vetvi_model)
                    index = self.scheme_vetvi_model.index(rows[0]-1, 0)
                    self.table_vetvi.setCurrentIndex(index)
        elif self.tab_main.currentIndex() == TAB_BRANCHES['usli']:
            # Вкладка "Узлы".
            # Определение строк выделенных ячеек.
            indexes = self.table_usli.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                # Если в выделении нет первой строки.
                if rows[0] != 0:
                    for row in rows:
                        usel = self.node_scheme.usli.child(row)
                        self.node_scheme.usli.remove_child(row)
                        self.node_scheme.usli.insert_child(usel, row-1)
                    # Обновление модели таблицы узлов.
                    self.scheme_usli_model = models.UsliModel(self.node_scheme, self.table_usli)
                    self.table_usli.setModel(self.scheme_usli_model)
                    index = self.scheme_usli_model.index(rows[0]-1, 0)
                    self.table_usli.setCurrentIndex(index)

    def move_down(self):
        """ Метод, вызываемый при нажатии кнопки "Переместить вниз".

        Перемещает элемент или узел вниз по списку. Работает только во вкладках "Элементы" и "Узлы".
        """
        if not self.node_scheme:
            return

        if self.tab_main.currentIndex() == TAB_BRANCHES['vetvi']:
            # Вкладка "Элементы".
            # Определение строк выделенных ячеек.
            indexes = self.table_vetvi.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                rows.reverse()
                # Если в выделении нет последней строки.
                if rows[0] != self.node_scheme.vetvi.child_count()-1:
                    for row in rows:
                        vetv = self.node_scheme.vetvi.child(row)
                        self.node_scheme.vetvi.remove_child(row)
                        self.node_scheme.vetvi.insert_child(vetv, row+1)
                    # Обновление модели таблицы элементов.
                    self.scheme_vetvi_model = models.VetviModel(self.node_scheme, self.table_vetvi)
                    self.table_vetvi.setModel(self.scheme_vetvi_model)
                    index = self.scheme_vetvi_model.index(rows[0]+1, 0)
                    self.table_vetvi.setCurrentIndex(index)
        elif self.tab_main.currentIndex() == TAB_BRANCHES['usli']:
            # Вкладка "Узлы".
            # Определение строк выделенных ячеек.
            indexes = self.table_usli.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                rows.reverse()
                # Если в выделении нет последней строки.
                if rows[0] != self.node_scheme.usli.child_count()-1:
                    for row in rows:
                        usel = self.node_scheme.usli.child(row)
                        self.node_scheme.usli.remove_child(row)
                        self.node_scheme.usli.insert_child(usel, row+1)
                    # Обновление модели таблицы узлов.
                    self.scheme_usli_model = models.UsliModel(self.node_scheme, self.table_usli)
                    self.table_usli.setModel(self.scheme_usli_model)
                    index = self.scheme_usli_model.index(rows[0]+1, 0)
                    self.table_usli.setCurrentIndex(index)

    def rotate(self, direction=1):
        """ Поворот элемента или узла.

        direction=1 - влево, direction=-1 - вправо.
        """
        if not self.node_scheme:
            return
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            selected_items = self.scheme_scene.selectedItems()
            if selected_items:
                # Проход по списку выделенных элементов.
                for selected_item in selected_items:
                    rotation = selected_item.element.gui_parameters().rotation()
                    new_rotation_value = rotation.value() - 45. * direction
                    if new_rotation_value > 180:
                        new_rotation_value = (new_rotation_value - 180. * (new_rotation_value // 180.)) - 180.
                    elif new_rotation_value < -180:
                        new_rotation_value = (new_rotation_value - 180. * (new_rotation_value // 180.))
                    rotation.set_value(new_rotation_value)
                    selected_item.update_group()

    def rotate_left(self):
        """ Метод, вызываемый при нажатии кнопки "Повернуть влево".

        Работает только на вкладке "Схема".
        """
        self.rotate(1)

    def rotate_right(self):
        """ Метод, вызываемый при нажатии кнопки "Повернуть вправо".

        Работает только на вкладке "Схема".
        """
        self.rotate(-1)

    def create_scheme(self, ask_save=True):
        """ Метод, вызываемый при нажатии кнопки "Создать".

        Создает новую схему.
        """
        messages = language.Messages()
        # Если аргумент ask_save=True, то вызывается диалоговое окно с вопросом о необходимости сохранения
        # открытой схемы.
        if ask_save:
            result = QtGui.QMessageBox.question(self, messages.messages['save scheme'],
                                                messages.messages['save scheme?'],
                                                QtGui.QMessageBox.Save | QtGui.QMessageBox.No, QtGui.QMessageBox.No)
            if result == QtGui.QMessageBox.Save:
                self.save_scheme()
        # Создание объекта схемы.
        self.node_scheme = nv_plus.NodeScheme(self.settings)
        # Изменение заголовка окна.
        self.setWindowTitle(u'Node Volt - %s' % _translate('main window title', 'New scheme', None))
        # Открытие диалога "Настройка схемы".
        self.scheme_settings(cancel_enable=False)
        # Скрытие кнопки оптимизации схемы и вкладки результатов оптимизации.
        self.optimization_on_off(False)
        self.main_tab_change()

    def remove_element(self):
        """ Метод, вызываемый при нажатии кнопки "Удалить".

        Работает только на вкладке "Схема".
        Удаляет элемент или узел.
        """
        if not self.node_scheme and not self.scheme_vetvi_model:
            return
        # Удаление элемента из списка элементов.
        if self.tab_main.currentIndex() == TAB_BRANCHES['vetvi']:
            # Определение строк выделенных ячеек.
            indexes = self.table_vetvi.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                i = 0
                for row in rows:
                    self.scheme_vetvi_model.removeRow(position=(row-i))
                    i += 1
        # Удаление узла из списка узлов.
        if self.tab_main.currentIndex() == TAB_BRANCHES['usli']:
            # Определение строк выделенных ячеек.
            indexes = self.table_usli.selectionModel().selectedIndexes()
            if indexes:
                rows = sorted(list(set([index.row() for index in indexes])))
                i = 0
                for row in rows:
                    self.scheme_usli_model.removeRow(position=(row-i))
                    i += 1
        # Удаление элемента со схемы.
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            self.scheme_scene.remove_selected_elements()

    def add_new_element(self, new_element_group_name, new_element_obj):
        """ Метод, вызываемый при нажатии одной из кнопок добавления нового элемента.

        Работает только на вкладке "Схема".
        """
        if not self.node_scheme and not self.scheme_vetvi_model:
            return
        if not isinstance(new_element_obj, classes.Parameters):
            return

        # Вкладка "Схема".
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            begin = new_element_obj.find_child('begin')
            end = new_element_obj.find_child('end')
            vetv_type = new_element_obj.find_child('vetv_type')
            view = new_element_obj.find_child('view')
            switcher_permit = new_element_obj.find_child('switcher_permit')

            if not vetv_type:
                return

            if vetv_type.value() == 'ground':
                element_name = 'g'
            else:
                element_name = ''

            self.node_scheme.insert_element(element_name=element_name)

            vetv = self.node_scheme.vetvi.child(self.node_scheme.vetvi.child_count()-1)
            input_parameters = vetv.input_parameters()
            input_parameters.vetv_type().set_value(vetv_type.value())
            if begin:
                input_parameters.begin().set_value(begin.value())
            if end:
                input_parameters.end().set_value(end.value())
            if view:
                vetv.gui_parameters().view().set_value(view.value())
            if switcher_permit:
                vetv.optimization_parameters().switcher_permit().set_value(switcher_permit.value())
            input_parameters.new_element_name().set_value(u'%s/%s' % (new_element_group_name, new_element_obj.name()))

            self.scheme_scene.add_new_element(vetv)

    def add_new_usel(self):
        """ Метод, вызываемый при нажатии кнопки добавления узла.

        Работает только на вкладке "Схема".
        """
        if not self.node_scheme and not self.scheme_usli_model:
            return
        # Вкладка "Схема".
        if self.tab_main.currentIndex() == TAB_BRANCHES['scheme']:
            self.node_scheme.insert_usel()
            usel = self.node_scheme.usli.child(self.node_scheme.usli.child_count()-1)
            self.scheme_scene.add_new_usel(usel)

    def scheme_settings(self, cancel_enable=True):
        """ Отображение диалогового окна настроек схемы.
        """
        if not self.node_scheme:
            return
        # Отображение диалога "Настройки схемы".
        scheme_settings_dialog = SchemeSettings(cancel_enable=cancel_enable)
        scheme_settings_dialog.main(self.node_scheme)
        # Если нажата кнопка "OK".
        if scheme_settings_dialog.result():
            properties = language.Properties()
            # Запись нового типа схемы.
            self.node_scheme.scheme_type = \
                properties.scheme_types[scheme_settings_dialog.combobox_scheme_type.currentIndex()][1]
            # Запись базисного напряжения.
            self.node_scheme.base_u_input = unicode(scheme_settings_dialog.lineedit_base_u.text())
            # Запись единицы измерения напряжения.
            self.node_scheme.unit_voltage = unicode(scheme_settings_dialog.combobox_unit_voltage.currentText())
            # Запись единицы измерения тока.
            self.node_scheme.unit_current = unicode(scheme_settings_dialog.combobox_unit_current.currentText())
            # Запись единицы измерения активной мощности.
            self.node_scheme.unit_active_power = \
                unicode(scheme_settings_dialog.combobox_unit_active_power.currentText())
            # Запись единицы измерения реактивной мощности.
            self.node_scheme.unit_reactive_power = \
                unicode(scheme_settings_dialog.combobox_unit_reactive_power.currentText())
            # Запись единицы измерения полной мощности.
            self.node_scheme.unit_full_power = unicode(scheme_settings_dialog.combobox_unit_full_power.currentText())
            self.node_scheme.read_units(True)
            # Запись необходимости вывода напряжений в узлах после расчета.
            self.node_scheme.print_usli_voltage = scheme_settings_dialog.checkbox_print_usli_voltage.isChecked()
            # Запись необходимости вывода напряжений на элементах после расчета.
            self.node_scheme.print_vetvi_voltage = scheme_settings_dialog.checkbox_print_vetvi_voltage.isChecked()
            # Запись необходимости вывода потоков мощности в элементах после расчета.
            self.node_scheme.print_vetvi_power = scheme_settings_dialog.checkbox_print_vetvi_power.isChecked()
            # Запись необходимости вывода токов после расчета.
            self.node_scheme.print_vetvi_current = scheme_settings_dialog.checkbox_print_current.isChecked()
            # Запись необходимости вывода мощностей приемников и источников после расчета.
            self.node_scheme.print_power = scheme_settings_dialog.checkbox_print_power.isChecked()
            # Запись необходимости использования температурного коэффициента в расчетах.
            self.node_scheme.use_temperature_coefficient = \
                scheme_settings_dialog.checkbox_use_temperature_coefficient.isChecked()
            # Обновление моделей схемы.
            self.main_tab_change()

    def prog_settings(self):
        """ Метод, вызываемый при нажатии кнопки "Настройка программы".

        Отображает диалоговое окно настроек программы.
        """
        errors = language.Errors()
        # Списки возможных значений настроек.
        file_points = [',', '.']
        round_ns = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10']
        delta_norms = ['1000.0', '100.0', '10.0', '1.0', '0.1', '0.01', '0.001', '0.0001', '0.00001', '0.000001']
        # Отображение диалога "Настройки программы".
        prog_settings_dialog = ProgSettings()
        prog_settings_dialog.main(self.settings, file_points, round_ns, delta_norms)
        # Если нажата кнопка "OK".
        if prog_settings_dialog.result():
            # Запись показателя сходимости.
            self.settings.delta_norm = float(delta_norms[prog_settings_dialog.combobox_delta_norm.currentIndex()])
            # Запись показателя округления.
            self.settings.round_n = int(round_ns[prog_settings_dialog.combobox_round_n.currentIndex()])
            # Запись разделителя десятичных дробей.
            self.settings.file_point = file_points[prog_settings_dialog.combobox_file_point.currentIndex()]
            # Запись максимального количества итераций.
            try:
                self.settings.max_shod = int(prog_settings_dialog.lineedit_max_shod.text())
            except ValueError:
                message(parent=self, mes_type='warning', title=errors.titles['error'],
                        mes_list=[errors.messages['invalid maximum number of iterations']])
            if self.settings.max_shod < 0:
                self.settings.max_shod = 1
                message(parent=self, mes_type='warning', title=errors.titles['error'],
                        mes_list=[errors.messages['invalid maximum number of iterations']])
            # Запись языка.
            self.settings.language = str(prog_settings_dialog.combobox_language.currentText())
            # Запись новых настроек в файл.
            self.settings.write_settings()
            self.settings.write_gui_settings()

    def open_scheme(self, scheme_name=u''):
        """ Открытие схемы из файла.
        """
        errors = language.Errors()
        messages = language.Messages()
        if not scheme_name:
            # открытие диалогового окна выбора файла.
            scheme_name = unicode(QtGui.QFileDialog.getOpenFileName(self, messages.messages['open scheme'],
                                                                    QtCore.QDir.homePath(),
                                                                    u"XNVS-%s (*.xnvs);;NVS-%s (*.nvs)" %
                                                                    (messages.messages['scheme file'],
                                                                     messages.messages['scheme file'])))
            if not scheme_name:
                return
        extension = os.path.splitext(scheme_name)[1]
        self.statusbar.showMessage(u'%s %s' % (messages.messages['opening a file'], scheme_name), 0)
        # Списки для записи ошибок, предупреждений и сообщений.
        errs = []
        warnings = []
        messages_1 = []
        # Создание объекта схемы.
        node_scheme_new = nv_plus.NodeScheme(self.settings)
        node_scheme_new.scheme_file_name = scheme_name
        if extension == '.xnvs':
            # Чтение XML-файла.
            result = node_scheme_new.read_scheme_file_xml()
        elif extension == '.nvs':
            # Чтение NVS-файла.
            result = node_scheme_new.read_scheme_file_nvs()
        else:
            result = node_scheme_new.read_scheme_file_xml()
        errs += result[0]
        warnings += result[1]
        messages_1 += result[2]
        # Проверка на наличие критических ошибок после чтения файла.
        # Если есть критические ошибки после чтения файла.
        if errs:
            node_scheme_new = None
            message(parent=self, mes_type='warning', title=errors.titles['open file fatal error'],
                    mes_list=errs)
            self.statusbar.clearMessage()
        # Если нет критических ошибок после чтения файла.
        if not errs and extension and node_scheme_new:
            # Если есть не критические ошибки после чтения файла.
            if warnings:
                message(parent=self, mes_type='warning', title=errors.titles['open file fatal error'],
                        mes_list=warnings+[errors.messages['can continue work']])
            self.node_scheme = nv_plus.deep_copy_scheme(node_scheme_new)
            self.setWindowTitle(u'Node Volt - %s [%s]' % (os.path.basename(self.node_scheme.scheme_file_name),
                                                          self.node_scheme.scheme_file_name))
            self.main_tab_change()
            self.statusbar.clearMessage()
            self.statusbar.showMessage(u'%s %s' % (messages.messages['open the file'],
                                                   self.node_scheme.scheme_file_name), 5000)
            # Сохранение имени открытого файла в файле recent (последние открытые файлы).
            self.settings.add_recent_file(self.node_scheme.scheme_file_name)
            # Скрытие кнопки оптимизации схемы и вкладки результатов оптимизации.
            self.optimization_on_off(False)

    def save_scheme(self):
        """ Сохранение файла схемы.
        """
        errors = language.Errors()
        messages = language.Messages()
        # Если не открыта схема.
        if not self.node_scheme:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])
            return
        # Если есть имя файла.
        if self.node_scheme.scheme_file_name:
            # Списки для записи ошибок, предупреждений и сообщений.
            errs = []
            warnings = []
            messages_1 = []
            # Расширение файла.
            extension = os.path.splitext(self.node_scheme.scheme_file_name)[1]
            self.statusbar.showMessage(u'%s %s' % (messages.messages['saving a file'],
                                                   self.node_scheme.scheme_file_name), 0)
            if extension == '.xnvs':
                result = self.node_scheme.write_scheme_file_xml()
            elif extension == '.nvs':
                result = self.node_scheme.write_scheme_file_nvs()
            else:
                result = self.node_scheme.write_scheme_file_nvs()
            errs += result[0]
            warnings += result[1]
            messages_1 += result[2]
            # Если после записи файла возникли ошибки.
            if errs:
                message(parent=self, mes_type='warning', title=errors.titles['error saving'], mes_list=errs)
            else:
                self.setWindowTitle(u'Node Volt - %s [%s]' % (os.path.basename(self.node_scheme.scheme_file_name),
                                                              self.node_scheme.scheme_file_name))
                self.statusbar.clearMessage()
                self.statusbar.showMessage(u'%s %s' % (messages.messages['save the file'],
                                                       self.node_scheme.scheme_file_name), 5000)
        # Если нет имени файла.
        elif not self.node_scheme.scheme_file_name:
            self. save_as_scheme()

    def save_as_scheme(self):
        """ Сохранение схемы в новый файл.
        """
        errors = language.Errors()
        messages = language.Messages()
        # Если не открыта схема.
        if not self.node_scheme:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])
            return
        scheme_file_name = \
            unicode(QtGui.QFileDialog.getSaveFileName(self, messages.messages['save scheme'],
                                                      QtCore.QDir.homePath(),
                                                      u"XNVS-%s (*.xnvs);;NVS-%s (*.nvs)" %
                                                      (messages.messages['scheme file'],
                                                       messages.messages['scheme file'])))
        if scheme_file_name:
            self.node_scheme.scheme_file_name = scheme_file_name
            self.save_scheme()
            # Сохранение имени открытого файла в файле recent (последние открытые файлы).
            self.settings.add_recent_file(self.node_scheme.scheme_file_name)

    def calculate_scheme(self):
        """ Метод, вызываемый при нажатии кнопки "Расчет".
        """
        errors = language.Errors()
        messages = language.Messages()
        # Если не открыта или не создана схема.
        if not self.node_scheme:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])
            return
        # Если словарь с параметрами элементов не пустой.
        if self.node_scheme.vetvi.children_list():
            # Списки для записи ошибок, предупреждений и сообщений.
            errs = []
            warnings = []
            messages_1 = []
            progress_window = ProgressWindow()
            progress_window.show()
            progress_bar = progress_window.progress_bar
            progress_bar.setValue(0)
            timer = time.time()
            # Расчет схемы.
            result = self.node_scheme.calculate_plus(progress_window.message_box, progress_bar, 0, 90)
            errs += result[0]
            warnings += result[1]
            messages_1 += result[2]
            timer = time.time() - timer
            progress_bar.setValue(90)
            # Проверка на наличие критических ошибок после расчета.
            # Если есть критические ошибки после расчета.
            if errs:
                progress_window.message_box.append(messages.messages['calculation critical errors'])
            # Если критических ошибок нет после расчета.
            else:
                message_text = u'%s\n' \
                               u'%s: %s\n' \
                               u'%s: %s' % (messages.messages['complete calculation'],
                                            messages.messages['iterations number'], self.node_scheme.n_shod,
                                            messages.messages['runtime'], timer)
                progress_window.message_box.append(message_text)
                # Показ кнопки оптимизации схемы и вкладки результатов оптимизации.
                self.optimization_on_off(True)
            progress_bar.setValue(100)
            progress_window.main()
            self.main_tab_change()
        # Если словарь параметров элементов пуст, то отображается сообщение с предупреждением.
        else:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])

    def optimize_scheme(self):
        """ Метод, вызываемый при нажатии кнопки "Оптимизация".
        """
        errors = language.Errors()
        messages = language.Messages()
        # Нажатие кнопки "Оптимизация".
        # Если не открыта или не создана схема.
        if not self.node_scheme:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])
            return
        # Если словарь с параметрами элементов не пустой.
        if self.node_scheme.vetvi.children_list():
            # Списки для записи ошибок, предупреждений и сообщений.
            errs = []
            warnings = []
            messages_1 = []
            progress_window = ProgressWindow()
            progress_window.show()
            progress_bar = progress_window.progress_bar
            progress_bar.setValue(0)
            timer = time.time()
            # Оптимизация схемы.
            optimize = optimization.SchemeOptimization(self.node_scheme)
            result = optimize.optimize(progress_window.message_box, progress_bar, 0, 95)
            errs += result[0]
            warnings += result[1]
            messages_1 += result[2]
            timer = time.time() - timer
            if errs:
                message_text = messages.messages['optimization errors']
            elif errs:
                message_text = messages.messages['optimization warnings']
            else:
                message_text = messages.messages['complete optimization']
            message_text += u'\n%s: %s' % (messages.messages['runtime'], timer)
            progress_window.message_box.append(message_text)
            self.optimization_table_normal_model = models.OptimizationTableModel(optimize.schemes_normal,
                                                                                 self.node_scheme)
            self.table_normal_schemes.setModel(self.optimization_table_normal_model)
            self.optimization_table_overdrive_model = models.OptimizationTableModel(optimize.schemes_overdrive,
                                                                                    self.node_scheme)
            self.table_overdrive_schemes.setModel(self.optimization_table_overdrive_model)
            self.optimization_table_bad_model = models.OptimizationTableModel(optimize.schemes_bad, self.node_scheme)
            self.table_bad_schemes.setModel(self.optimization_table_bad_model)
            # Установка ширин столбцов таблиц схем.
            for i in range(4):
                self.table_overdrive_schemes.setColumnWidth(i, 180)
                self.table_normal_schemes.setColumnWidth(i, 180)
                self.table_bad_schemes.setColumnWidth(i, 180)
            progress_bar.setValue(100)
            progress_window.main()
            self.main_tab_change()
        # Если словарь параметров элементов пуст, то отображается сообщение с предупреждением.
        else:
            message(parent=self, mes_type='warning', title=errors.titles['no scheme'],
                    mes_list=[errors.messages['no scheme']])

    def export_results_csv(self):
        """ Метод, вызываемый при нажатии кнопки "Экспорт в CSV".
        """
        errors = language.Errors()
        messages = language.Messages()
        # Если открыта или создана схема.
        if not self.node_scheme:
            message(parent=self, mes_type='warning', title=errors.titles['export do not complete'],
                    mes_list=[errors.messages['no scheme']])
            return
        # Если словарь с расчетными параметрами не пуст.
        if self.node_scheme.vetvi.children_list():
            # Списки для записи ошибок, предупреждений и сообщений.
            errs = []
            warnings = []
            messages_1 = []
            file_name = unicode(QtGui.QFileDialog.getSaveFileName(self, messages.messages['results export'],
                                                                  QtCore.QDir.homePath(),
                                                                  u"CSV-%s (*.csv)" % messages.messages['scheme file']))
            if file_name:
                result = nv_plus.results_data(node_scheme=self.node_scheme, file_name=file_name,
                                              round_n=self.settings.round_n, file_point=self.settings.file_point)
                errs += result[0]
                warnings += result[1]
                messages_1 += result[2]
                if errs:
                    message(parent=self, mes_type='warning', title=errors.titles['export do not complete'],
                            mes_list=errs)
        # Если словарь с расчетными параметрами пуст.
        else:
            message(parent=self, mes_type='warning', title=errors.titles['export do not complete'],
                    mes_list=[errors.messages['calculation need']])
    
    def create_style(self):
        # Создание стиля.
        self.setStyleSheet(style.MAIN_STYLE_SHEET)
        # Стиль области, ограничивающей главное меню.
        self.frame.setStyleSheet(style.MAIN_MENU_FRAME_STYLE_SHEET)
        # Стиль главного меню.
        self.tab_menu.setStyleSheet(style.MAIN_MENU_TAB_STYLE_SHEET)
        # Стиль области, ограничивающей основное меню в главном меню.
        self.frame_main_menu.setStyleSheet(style.MAIN_MENU_FRAME_MAIN_STYLE_SHEET)
        # Стиль области, ограничивающей меню редактирования в главном меню.
        self.frame_edit_menu.setStyleSheet(style.MAIN_MENU_FRAME_SETTINGS_STYLE_SHEET)
        # Стиль области, ограничивающей рабочую область (схема, списки элементов, узлов и др.).
        self.frame_2.setStyleSheet(style.MAIN_FRAME_STYLE_SHEET)
        # Стиль вкладок рабочей области (схема, элементы, узлы, результаты).
        self.tab_main.setStyleSheet(style.MENU_TAB_STYLE_SHEET)
        # Стиль вкладки "Схема".
        self.tab_scheme.setStyleSheet(style.TAB_SCHEME_STYLE_SHEET)
        # Стиль дерева параметров.
        self.tree_properties.setStyleSheet(style.TREE_PROPERTIES_STYLE_SHEET)
        # Стиль таблиц элементов, узлов, результатов расчета и результатов оптимизации.
        self.table_vetvi.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        self.table_usli.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        self.table_results.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        self.table_overdrive_schemes.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        self.table_normal_schemes.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        self.table_bad_schemes.setStyleSheet(style.ELEMENTS_TABLE_STYLE_SHEET)
        # Стиль разделителя.
        self.splitter.setStyleSheet(style.SPLITTER_STYLE_SHEET)

    def optimization_on_off(self, on=True):
        if on:
            self.btn_optimization.setEnabled(True)
            self.tab_optimization.setVisible(True)
            self.tab_optimization.setEnabled(True)
            self.tab_main.setCurrentIndex(0)
        else:
            self.btn_optimization.setEnabled(False)
            self.tab_optimization.setVisible(False)
            self.tab_optimization.setEnabled(False)
            self.optimization_table_normal_model = None
            self.table_normal_schemes.setModel(None)
            self.optimization_table_overdrive_model = None
            self.table_overdrive_schemes.setModel(None)
            self.optimization_table_bad_model = None
            self.table_bad_schemes.setModel(None)

    def open_start_window(self):
        # Начальное окно ("Новая схема", "Открыть").
        start_window = StartWindow()
        start_window_result = start_window.main(self.settings)
        if start_window_result[0] == 1:
            # Диалог создания новой схемы.
            self.create_scheme(ask_save=False)
        elif start_window_result[0] in (2, 3):
            # Диалог открытия схемы.
            self.open_scheme(start_window_result[1])

    def main(self, scheme_name=u''):
        # Установка настроек разделителя на вкладке "Схема", который разделяет схему и дерево параметров.
        self.splitter.setStretchFactor(0, 0)
        self.splitter.setStretchFactor(1, 1)
        # Создание стиля.
        self.create_style()
        # Установка настроек таблицы результатов
        self.table_results.verticalHeader().setVisible(False)
        self.table_results.horizontalHeader().setVisible(False)
        # Установка настроек графической сцены.
        self.graphics_scheme.setDragMode(QtGui.QGraphicsView.RubberBandDrag)
        self.graphics_scheme.setRubberBandSelectionMode(QtCore.Qt.IntersectsItemBoundingRect)
        #self.graphics_scheme.setDragMode(QtGui.QGraphicsView.ScrollHandDrag)
        self.graphics_scheme.setRenderHint(QtGui.QPainter.Antialiasing)
        if self.graphics_scheme:
            m = QtGui.QTransform().fromScale(2, 2)
            self.graphics_scheme.setTransform(m)
        # Скрытие кнопки оптимизации схемы и вкладки результатов оптимизации.
        self.optimization_on_off(False)
        # Переход к заданым вкладкам.
        self.tab_menu.setCurrentIndex(0)
        self.tab_main.setCurrentIndex(0)
        # Установка размеров.
        if self.settings.geometry is not None:
            self.restoreGeometry(self.settings.geometry)
        if self.settings.splitter_scheme is not None:
            self.splitter.restoreState(self.settings.splitter_scheme)
        # Запуск.
        self.show()
        if not scheme_name:
            # Открытие начального окна.
            self.open_start_window()
        else:
            self.open_scheme(scheme_name)

    def closeEvent(self, event):
        messages = language.Messages()
        self.settings.geometry = self.saveGeometry()
        self.settings.splitter_scheme = self.splitter.saveState()
        self.settings.write_gui_settings()

        if self.node_scheme:
            result = QtGui.QMessageBox.question(self, messages.messages['save scheme'],
                                                messages.messages['save scheme?'],
                                                QtGui.QMessageBox.Save | QtGui.QMessageBox.No |
                                                QtGui.QMessageBox.Cancel,
                                                QtGui.QMessageBox.No)
            if result == QtGui.QMessageBox.Save:
                self.save_scheme()
            elif result == QtGui.QMessageBox.Cancel:
                event.ignore()


if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)

    if len(sys.argv) >= 2:
        open_file = sys.argv[1]
    else:
        open_file = u''
    # Считывание настроек.
    settings = Settings()
    settings.read_settings()
    settings.read_gui_settings()

    # Установка языка.
    module_path_1 = module_path()
    my_translator = QtCore.QTranslator()
    qt_translator = QtCore.QTranslator()
    language_name = settings.language
    if language_name in language.LANGUAGES:
        language_name = language_name.split(' - ')[0]
        if module_path_1:
            my_translator.load(u'%s/language/node_volt_%s' % (module_path_1, language_name))
            qt_translator.load(u'%s/language/qt_%s' % (module_path_1, language_name))
        else:
            my_translator.load(u'language/node_volt_%s' % language_name)
            qt_translator.load(u'language/qt_%s' % (language_name))
    else:
        if module_path_1:
            my_translator.load(u'%s/language/node_volt_ru' % module_path_1)
            qt_translator.load(u'%s/language/qt_ru' % module_path_1)
        else:
            my_translator.load(u'language/node_volt_ru')
            qt_translator.load(u'language/qt_ru')
    app.installTranslator(my_translator)
    app.installTranslator(qt_translator)

    node_volt = NodeVolt(settings)
    node_volt.main(open_file)
    
    # Запуск.
    app.exec_()