# -*- coding: utf-8 -*-
""" Модуль описывает классы древовидной структуры набора элементов и узлов схемы с их параметрами.
"""

UNICODE_PARAMETERS = ['vetv_type', 'note', 'begin', 'end', 'end_2', 'connection_diagram', 'view',
                      'generator_switcher']
FLOAT_PARAMETERS = ['real_voltage', 'real_phase', 'voltage', 'voltage_psi', 'active_power', 'reactive_power',
                    'active_resistance', 'reactive_resistance', 'active_resistance_2', 'reactive_resistance_2',
                    'active_resistance_3', 'reactive_resistance_3', 'eds', 'eds_psi', 'current', 'current_psi',
                    'active_conductivity_tr1', 'reactive_conductivity_tr1', 'active_conductivity_tr2',
                    'reactive_conductivity_tr2', 'real_u', 'real_e', 'real_u_2', 'real_u_3', 'real_u1', 'real_u1_2',
                    'real_u1_3', 'real_u2', 'real_u2_2', 'real_u2_3', 'real_i', 'real_i_2', 'real_i_3', 's1', 's1_2',
                    's1_3', 's2', 's2_2', 's2_3', 'ist_s', 'priem_s', 'priem_s_2', 'priem_s_3', 'real_i1_tr',
                    'real_i2_tr', 'real_i3_tr', 'priem_s1_tr', 'priem_s2_tr', 'priem_s3_tr', 'position_x',
                    'position_y', 'name_position_x', 'name_position_y', 'rotation', 'nominal_temperature',
                    'operating_temperature', 'short_current_temperature',
                    'temperature_coefficient', 'generator_priority', 'generator_nominal_active_power',
                    'generator_nominal_reactive_power', 'generator_nominal_full_power',
                    'generator_normal_power_coefficient', 'generator_overdrive_power_coefficient',
                    'minimum_voltage', 'maximum_voltage', 'voltage_addition', 'voltage_addition_psi',
                    'transformation_coefficient', 'transformation_coefficient_psi',
                    'transformation_coefficient_2', 'transformation_coefficient_psi_2']
BOOL_PARAMETERS = ['mirror_x', 'mirror_y', 'not_visible', 'rotate_line_1', 'rotate_line_2', 'rotate_line_3',
                   'use_for_optimization', 'switcher_permit', 'check_direction']
LIST_PARAMETERS = ['real_u_usel', 'vetvi']


class Node(object):

    def __init__(self, node_name, node_type='', node_value=u'', parent=None, node_description=u''):
        self._name = u''
        self._value = ''
        self._type = ''
        self._description = u''
        self._children = []
        self._parent = parent

        self.set_name(node_name)
        self.set_type(node_type)
        self.set_value(node_value)
        self.set_description(node_description)

    def children_list(self):
        return [child.name() for child in self._children]

    def value(self):
        return self._value

    def type(self):
        return self._type

    def description(self):
        return self._description

    def set_description(self, node_description):
        if isinstance(node_description, basestring):
            self._description = node_description
            return True
        return False

    def set_type(self, node_type):
        if node_type in (None, '', 'bool', 'unicode', 'str', 'int', 'float', 'complex', 'list', 'list_complex',
                         'parameters', 'vetv', 'usel'):
            self._type = node_type
            self.set_value(self._value)
            return True
        return False

    def set_value(self, node_value):
        if self._type in ('', 'unicode', 'parameters', 'vetv', 'usel'):
            self._value = unicode(node_value)
            return self._value
        elif self._type == 'bool':
            if node_value == 'False':
                self._value = False
            else:
                self._value = bool(node_value)
            return self._value
        elif self._type == 'str':
            self._value = str(node_value)
            return self._value
        elif self._type == 'int':
            if isinstance(node_value, basestring):
                node_value = node_value.replace(',', '.')
                node_value = node_value.split('.')[0]
            if not node_value:
                self._value = 0
                return self._value
            try:
                self._value = int(node_value)
                return self._value
            except (ValueError, TypeError):
                return False
        elif self._type == 'float':
            if isinstance(node_value, basestring):
                node_value = node_value.replace(',', '.')
            if node_value == '':
                self._value = 0.
                return self._value
            try:
                self._value = float(node_value)
                return self._value
            except (ValueError, TypeError):
                return False
        elif self._type == 'complex':
            if isinstance(node_value, basestring):
                node_value = node_value.replace(',', '.')
            if node_value == '':
                self._value = 0. + 0.j
                return self._value
            try:
                self._value = complex(node_value)
                return self._value
            except (ValueError, TypeError):
                return False
        elif self._type in ('list', 'list_complex'):
            if isinstance(node_value, list):
                self._value = node_value
            elif isinstance(node_value, basestring):
                node_value = node_value.replace('[', '')
                node_value = node_value.replace(']', '')
                node_value = node_value.split(', ')
                if self._type == 'list_complex':
                    self._value = []
                    for value in node_value:
                        try:
                            self._value.append(complex(value))
                        except ValueError:
                            pass
                else:
                    self._value = []
            else:
                self._value = []
            return self._value
        return False

    def add_child(self, child):
        self._children.append(child)
        child._parent = self
        return True

    def insert_child(self, child, position=None):
        if not isinstance(position, int):
            position = None
        if position is None:
            if position < 0 or position > len(self._children):
                self._children.insert(len(self._children), child)
        self._children.insert(position, child)
        child._parent = self
        return True

    def remove_child(self, position):
        if position < 0 or position >= len(self._children):
            return False
        child = self._children.pop(position)
        child._parent = None
        return True

    def name(self):
        return unicode(self._name)

    def set_name(self, name):
        if self._parent is not None:
            if name not in self._parent.children_list():
                self._name = unicode(name)
                return True
            return False
        else:
            self._name = unicode(name)
            return True

    def child(self, position):
        if position < 0 or position >= len(self._children):
            return False
        return self._children[position]

    def child_count(self):
        return len(self._children)

    def find_child(self, child_name):
        if isinstance(child_name, basestring):
            j = 0
            for child in self._children:
                if child.name() == child_name:
                    return self._children[j]
                j += 1
        return False

    def find_child_position(self, child_name):
        if isinstance(child_name, basestring):
            for j in range(self.child_count()):
                if self.child(j).name() == child_name:
                    return j
        return False

    def parent(self):
        return self._parent

    def set_parent(self, parent):
        self._parent = parent

    def print_parent(self):
        print self._parent

    def row(self):
        if self._parent is not None:
            return self._parent._children.index(self)

    def log(self, tab_level=-1):
        output = u''
        tab_level += 1
        for j in range(tab_level):
            output += u"\t"
        output += u"|------%s = %s\n" % (unicode(self._name), unicode(self._value))
        for child in self.children_list():
            child = self.find_child(child)
            if child:
                output += child.log(tab_level)
        tab_level -= 1
        return output

    def __repr__(self):
        return self.log()

    def remove_children(self):
        for child in self._children:
            child._parent = None
        self._children = []

    def self_copy(self):
        self_copy = deep_copy_element(self)
        return self_copy


class Parameters2(Node):

    def __init__(self, node_name='', node_type='', node_value=u'', parent=None, node_description=u''):
        Node.__init__(self, node_name, node_type, node_value, parent, node_description)
        self._children = {}
        self._children_list = []

    def children_list(self):
        return self._children.keys()

    def add_child(self, child):
        if child.name() not in self._children:
            self._children[child.name()] = child
            child._parent = self
            return True
        else:
            return False

    def remove_child(self, child_name=''):
        child = self.find_child(child_name)
        if child:
            del self._children[child_name]
            child._parent = None
            return True
        return False

    def find_child(self, child_name):
        return self._children.get(child_name, False)

    def parameter(self, param_name, param_type=None):
        if param_name not in self._children:
            self.add_child(Parameters(node_name=param_name, node_type=param_type, parent=self))
        return self._children[param_name]

    def remove_children(self):
        for child in self._children.values():
            child._parent = None
        self._children = {}

    def log(self, tab_level=-1):
        output = u''
        tab_level += 1
        for j in range(tab_level):
            output += u"\t"
        output += u"|------%s = %s\n" % (unicode(self._name), unicode(self._value))
        for child in self._children.values():
            output += child.log(tab_level)
        tab_level -= 1
        return output


class Parameters(Node):

    def __init__(self, node_name='', node_type='', node_value=u'', parent=None, node_description=u''):
        Node.__init__(self, node_name, node_type, node_value, parent, node_description)
        self._children = {}
        self._children_list = []

    def children_list(self):
        return self._children_list[:]

    def add_child(self, child):
        if child.name() not in self._children:
            self._children[child.name()] = child
            self._children_list.append(child.name())
            child._parent = self
            return True
        else:
            return False

    def insert_child(self, child, position=None):
        if not isinstance(position, int):
            position = None
        if self.add_child(child):
            if position is not None:
                if not (position < 0 or position > len(self._children)):
                    self._children_list.pop(len(self._children) - 1)
                    self._children_list.insert(position, child.name())
            return True
        else:
            return False

    def remove_child(self, child_name_or_position=''):
        if isinstance(child_name_or_position, basestring):
            child = self.find_child(child_name_or_position)
            if child and child_name_or_position in self._children_list:
                del self._children[child_name_or_position]
                child._parent = None
                self._children_list.remove(child_name_or_position)
                return True
        elif isinstance(child_name_or_position, int):
            if not (child_name_or_position < 0 or child_name_or_position > len(self._children)):
                child_name = self._children_list[child_name_or_position]
                child = self.find_child(child_name)
                del self._children[child_name]
                child._parent = None
                self._children_list.pop(child_name_or_position)
                return True
        return False

    def child(self, position):
        if position < 0 or position >= len(self._children):
            return False
        child_name = self._children_list[position]
        return self._children.get(child_name, False)

    def find_child(self, child_name):
        return self._children.get(child_name, False)

    def find_child_position(self, child_name):
        if isinstance(child_name, basestring):
            for j in range(len(self._children)):
                if self._children_list[j] == child_name:
                    return j
        return False

    def row(self):
        if self._parent is not None:
            return self._parent.find_child_position(self._name)

    def parameter(self, param_name, param_type=None):
        if param_name not in self._children:
            self.add_child(Parameters(node_name=param_name, node_type=param_type, parent=self))
        return self._children[param_name]

    def remove_children(self):
        for child in self._children.values():
            child._parent = None
        self._children = {}
        self._children_list = []

    def log(self, tab_level=-1):
        output = u''
        tab_level += 1
        for j in range(tab_level):
            output += u"\t"
        output += u"|------%s = %s\n" % (unicode(self._name), unicode(self._value))
        for child in self._children.values():
            output += child.log(tab_level)
        tab_level -= 1
        return output


class InputParameters(Parameters):

    def __init__(self, node_name='input_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def vetv_type(self):
        # Тип ветви.
        return self.parameter('vetv_type', 'unicode')

    def note(self):
        # Примечание, отображаемое на схеме.
        return self.parameter('note', 'unicode')

    def new_element_name(self):
        # Имя которое было присвоено при добавлении элемента. Имена в файле new_elements.xml
        return self.parameter('new_element_name', 'unicode')

    def begin(self):
        # Имя начального узла ветви.
        return self.parameter('begin', 'unicode')

    def end(self):
        # Имя конечного узла ветви.
        return self.parameter('end', 'unicode')

    def end_2(self):
        # Имя второго конечного узла ветви.
        return self.parameter('end_2', 'unicode')

    def transformation_coefficient(self):
        # Модуль коэффициента трансформации.
        return self.parameter('transformation_coefficient', 'float')

    def transformation_coefficient_psi(self):
        # Фаза коэффициента трансформации.
        return self.parameter('transformation_coefficient_psi', 'float')

    def transformation_coefficient_2(self):
        # Модуль коэффициента трансформации.
        return self.parameter('transformation_coefficient_2', 'float')

    def transformation_coefficient_psi_2(self):
        # Фаза коэффициента трансформации.
        return self.parameter('transformation_coefficient_psi_2', 'float')

    def real_voltage(self):
        # Реальное напряжение ветви.
        return self.parameter('real_voltage', 'float')

    def real_phase(self):
        # Поворот фазы при приведении.
        return self.parameter('real_phase', 'float')

    def eds(self):
        # Модуль ЭДС ветви.
        return self.parameter('eds', 'float')

    def eds_psi(self):
        # Начальная фаза ЭДС ветви.
        return self.parameter('eds_psi', 'float')

    def voltage(self):
        # Модуль номинального напряжения источника или нагрузки.
        return self.parameter('voltage', 'float')

    def voltage_psi(self):
        # Начальная фаза номинального напряжения источника или нагрузки.
        return self.parameter('voltage_psi', 'float')

    def active_power(self):
        # Активная мощность источника или нагрузки.
        return self.parameter('active_power', 'float')

    def reactive_power(self):
        # Реактивная мощность источника или нагрузки.
        return self.parameter('reactive_power', 'float')

    def current(self):
        # Модуль тока источника тока.
        return self.parameter('current', 'float')

    def current_psi(self):
        # Начальная фаза тока источника тока.
        return self.parameter('current_psi', 'float')

    def active_resistance(self):
        # Активное сопротивление ветви.
        return self.parameter('active_resistance', 'float')

    def reactive_resistance(self):
        # Реактивное сопротивление ветви.
        return self.parameter('reactive_resistance', 'float')

    def active_resistance_2(self):
        # Активное сопротивление второй ветви.
        return self.parameter('active_resistance_2', 'float')

    def reactive_resistance_2(self):
        # Реактивное сопротивление второй ветви.
        return self.parameter('reactive_resistance_2', 'float')

    def active_resistance_3(self):
        # Активное сопротивление третьей ветви.
        return self.parameter('active_resistance_3', 'float')

    def reactive_resistance_3(self):
        # Реактивное сопротивление третьей ветви.
        return self.parameter('reactive_resistance_3', 'float')

    def connection_diagram(self):
        # Схема соединения (-, D, Y, Y0).
        return self.parameter('connection_diagram', 'unicode')

    def active_conductivity_tr1(self):
        # Активная проводимость поперечной ветви в начальном узле.
        return self.parameter('active_conductivity_tr1', 'float')

    def active_conductivity_tr2(self):
        # Активная проводимость поперечной ветви в конечном узле.
        return self.parameter('active_conductivity_tr2', 'float')

    def reactive_conductivity_tr1(self):
        # Реактивная проводимость поперечной ветви в начальном узле.
        return self.parameter('reactive_conductivity_tr1', 'float')

    def reactive_conductivity_tr2(self):
        # Реактивная проводимость поперечной ветви в конечном узле.
        return self.parameter('reactive_conductivity_tr2', 'float')

    def voltage_addition(self):
        # Модуль добавки напряжения.
        return self.parameter('voltage_addition', 'float')

    def voltage_addition_psi(self):
        # Фаза добавки напряжения.
        return self.parameter('voltage_addition_psi', 'float')


class UsliInputParameters(Parameters):

    def __init__(self, node_name='input_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def note(self):
        # Примечание, отображаемое на схеме.
        return self.parameter('note', 'unicode')


class GuiParameters(Parameters):

    def __init__(self, node_name='gui_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def position_x(self):
        # Позиция элемента по оси X.
        return self.parameter('position_x', 'float')

    def position_y(self):
        # Позиция элемента по оси Y.
        return self.parameter('position_y', 'float')

    def name_position_x(self):
        # Позиция имени элемента по оси X.
        return self.parameter('name_position_x', 'float')

    def name_position_y(self):
        # Позиция имени элемента по оси Y.
        return self.parameter('name_position_y', 'float')

    def rotation(self):
        # Поворот элемента
        return self.parameter('rotation', 'float')

    def mirror_x(self):
        # Отражение элемента по оси X.
        return self.parameter('mirror_x', 'bool')

    def mirror_y(self):
        # Отражение элемента по оси Y.
        return self.parameter('mirror_y', 'bool')

    def view(self):
        # Графический вид элемента.
        return self.parameter('view', 'unicode')

    def not_visible(self):
        # Невидимость элемента на схеме.
        return self.parameter('visible', 'bool')

    def rotate_line_1(self):
        # Повернуть соединительную линию 1.
        return self.parameter('rotate_line_1', 'bool')

    def rotate_line_2(self):
        # Повернуть соединительную линию 2.
        return self.parameter('rotate_line_2', 'bool')

    def rotate_line_3(self):
        # Повернуть соединительную линию 3.
        return self.parameter('rotate_line_3', 'bool')


class OutputParameters(Parameters):

    def __init__(self, node_name='output_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    # Промежуточные исходные данные.

    def e(self):
        # Комплекс приведенной ЭДС ветви.
        return self.parameter('e', 'complex')

    def u(self):
        # Комплекс номинального напряжения ветви.
        return self.parameter('u', 'complex')

    def y(self):
        # Комплекс проводимости ветви.
        return self.parameter('y', 'complex')

    def j(self):
        # Комплекс тока источника тока ветви.
        return self.parameter('j', 'complex')

    def s(self):
        # Комплекс номинальной полной мощности ветви.
        return self.parameter('s', 'complex')

    def real_voltage(self):
        # Реальное напряжение ветви.
        return self.parameter('real_voltage', 'float')

    def k_p(self):
        # Коэффициент приведения ветви.
        return self.parameter('k_p', 'complex')

    def begin(self):
        # Имя начального узла ветви.
        return self.parameter('begin', 'unicode')

    def end(self):
        # Имя конечного узла ветви.
        return self.parameter('end', 'unicode')

    # Промежуточные результаты расчета.

    def u1(self):
        # Комплекс приведенного напряжения в начале ветви.
        return self.parameter('u1', 'complex')

    def u2(self):
        # Комплекс приведенного напряжения в конце ветви.
        return self.parameter('u2', 'complex')

    def i(self):
        # Комплекс приведенного значения тока ветви.
        return self.parameter('i', 'complex')

    # Результаты расчета.

    def real_e(self):
        # Комплекс реальной ЭДС ветви.
        return self.parameter('real_e', 'complex')

    def s1(self):
        # Комплекс полной мощности в начале ветви.
        return self.parameter('s1', 'complex')

    def s1_2(self):
        # Комплекс полной мощности в начале второй ветви (трехлучевой).
        return self.parameter('s1_2', 'complex')

    def s1_3(self):
        # Комплекс полной мощности в начале третьей ветви (трехлучевой).
        return self.parameter('s1_3', 'complex')

    def s2(self):
        # Комплекс полной мощности в конце ветви.
        return self.parameter('s2', 'complex')

    def s2_2(self):
        # Комплекс полной мощности в конце второй ветви (трехлучевой).
        return self.parameter('s2_2', 'complex')

    def s2_3(self):
        # Комплекс полной мощности в конце третьей ветви (трехлучевой).
        return self.parameter('s2_3', 'complex')

    def real_u(self):
        # Комплекс реального напряжения на ветви.
        return self.parameter('real_u', 'complex')

    def real_u_2(self):
        # Комплекс реального напряжения на второй ветви (трехлучевой).
        return self.parameter('real_u_2', 'complex')

    def real_u_3(self):
        # Комплекс реального напряжения на третьей ветви (трехлучевой).
        return self.parameter('real_u_3', 'complex')

    def real_u1(self):
        # Комплекс реального напряжения в начале ветви.
        return self.parameter('real_u1', 'complex')

    def real_u1_2(self):
        # Комплекс реального напряжения в начале второй ветви (трехлучевой).
        return self.parameter('real_u1_2', 'complex')

    def real_u1_3(self):
        # Комплекс реального напряжения в начале третьей ветви (трехлучевой).
        return self.parameter('real_u1_3', 'complex')

    def real_u2(self):
        # Комплекс реального напряжения в конце ветви.
        return self.parameter('real_u2', 'complex')

    def real_u2_2(self):
        # Комплекс реального напряжения в конце второй ветви (трехлучевой).
        return self.parameter('real_u2_2', 'complex')

    def real_u2_3(self):
        # Комплекс реального напряжения в конце третьей ветви (трехлучевой).
        return self.parameter('real_u2_3', 'complex')

    def real_i(self):
        # Комплекс реального значения тока ветви.
        return self.parameter('real_i', 'complex')

    def real_i_2(self):
        # Комплекс реального значения тока второй ветви (трехлучевой).
        return self.parameter('real_i_2', 'complex')

    def real_i_3(self):
        # Комплекс реального значения тока третьей ветви (трехлучевой).
        return self.parameter('real_i_3', 'complex')

    def real_i1_tr(self):
        # Комплекс реального значения тока поперечной ветви в начальном узле.
        return self.parameter('real_i1_tr', 'complex')

    def real_i2_tr(self):
        # Комплекс реального значения тока поперечной ветви в конечном узле.
        return self.parameter('real_i2_tr', 'complex')

    def real_i3_tr(self):
        # Комплекс реального значения тока поперечной ветви в конечном узле 2.
        return self.parameter('real_i3_tr', 'complex')

    def priem_s(self):
        # Комплекс полной мощности пассивных элементов ветви.
        return self.parameter('priem_s', 'complex')

    def priem_s_2(self):
        # Комплекс полной мощности пассивных элементов второй ветви (трехлучевой).
        return self.parameter('priem_s_2', 'complex')

    def priem_s_3(self):
        # Комплекс полной мощности пассивных элементов третьей ветви (трехлучевой).
        return self.parameter('priem_s_3', 'complex')

    def priem_s1_tr(self):
        # Комплекс полной мощности поперечной ветви в начальном узле.
        return self.parameter('priem_s1_tr', 'complex')

    def priem_s2_tr(self):
        # Комплекс полной мощности поперечной ветви в конечном узле.
        return self.parameter('priem_s2_tr', 'complex')

    def priem_s3_tr(self):
        # Комплекс полной мощности поперечной ветви в конечном узле 2.
        return self.parameter('priem_s3_tr', 'complex')

    def ist_s(self):
        # Комплекс полной мощности активных элементов ветви.
        return self.parameter('ist_s', 'complex')


class UsliOutputParameters(Parameters):

    def __init__(self, node_name='output_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def real_u_usel(self):
        # Список комплексов реальных напряжений в узле.
        return self.parameter('real_u_usel', 'list_complex')

    def vetvi(self):
        # Список имен ветвей присоединенных к узлу.
        return self.parameter('vetvi', 'list')

    def u(self):
        # Комплекс напряжения узла.
        return self.parameter('u', 'complex')


class BufferParameters(Parameters):

    def __init__(self, node_name='buffer_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def active_resistance(self):
        # Активное сопротивление ветви.
        return self.parameter('active_resistance', 'float')

    def reactive_resistance(self):
        # Реактивное сопротивление ветви.
        return self.parameter('reactive_resistance', 'float')


class OptimizationParameters(Parameters):

    def __init__(self, node_name='optimization_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def use_for_optimization(self):
        # Тип элемента для оптимизации схемы.
        return self.parameter('use_for_optimization', 'bool')

    def generator_switcher(self):
        # Имя элемента являющегося выключателем источника.
        return self.parameter('generator_switcher', 'unicode')

    def generator_priority(self):
        # Приоритет источника.
        return self.parameter('generator_priority', 'float')

    def generator_nominal_active_power(self):
        # Номинальная мощность источника.
        return self.parameter('generator_nominal_active_power', 'float')

    def generator_nominal_reactive_power(self):
        # Номинальная реактивная мощность источника.
        return self.parameter('generator_nominal_reactive_power', 'float')

    def generator_nominal_full_power(self):
        # Номинальная полная мощность источника.
        return self.parameter('generator_nominal_full_power', 'float')

    def generator_normal_power_coefficient(self):
        # Коэффициент нормальной работы источника.
        return self.parameter('generator_normal_power_coefficient', 'float')

    def generator_overdrive_power_coefficient(self):
        # Коэффициент перегрузки источника.
        return self.parameter('generator_overdrive_power_coefficient', 'float')

    def minimum_voltage(self):
        # Минимальное напряжение узла.
        return self.parameter('minimum_voltage', 'float')

    def maximum_voltage(self):
        # Минимальное напряжение узла.
        return self.parameter('maximum_voltage', 'float')

    def switcher_permit(self):
        # Разрешение на изменение состояния коммутационного аппарата или источника.
        return self.parameter('switcher_permit', 'bool')

    def check_direction(self):
        # Проверять ли направление потока мощности.
        return self.parameter('check_direction', 'bool')


class AdditionalParameters(Parameters):

    def __init__(self, node_name='additional_parameters', node_type='parameters', node_value=u'', parent=None,
                 node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)

    def nominal_temperature(self):
        # Номинальная температура.
        return self.parameter('nominal_temperature', 'float')

    def operating_temperature(self):
        # Рабочая температура жил.
        return self.parameter('operating_temperature', 'float')

    def short_current_temperature(self):
        # Температура жил при коротком замыкании.
        return self.parameter('short_current_temperature', 'float')

    def temperature_coefficient(self):
        # Температурный коэффициент сопротивления.
        return self.parameter('temperature_coefficient', 'float')


class Vetv(Parameters):

    def __init__(self, node_name, node_type='vetv', node_value=u'', parent=None, node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)
        self.input_parameters()
        self.gui_parameters()
        self.output_parameters()
        self.buffer_parameters()
        self.additional_parameters()
        self.optimization_parameters()

    def input_parameters(self):
        if not self.find_child('input_parameters'):
            self.add_child(InputParameters(node_name='input_parameters', parent=self))
        return self.find_child('input_parameters')

    def gui_parameters(self):
        if not self.find_child('gui_parameters'):
            self.add_child(GuiParameters(node_name='gui_parameters', parent=self))
        return self.find_child('gui_parameters')

    def output_parameters(self):
        if not self.find_child('output_parameters'):
            self.add_child(OutputParameters(node_name='output_parameters', parent=self))
        return self.find_child('output_parameters')

    def buffer_parameters(self):
        if not self.find_child('buffer_parameters'):
            self.add_child(BufferParameters(node_name='buffer_parameters', parent=self))
        return self.find_child('buffer_parameters')

    def additional_parameters(self):
        if not self.find_child('additional_parameters'):
            self.add_child(AdditionalParameters(node_name='additional_parameters', parent=self))
        return self.find_child('additional_parameters')

    def optimization_parameters(self):
        if not self.find_child('optimization_parameters'):
            self.add_child(OptimizationParameters(node_name='optimization_parameters', parent=self))
        return self.find_child('optimization_parameters')

    def rotate_connection_line_1(self):
        # Функция изменения параметра gui_parameters().rotate_line_1(),
        # которая поворачивает соединительную линию ветви, связанную с начальным узлом
        if self.gui_parameters().rotate_line_1().value():
            self.gui_parameters().rotate_line_1().set_value(False)
        else:
            self.gui_parameters().rotate_line_1().set_value(True)

    def rotate_connection_line_2(self):
        # Функция изменения параметра gui_parameters().rotate_line_1(),
        # которая поворачивает соединительную линию ветви, связанную с конечным узлом 1
        if self.gui_parameters().rotate_line_2().value():
            self.gui_parameters().rotate_line_2().set_value(False)
        else:
            self.gui_parameters().rotate_line_2().set_value(True)

    def rotate_connection_line_3(self):
        # Функция изменения параметра gui_parameters().rotate_line_1(),
        # которая поворачивает соединительную линию ветви, связанную с конечным узлом 2
        if self.gui_parameters().rotate_line_3().value():
            self.gui_parameters().rotate_line_3().set_value(False)
        else:
            self.gui_parameters().rotate_line_3().set_value(True)


class Usel(Vetv):

    def __init__(self, node_name, node_type='usel', node_value=u'', parent=None, node_description=u''):
        Parameters.__init__(self, node_name, node_type, node_value, parent, node_description)
        self.input_parameters()
        self.gui_parameters()
        self.output_parameters()
        self.buffer_parameters()
        self.additional_parameters()
        self.optimization_parameters()

    def input_parameters(self):
        if not self.find_child('input_parameters'):
            self.add_child(UsliInputParameters(node_name='input_parameters', parent=self))
        return self.find_child('input_parameters')

    def gui_parameters(self):
        if not self.find_child('gui_parameters'):
            self.add_child(GuiParameters(node_name='gui_parameters', parent=self))
        return self.find_child('gui_parameters')

    def output_parameters(self):
        if not self.find_child('output_parameters'):
            self.add_child(UsliOutputParameters(node_name='output_parameters', parent=self))
        return self.find_child('output_parameters')

    def optimization_parameters(self):
        if not self.find_child('optimization_parameters'):
            self.add_child(OptimizationParameters(node_name='optimization_parameters', parent=self))
        return self.find_child('optimization_parameters')


def deep_copy_element(element):
    # Функция, которая создает копию узла или ветви со всеми параметрами.
    # Создание нового элемента.
    if element.type() == 'vetv':
        new_element = Vetv(element.name(), 'vetv', element.value(), None, element.description())
    elif element.type() == 'usel':
        new_element = Usel(element.name(), 'usel', element.value(), None, element.description())
    else:
        return False
    # Копирование всех параметров старого элемента в новый.
    for group_parameters in element.children_list():
        group_parameters = element.find_child(group_parameters)
        if group_parameters:
            # Создание новой группы параметров.
            if group_parameters.name() == 'input_parameters':
                new_group_parameters = new_element.input_parameters()
            elif group_parameters.name() == 'output_parameters':
                new_group_parameters = new_element.output_parameters()
            elif group_parameters.name() == 'gui_parameters':
                new_group_parameters = new_element.gui_parameters()
            elif group_parameters.name() == 'buffer_parameters':
                new_group_parameters = new_element.buffer_parameters()
            elif group_parameters.name() == 'additional_parameters':
                new_group_parameters = new_element.additional_parameters()
            elif group_parameters.name() == 'optimization_parameters':
                new_group_parameters = new_element.optimization_parameters()
            else:
                new_group_parameters = None
            if new_group_parameters is not None:
                new_element.add_child(new_group_parameters)
                # Создание новых параметров.
                for parameter in group_parameters.children_list():
                    parameter = group_parameters.find_child(parameter)
                    if parameter:
                        new_parameter = Parameters(parameter.name(), parameter.type(), parameter.value())
                        new_group_parameters.add_child(new_parameter)
    return new_element