# -*- coding: utf-8 -*-
""" Модуль описывает классы используемые для создания схемы с помощью графического редактора и описывающие способ
отображения элементов и узлов на графическом виде схемы.
"""

from PyQt4 import QtCore, QtGui
import xml.etree.ElementTree
import xml.etree.cElementTree as ET
import language
import models
import prog_settings


def read_graphics_elements_file(element_type, module_path):
    """ Функция чтениф файла графических изображений элементов и узлов.
    """
    # Чтение файла элементов или узлов.
    if element_type == 'vetv':
        if module_path:
            file_path = unicode(u'%s/graphics/elements.xml' % module_path)
        else:
            file_path = unicode(u'graphics/elements.xml')
    elif element_type == 'usel':
        if module_path:
            file_path = unicode(u'%s/graphics/usli.xml' % module_path)
        else:
            file_path = unicode(u'graphics/usli.xml')
    else:
        return None
    try:
        tree = ET.parse(file_path)
    except:
        return None
    root = tree.getroot()
    return root


class DrawElement(object):
    """ Класс отрисовки элемента на грфической сцене.
    """

    def __init__(self):
        self.graphics_elements = []
        self.geometry_parameters = {}
        self.root = None

    def read_size(self, graphics_view, status):
        # Чтение размеров.
        geometry_properties = graphics_view.find('geometry_properties')
        if geometry_properties is not None:
            if status:
                geometry_properties = geometry_properties.find('status_%s' % status)
                if geometry_properties is None:
                    return
            width = geometry_properties.find('width')
            height = geometry_properties.find('height')
            try:
                width = float(width.text)
                height = float(height.text)
            except ValueError:
                return
            name_position = geometry_properties.find('name_position')
            begin = geometry_properties.find('begin')
            end = geometry_properties.find('end')
            end_2 = geometry_properties.find('end_2')
            try:
                begin_x = float(begin.attrib['x'])
                begin_y = float(begin.attrib['y'])
                end_x = float(end.attrib['x'])
                end_y = float(end.attrib['y'])
            except:
                return
            try:
                name_position_x = float(name_position.attrib['x'])
                name_position_y = float(name_position.attrib['y'])
            except:
                name_position_x = 0.
                name_position_y = 0.
            try:
                end_x_2 = float(end_2.attrib['x'])
                end_y_2 = float(end_2.attrib['y'])
            except:
                end_x_2 = end_x
                end_y_2 = end_y
            self.geometry_parameters = {'width': width, 'height': height, 'begin_x': begin_x, 'begin_y': begin_y,
                                        'end_x': end_x, 'end_y': end_y, 'end_x_2': end_x_2, 'end_y_2': end_y_2,
                                        'name_position_x': name_position_x, 'name_position_y': name_position_y}

    def read_graphics_elements(self, graphics_view, status):
        # Чтение элементов графического вида.
        graphics_elements = graphics_view.find('graphics_elements')
        if graphics_elements is not None:
            if status:
                graphics_elements = graphics_elements.find('status_%s' % status)
                if graphics_elements is None:
                    return
            lines = graphics_elements.findall('line')
            rects = graphics_elements.findall('rect')
            fill_rects = graphics_elements.findall('fill_rect')
            arcs = graphics_elements.findall('arc')
            ellipses = graphics_elements.findall('ellipse')
            if lines:
                for line in lines:
                    try:
                        begin_x = float(line.attrib['begin_x'])
                        begin_y = float(line.attrib['begin_y'])
                        end_x = float(line.attrib['end_x'])
                        end_y = float(line.attrib['end_y'])
                    except:
                        begin_x = 0.
                        begin_y = 0.
                        end_x = 0.
                        end_y = 0.
                    self.graphics_elements.append({'name': 'line', 'begin_x': begin_x, 'begin_y': begin_y,
                                                   'end_x': end_x, 'end_y': end_y})
            if rects:
                for rect in rects:
                    try:
                        begin_x = float(rect.attrib['begin_x'])
                        begin_y = float(rect.attrib['begin_y'])
                        width = float(rect.attrib['width'])
                        height = float(rect.attrib['height'])
                    except:
                        begin_x = 0.
                        begin_y = 0.
                        width = 0.
                        height = 0.
                    self.graphics_elements.append({'name': 'rect', 'begin_x': begin_x, 'begin_y': begin_y,
                                                   'width': width, 'height': height})
            if ellipses:
                for ellipse in ellipses:
                    try:
                        begin_x = float(ellipse.attrib['begin_x'])
                        begin_y = float(ellipse.attrib['begin_y'])
                        width = float(ellipse.attrib['width'])
                        height = float(ellipse.attrib['height'])
                    except:
                        begin_x = 0.
                        begin_y = 0.
                        width = 0.
                        height = 0.
                    self.graphics_elements.append({'name': 'ellipse', 'begin_x': begin_x, 'begin_y': begin_y,
                                                   'width': width, 'height': height})
            if fill_rects:
                for rect in fill_rects:
                    try:
                        begin_x = float(rect.attrib['begin_x'])
                        begin_y = float(rect.attrib['begin_y'])
                        width = float(rect.attrib['width'])
                        height = float(rect.attrib['height'])
                        color = rect.attrib['color']
                    except:
                        begin_x = 0.
                        begin_y = 0.
                        width = 0.
                        height = 0.
                        color = ''
                    self.graphics_elements.append({'name': 'fill_rect', 'begin_x': begin_x, 'begin_y': begin_y,
                                                   'width': width, 'height': height, 'color': color})
            if arcs:
                for arc in arcs:
                    try:
                        begin_x = float(arc.attrib['begin_x'])
                        begin_y = float(arc.attrib['begin_y'])
                        width = float(arc.attrib['width'])
                        height = float(arc.attrib['height'])
                        start_ang = float(arc.attrib['start_ang'])
                        end_ang = float(arc.attrib['end_ang'])
                    except:
                        begin_x = 0.
                        begin_y = 0.
                        width = 0.
                        height = 0.
                        start_ang = 0.
                        end_ang = 0.
                    start_ang = int(start_ang * 16)
                    end_ang = int(end_ang * 16)
                    self.graphics_elements.append({'name': 'arc', 'begin_x': begin_x, 'begin_y': begin_y,
                                                   'width': width, 'height': height, 'start_ang': start_ang,
                                                   'end_ang': end_ang})

    def read_element_view(self, element, status):
        view = element.gui_parameters().view().value()
        self.graphics_elements = []
        self.geometry_parameters = {}

        if self.root is None:
            return

        # Чтение параметров графического элемента.
        graphics_view = self.root.find(view)
        if graphics_view is None:
            return
        # Чтение размеров.
        self.read_size(graphics_view, status)
        # Чтение элементов графического вида.
        self.read_graphics_elements(graphics_view, status)

    def element_geometry(self, element_type):
        # Функция возвращает список геометрических параметров элемента (элемента или узла).
        if not self.geometry_parameters:
            if element_type == 'vetv':
                return [15., 10., 7.5, 0., 7.5, 10., 7.5, 10., 17., 0.]
            else:
                return [5., 5., 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 6., 0.]
        return [self.geometry_parameters['width'], self.geometry_parameters['height'],
                self.geometry_parameters['begin_x'], self.geometry_parameters['begin_y'],
                self.geometry_parameters['end_x'], self.geometry_parameters['end_y'],
                self.geometry_parameters['end_x_2'], self.geometry_parameters['end_y_2'],
                self.geometry_parameters['name_position_x'], self.geometry_parameters['name_position_y']]

    def draw_element(self, painter, begin_x=0., begin_y=0., element_type='vetv'):
        # Отрисовка элемента (элемента или узла) на графической сцене.
        if not self.graphics_elements:
            if element_type == 'vetv':
                painter.drawRect(QtCore.QRectF(begin_x, begin_y, 15., 10.))
            else:
                painter.drawEllipse(QtCore.QRectF(0., 0., 5., 5.))
            return
        for element in self.graphics_elements:
            if element['name'] == 'line':
                painter.drawLine(QtCore.QLineF(begin_x+element['begin_x'], begin_y+element['begin_y'],
                                               begin_x+element['end_x'], begin_y+element['end_y']))
            elif element['name'] == 'rect':
                painter.drawRect(QtCore.QRectF(begin_x+element['begin_x'], begin_y+element['begin_y'],
                                               element['width'], element['height']))
            elif element['name'] == 'fill_rect':
                if element['color']:
                    background = QtGui.QBrush(QtGui.QColor(element['color']))
                else:
                    background = painter.pen().brush()
                painter.fillRect(QtCore.QRectF(begin_x+element['begin_x'], begin_y+element['begin_y'],
                                               element['width'], element['height']), background)
            elif element['name'] == 'ellipse':
                painter.drawEllipse(QtCore.QRectF(begin_x+element['begin_x'], begin_y+element['begin_y'],
                                                  element['width'], element['height']))
            elif element['name'] == 'arc':
                painter.drawArc(QtCore.QRectF(begin_x+element['begin_x'], begin_y+element['begin_y'],
                                              element['width'], element['height']),
                                element['start_ang'], element['end_ang'])


class Point(QtGui.QGraphicsItem):
    """ Класс, описывающий изображение точки, обозначающей начало или конец элемента или узла.
    """

    def __init__(self, parent=None, scene=None):
        QtGui.QGraphicsItem.__init__(self, parent, scene)

    def boundingRect(self):
        rect = QtCore.QRectF(0, 0, 1, 1)
        return rect

    def paint(self, painter, options, widget=None):
        painter.drawEllipse(QtCore.QRectF(-0.5, -0.5, 1., 1.))


class ElementName(QtGui.QGraphicsItem):
    """ Класс описывающий изображение на графической сцене названия элемента или узла.
    """

    def __init__(self, element, parent=None, scene=None):
        QtGui.QGraphicsItem.__init__(self, parent, scene)
        self.setFlag(QtGui.QGraphicsItem.ItemIsMovable)
        self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
        self.setFlag(QtGui.QGraphicsItem.ItemSendsGeometryChanges)
        self.element = element  # Элемент или узел, к которому привязано имя
        self.font = QtGui.QFont()
        self.font.setPixelSize(5)
        self.height = 5
        self.width = 10
        self.point = QtCore.QPointF(0., 0.)

    def name(self):
        return 'ElementName'

    def itemChange(self, change, value):
        if change == QtGui.QGraphicsItem.ItemPositionChange and self.isSelected():
            # Если имя выделено и произошло изменение позиции имени на графической сцене,
            # то позиция (x, y) записывается в параметры элемента (элемента или узла).
            point = value.toPointF()
            self.element.gui_parameters().name_position_x().set_value(point.x())
            self.element.gui_parameters().name_position_y().set_value(point.y())
            # Обзязательно нужно возвращать точку.
            return point
        return QtGui.QGraphicsItem.itemChange(self, change, value)

    def boundingRect(self):
        element_note = self.element.input_parameters().note().value()
        # Определение размеров имени элемента.
        fm = QtGui.QFontMetricsF(self.font)
        self.height = fm.height()
        if element_note:
            self.height *= 1 + len(element_note.split('\n'))
        width_name = fm.width(self.element.name())
        width_note = fm.width(element_note)
        self.width = max(width_name, width_note) + 4.5
        # Создание охватывающей рамки.
        rect = QtCore.QRectF(self.point, QtCore.QSizeF(self.width, self.height))
        return rect

    def paint(self, painter, options, widget=None):
        name_and_note = u'%s\n%s' % (self.element.name(), self.element.input_parameters().note().value())
        painter.setFont(self.font)
        if self.isSelected():
            # Если имя выделено, то оно подсвечивается другим цветом.
            painter.setPen(QtCore.Qt.blue)
        painter.drawText(self.boundingRect(), QtCore.Qt.AlignLeft, name_and_note)


class ElementView(QtGui.QGraphicsItem):
    """ Класс описывающий изображение элемента или узла на графической сцене.
    """

    def __init__(self, element, draw_element, parent=None, scene=None):
        QtGui.QGraphicsItem.__init__(self, parent, scene)
        self.draw_element = DrawElement()   # Объект с графическими параметрами элемента (класс DrawElement)
        self.draw_element.root = draw_element
        self.element = element  # Элемент - элемент или узел
        # Высота и ширина объекта.
        self.width = 15
        self.height = 15
        # Координаты начальной и конечной точек (относительные, т.е. координаты в самом объекте)
        self.begin_x = 0.
        self.begin_y = self.height / 2
        self.end_x = self.width
        self.end_y = self.height / 2
        self.end_x_2 = self.end_x
        self.end_y_2 = self.end_y
        # Координаты положения имени элемента относительно элемента.
        self.name_position_x = 0.
        self.name_position_y = 0.

        self.update_view()

    def boundingRect(self):
        # Получение графических параметров элемента (ширина, высота, координаты начальной и конечной точек).
        geometry = self.draw_element.element_geometry(self.element.type())
        self.width = geometry[0]
        self.height = geometry[1]
        self.begin_x = geometry[2]
        self.begin_y = geometry[3]
        self.end_x = geometry[4]
        self.end_y = geometry[5]
        self.end_x_2 = geometry[6]
        self.end_y_2 = geometry[7]
        self.name_position_x = geometry[8]
        self.name_position_y = geometry[9]
        # Создание окружающей рамки.
        rect = QtCore.QRectF(0, 0, self.width, self.height)
        return rect

    def paint(self, painter, option, widget=None):
        self.boundingRect()
        if self.isSelected():
            # Если элемент выделен, то он подсвечивается другим цветом.
            painter.setPen(QtCore.Qt.blue)
        # Отрисовка элемента на графической сцене с помощью объекта (класс DrawElement).
        self.draw_element.draw_element(painter=painter, element_type=self.element.type())

    def update_view(self):
        # Обновление изображения элемента.
        view = self.element.gui_parameters().view().value()
        status = 0
        if view in prog_settings.VETVI_VIEW_SWITCHERS:
            # Если элемент входит в список коммутационных аппаратов, то определяется
            # статус элемента (включен / выключен) по активному и реактивному сопротивлениям.
            input_parameters = self.element.input_parameters()
            active_resistance = input_parameters.active_resistance().value()
            reactive_resistance = input_parameters.reactive_resistance().value()
            if active_resistance >= 0. and reactive_resistance >= 0.:
                status = 1
        #self.prepareGeometryChange()
        # Создание объекта с графическими параметрами элемента.
        self.draw_element.read_element_view(self.element, status)
        self.boundingRect()


class ElementGroup(QtGui.QGraphicsItemGroup):
    """ Класс объединяющий изображение элемента или узла и точки обозначающие начало и конец элемента или узла.
    """

    def __init__(self, element, draw_element, parent=None, scene=None):
        QtGui.QGraphicsItemGroup.__init__(self, parent, scene)
        self.element = element  # Элемент или узел
        # Создание объекта, описывающего графическое изображение элемента.
        self.view = ElementView(element, draw_element)
        # Создание объектов начальной и конечной точек.
        self.begin_point = Point()
        self.end_point_1 = Point()
        self.end_point_2 = Point()
        # Объединение созданных элеметров в группу.
        self.addToGroup(self.view)
        self.addToGroup(self.begin_point)
        self.addToGroup(self.end_point_1)
        self.addToGroup(self.end_point_2)
        # Обновление параметров группы.
        self.update_group()

    def update_group(self):
        self.setRotation(0)
        self.setPos(0, 0)
        # Разгруппировка элементов.
        self.removeFromGroup(self.view)
        self.removeFromGroup(self.begin_point)
        self.removeFromGroup(self.end_point_1)
        self.removeFromGroup(self.end_point_2)
        # Обновление изображения элемента
        self.view.update_view()
        # Установка позиций начальной и конечной точек.
        self.begin_point.setPos(self.view.begin_x, self.view.begin_y)
        self.end_point_1.setPos(self.view.end_x, self.view.end_y)
        self.end_point_2.setPos(self.view.end_x_2, self.view.end_y_2)
        # Объединение объектов в группу.
        self.addToGroup(self.view)
        self.addToGroup(self.begin_point)
        self.addToGroup(self.end_point_1)
        self.addToGroup(self.end_point_2)
        # Установка поворота группы.
        gui_parameters = self.element.gui_parameters()
        rotation = gui_parameters.rotation().value()
        self.setRotation(rotation)
        # Установка отражения группы. Не работает.
        mirror_x = gui_parameters.mirror_x().value()
        mirror_y = gui_parameters.mirror_y().value()
        scale_x = self.transform().m11()
        scale_y = self.transform().m22()
        if (scale_x > 0 and mirror_x) or (scale_x < 0 and not mirror_x):
            set_scale_x = -1.
        else:
            set_scale_x = 1.
        if (scale_y > 0 and mirror_y) or (scale_y < 0 and not mirror_y):
            set_scale_y = -1.
        else:
            set_scale_y = 1.
        self.scale(set_scale_x, set_scale_y)


class ViewGroup(QtGui.QGraphicsItemGroup):
    """ Класс объединяющий изображение элемента или узла и имя элемента или узла.
    """

    def __init__(self, element, tree_properties, node_scheme, draw_element, parent=None, scene=None):
        QtGui.QGraphicsItemGroup.__init__(self, parent, scene)
        self.element = element  # Элемент или узел
        self.setFlag(QtGui.QGraphicsItem.ItemIsMovable)
        self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
        self.setFlag(QtGui.QGraphicsItem.ItemSendsGeometryChanges)
        self.node_scheme = node_scheme  # Объект схемы
        self.tree_properties = tree_properties  # Объект дерева параметров
        self.connection_lines = []  # Список содержащий соединительные линии
        # Переменная, определяющая сгруппированы ли графическое изображение элемента и его имя.
        self.name_in_group = True
        # Создание объекта графического изображения элемента с начальной и конечной точками.
        self.element_group = ElementGroup(element, draw_element)
        # Создание объекта графического изображения имени элемента.
        self.view_name = ElementName(element)
        # Объединение созданных объектов в группу.
        self.addToGroup(self.element_group)
        self.addToGroup(self.view_name)
        # Обновление изображения группы.
        self.update_group()

    def shape(self):
        path = QtGui.QPainterPath()
        path.addRect(self.element_group.boundingRect())
        return path

    def check_position(self):
        pass

    def name(self):
        return 'ViewGroup'

    def mousePressEvent(self, event):
        # При нажатии левой кнопкой мыши на изображении происходит обновление дерева параметров.
        button = event.button()
        if button == QtCore.Qt.LeftButton:
            QtGui.QGraphicsItem.mousePressEvent(self, event)
            self.update_properties_tree()

    def name_move_begin(self):
        # Отсоединение изображения имени от изображения элемента для его перемещения.
        self.removeFromGroup(self.view_name)
        self.name_in_group = False

    def name_move_end(self):
        # Присоединение изображения имени к изображению элемента, чтобы они перемещались вместе.
        self.addToGroup(self.view_name)
        self.name_in_group = True

    def switcher_status_on(self):
        if self.element.gui_parameters().view().value() in prog_settings.VETVI_VIEW_SWITCHERS:
            # Если элемент входит в список коммутационных аппаратов, то эта функция включает его и
            # устанавливает активное и реактивное сопротивления из буфера (группа параметров buffer).
            input_parameters = self.element.input_parameters()
            buffer_parameters = self.element.buffer_parameters()
            active_resistance = input_parameters.active_resistance()
            reactive_resistance = input_parameters.reactive_resistance()
            if active_resistance.value() < 0. or reactive_resistance.value() < 0.:
                active_resistance.set_value(buffer_parameters.active_resistance().value())
                reactive_resistance.set_value(buffer_parameters.reactive_resistance().value())
        self.update_group()
        self.update_properties_tree()

    def switcher_status_off(self):
        if self.element.gui_parameters().view().value() in prog_settings.VETVI_VIEW_SWITCHERS:
            # Если элемент входит в список коммутационных аппаратов, то эта функция отключает его и
            # устанавливает активное и реактивное сопротивления равными '-1', а старые параметры записывает в буфер.
            input_parameters = self.element.input_parameters()
            buffer_parameters = self.element.buffer_parameters()
            active_resistance = input_parameters.active_resistance()
            reactive_resistance = input_parameters.reactive_resistance()
            # Сохранение сопротивлений в буфер, если сопротивления больше 0.
            if active_resistance.value() >= 0. and reactive_resistance.value() >= 0.:
                buffer_parameters.active_resistance().set_value(active_resistance.value())
                buffer_parameters.reactive_resistance().set_value(reactive_resistance.value())
            active_resistance.set_value(-1.)
            reactive_resistance.set_value(-1.)
        self.update_group()
        self.update_properties_tree()

    def update_properties_tree(self):
        # Обновление дерева параметров.
        properties_model = models.PropertiesModel(self, self.node_scheme, self.tree_properties)
        self.tree_properties.setModel(properties_model)
        self.tree_properties.expandAll()
        # Установка ширин столбцов дерева параметров.
        self.tree_properties.setColumnWidth(0, self.node_scheme.settings.tree_properties_column_widths[0])
        self.tree_properties.setColumnWidth(1, self.node_scheme.settings.tree_properties_column_widths[1])

    def itemChange(self, change, value):
        if change == QtGui.QGraphicsItem.ItemPositionChange:
            # При изменении положения элемента происходит обновление параметров элемента и
            # обновление соединительных линий.
            point = value.toPointF()
            self.set_position()
            for connection_line in self.connection_lines:
                connection_line.adjust()
            # Возвращать точку нужно обязательно.
            return point
        return QtGui.QGraphicsItem.itemChange(self, change, value)

    def set_position(self):
        # Обновление параметров (координаты положения элемента и положения имени элемента).
        position_x = self.x() + self.element_group.x()
        position_y = self.y() + self.element_group.y()
        gui_parameters = self.element.gui_parameters()
        gui_parameters.position_x().set_value(position_x)
        gui_parameters.position_y().set_value(position_y)
        if self.name_in_group:
            # Если изображение имени присоединено к изображению элемента, то обновляются и его координаты.
            # Координаты положения имени элемента задаются в координатах графической сцены,
            # а не относительно положения самого элемента.
            name_position_x = self.x() + self.view_name.x()
            name_position_y = self.y() + self.view_name.y()
            gui_parameters.name_position_x().set_value(name_position_x)
            gui_parameters.name_position_y().set_value(name_position_y)

    def update_position(self):
        # Обновление положения изображения элемента и изображения имени на графической сцене при изменении
        # параметров элемента.
        gui_parameters = self.element.gui_parameters()

        if 'position_x' in gui_parameters.children_list():
            position_x = gui_parameters.position_x().value()
            self.element_group.setX(position_x)
        if 'position_y' in gui_parameters.children_list():
            position_y = gui_parameters.position_y().value()
            self.element_group.setY(position_y)

        if 'name_position_x' in gui_parameters.children_list():
            self.view_name.setX(gui_parameters.name_position_x().value())
        else:
            self.view_name.setX(gui_parameters.position_x().value() + self.element_group.view.name_position_x)
        if 'name_position_y' in gui_parameters.children_list():
            self.view_name.setY(gui_parameters.name_position_y().value())
        else:
            self.view_name.setY(gui_parameters.position_y().value() + self.element_group.view.name_position_y)

    def update_group(self):
        # Обновление изображения элемента и изображения имени.
        # Обновление группы - изображение элемента и точки начала и конца.
        self.element_group.update_group()
        # Разгруппировка изображения элемента и изображения имени.
        self.removeFromGroup(self.element_group)
        self.removeFromGroup(self.view_name)
        # Обновление положения объектов.
        self.update_position()
        # Группировка объектов.
        self.addToGroup(self.element_group)
        self.addToGroup(self.view_name)
        # Обновление параметров (координат положения) элемента.
        self.set_position()
        # Обновление соединительных линий.
        for connection_line in self.connection_lines:
            connection_line.adjust()

        self.name_in_group = True


class ConnectionLine(QtGui.QGraphicsItem):
    # Класс, описывающий соединительную линию между началом или концом элемента и узлом.

    def __init__(self, line_type, vetv, usel, parent=None, scene=None):
        QtGui.QGraphicsItem.__init__(self, parent, scene)
        # Объекты двух точек линии.
        self.source_point = QtCore.QPointF()
        self.dest_point = QtCore.QPointF()
        # Тип линии:
        # begin - если линия соединяет начало элемента с узлом,
        # end - если линия соединяет конец элемента с узлом.
        self.line_type = line_type

        self.setAcceptedMouseButtons(QtCore.Qt.NoButton)
        # Объекты элемента и узла.
        self.vetv = vetv
        self.usel = usel
        # Добавление данной линии к спискам соединительных линий элемента и узла.
        self.vetv.connection_lines.append(self)
        self.usel.connection_lines.append(self)
        # Обновление линии.
        self.adjust()

    def adjust(self):
        # Обновление линии.
        if not self.vetv or not self.usel:
            return

        if self.line_type == 'begin':
            line = QtCore.QLineF(self.mapFromItem(self.vetv.element_group.begin_point, 0, 0),
                                 self.mapFromItem(self.usel.element_group.begin_point, 0, 0))
        elif self.line_type == 'end_1':
            line = QtCore.QLineF(self.mapFromItem(self.vetv.element_group.end_point_1, 0, 0),
                                 self.mapFromItem(self.usel.element_group.end_point_1, 0, 0))
        elif self.line_type == 'end_2':
            line = QtCore.QLineF(self.mapFromItem(self.vetv.element_group.end_point_2, 0, 0),
                                 self.mapFromItem(self.usel.element_group.end_point_2, 0, 0))

        self.prepareGeometryChange()

        self.source_point = line.p1()
        self.dest_point = line.p2()

    def boundingRect(self):
        # Создание области ограничивающей линию.
        if not self.vetv or not self.usel:
            return QtCore.QRectF()

        return QtCore.QRectF(self.source_point, self.dest_point)

    def paint(self, painter, option, widget=None):
        if not self.vetv or not self.usel:
            return

        # Промежуточная точка.
        interim_point = QtCore.QPointF(self.source_point.x(), self.dest_point.y())
        if self.line_type == 'begin':
            if self.vetv.element.gui_parameters().rotate_line_1().value():
                interim_point = QtCore.QPointF(self.dest_point.x(), self.source_point.y())
        if self.line_type == 'end_1':
            if self.vetv.element.gui_parameters().rotate_line_2().value():
                interim_point = QtCore.QPointF(self.dest_point.x(), self.source_point.y())
        if self.line_type == 'end_2':
            if self.vetv.element.gui_parameters().rotate_line_3().value():
                interim_point = QtCore.QPointF(self.dest_point.x(), self.source_point.y())
        # Соединительная линия, состаящая из двух линий.
        line_1 = QtCore.QLineF(self.source_point, interim_point)
        line_2 = QtCore.QLineF(interim_point, self.dest_point)

        if line_1.length() == 0.0 and line_2.length() == 0.0:
            return

        if line_1.length():
            painter.drawLine(line_1)

        if line_2.length():
            painter.drawLine(line_2)


class SchemeView(QtGui.QGraphicsScene):

    def __init__(self, tree_properties, status_bar, new_elements_obj, module_path, node_volt=None, parent=None):
        QtGui.QGraphicsScene.__init__(self, parent)
        self.parent = parent
        self.node_volt = node_volt
        self.module_path = module_path
        self.new_elements_obj = new_elements_obj
        self.status_bar = status_bar
        self.tree_properties = tree_properties  # Объект дерева параметров.
        # Задание области графической сцены.
        self.setSceneRect(QtCore.QRectF(0, 0, 400, 400))
        self.addRect(QtCore.QRectF(0, 0, 400, 400))
        # При изменении выбора элемента на графической сцене вызывается соответствующая функция.
        self.selectionChanged.connect(self.selection_changed)

        self.node_scheme = None

        self.draw_vetv = None
        self.draw_usel = None
        # Списки изображений узлов, элементов и соединительных линий.
        self.vetvi_views = []
        self.usli_views = []
        self.connection_lines = []
        # Переменные, определяющие, были ли нажаты пункты контекстного меню "Выбрать начальный узел" или
        # "Выбрать конечный узле".
        self.select_begin_on = False
        self.select_end_on = False
        self.select_end_2_on = False

        self.shortcut_select_begin = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+1"), self.parent)
        self.shortcut_select_begin.activated.connect(self.select_begin)
        self.shortcut_select_end = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+2"), self.parent)
        self.shortcut_select_end.activated.connect(self.select_end)
        self.shortcut_select_end_2 = QtGui.QShortcut(QtGui.QKeySequence("Ctrl+3"), self.parent)
        self.shortcut_select_end_2.activated.connect(lambda: self.select_end(end_2=True))

    def select_all(self):
        # Выделение всех элементов на графической сцене.
        for item in self.items():
            item.setSelected(True)

    def selection_changed(self):
        # Функция вызываемая при изменении выделения элемента на графической сцене.
        # Обнуляется модель дерева параметров. Дерево параметров заполняется в соответствующей функции в описании
        # класса ViewGroup.
        # Если ранее был выбран пункт контектсного меню "Выбрать начальный узел" или "Выбрать конечный узел" и сейчас
        # выбран узел, то происходит присвоение имени этого узла соответветсвтвующему параметру ранее выбранного
        # элемента.
        self.parent.unsetCursor()
        selected_item = self.selectedItems()
        if not self.select_begin_on and not self.select_end_on and not self.select_end_2_on:
            self.tree_properties.setModel(None)
            return
        elif len(selected_item) == 0:
            self.select_begin_on = False
            self.select_end_on = False
            self.select_end_2_on = False
            self.status_bar.clearMessage()
            self.tree_properties.setModel(None)
            return
        elif selected_item[0].element.type() == 'usel':
            if self.select_begin_on and not self.select_end_on and not self.select_end_2_on:
                self.select_begin_on.element.input_parameters().begin().set_value(selected_item[0].element.name())
            elif self.select_end_on and not self.select_end_2_on and not self.select_begin_on:
                self.select_end_on.element.input_parameters().end().set_value(selected_item[0].element.name())
            elif self.select_end_2_on and not self.select_end_on and not self.select_begin_on:
                self.select_end_2_on.element.input_parameters().end_2().set_value(selected_item[0].element.name())
        self.select_begin_on = False
        self.select_end_on = False
        self.select_end_2_on = False
        self.status_bar.clearMessage()
        self.update_connection_lines()
        return

    def key_esc(self):
        # Функция вызываемая при нажатии кнопки Esc.
        self.select_begin_on = False
        self.select_end_on = False
        self.select_end_2_on = False
        self.clearSelection()
        self.status_bar.clearMessage()
        self.parent.unsetCursor()

    def context_menu_select_usel(self, context_menu, vetv):
        lang_menu = language.Other()
        # Поиск элемента в объекте добавления новых элементов.
        # Если такой элемент будет найден, то проверяется заданы ли у него значения для узлов изначально.
        new_element_name = vetv.input_parameters().new_element_name().value()
        is_begin_set = False
        is_end_set = False
        is_end_2_set = False
        if new_element_name:
            new_element_name_path = new_element_name.split('/')
            if len(new_element_name_path) == 2:
                new_element_group_obj = self.new_elements_obj.find_child(new_element_name_path[0])
                if new_element_group_obj:
                    new_element_obj = new_element_group_obj.find_child(new_element_name_path[1])
                    if new_element_obj:
                        is_begin_set = new_element_obj.find_child('begin')
                        is_end_set = new_element_obj.find_child('end')
                        is_end_2_set = new_element_obj.find_child('end_2')
        if not is_begin_set:
            # Создание меню выбора начального узла.
            select_begin_menu = QtGui.QMenu(context_menu)
            select_begin_menu.setTitle(lang_menu.graphics_view_context_menu['select_begin'])

            select_begin_action = QtGui.QAction(u'%s (Ctrl+1)' % lang_menu.graphics_view_context_menu['select'], self)
            select_begin_menu.addAction(select_begin_action)
            select_begin_menu.connect(select_begin_action, QtCore.SIGNAL('triggered()'), lambda: self.select_begin())

            select_begin_0 = QtGui.QAction(u'0', self)
            select_begin_menu.addAction(select_begin_0)
            select_begin_menu.connect(select_begin_0, QtCore.SIGNAL('triggered()'), lambda: self.select_begin('0'))

            # select_begin_menu.addSeparator()

            # if len(self.node_scheme.usli.children_list()) <= 15:
            #     select_begin_usli = []
            #     i = 0
            #     for usel in self.node_scheme.usli.children_list():
            #         select_begin_usli.append(QtGui.QAction(usel, self))
            #         select_begin_menu.addAction(select_begin_usli[i])
            #         select_begin_menu.connect(select_begin_usli[i], QtCore.SIGNAL('triggered()'),
            #                                   lambda: self.select_begin(usel))
            #         i += 1

            context_menu.addMenu(select_begin_menu)
        if not is_end_set:
            # Создание меню выбора конечного узла 1.
            select_end_menu = QtGui.QMenu(context_menu)
            select_end_menu.setTitle(lang_menu.graphics_view_context_menu['select_end'])

            select_end_action = QtGui.QAction(u'%s (Ctrl+2)' % lang_menu.graphics_view_context_menu['select'], self)
            select_end_menu.addAction(select_end_action)
            select_end_menu.connect(select_end_action, QtCore.SIGNAL('triggered()'), lambda: self.select_end())

            select_end_0 = QtGui.QAction(u'0', self)
            select_end_menu.addAction(select_end_0)
            select_end_menu.connect(select_end_0, QtCore.SIGNAL('triggered()'), lambda: self.select_end('0'))

            # select_end_menu.addSeparator()
            #
            # if len(self.node_scheme.usli.children_list()) <= 15:
            #     select_end_usli = []
            #     i = 0
            #     for usel in self.node_scheme.usli.children_list():
            #         select_end_usli.append(QtGui.QAction(usel, self))
            #         select_end_menu.addAction(select_end_usli[i])
            #         select_end_menu.connect(select_end_usli[i], QtCore.SIGNAL('triggered()'),
            #                                 lambda: self.select_end(usel))
            #         i += 1
            context_menu.addMenu(select_end_menu)
        if vetv.gui_parameters().view().value() in prog_settings.VETVI_VIEW_3_LINES and not is_end_2_set:
            # Создание меню выбора конечного узла 2.
            select_end_2_menu = QtGui.QMenu(context_menu)
            select_end_2_menu.setTitle(lang_menu.graphics_view_context_menu['select_end_2'])

            select_end_2_action = QtGui.QAction(u'%s (Ctrl+3)' % lang_menu.graphics_view_context_menu['select'], self)
            select_end_2_menu.addAction(select_end_2_action)
            select_end_2_menu.connect(select_end_2_action, QtCore.SIGNAL('triggered()'),
                                      lambda: self.select_end(end_2=True))

            select_end_2_0 = QtGui.QAction(u'0', self)
            select_end_2_menu.addAction(select_end_2_0)
            select_end_2_menu.connect(select_end_2_0, QtCore.SIGNAL('triggered()'), lambda: self.select_end('0', True))

            context_menu.addMenu(select_end_2_menu)

    def context_menu_on_off(self, context_menu, selected_items):
        lang_menu = language.Other()
        vetv = selected_items[0].element
        input_parameters = vetv.input_parameters()
        if input_parameters.active_resistance().value() < 0. or input_parameters.reactive_resistance().value() < 0.:
            # Добавление пункта контекстного меню включения коммутационного аппарата.
            on_action = QtGui.QAction(lang_menu.graphics_view_context_menu['on'], self)
            context_menu.addAction(on_action)
            context_menu.connect(on_action, QtCore.SIGNAL('triggered()'),
                                 selected_items[0].switcher_status_on)
        else:
            # Добавление пункта контекстного меню выключения коммутационного аппарата.
            off_action = QtGui.QAction(lang_menu.graphics_view_context_menu['off'], self)
            context_menu.addAction(off_action)
            context_menu.connect(off_action, QtCore.SIGNAL('triggered()'),
                                 selected_items[0].switcher_status_off)

    def context_menu_rotate_conn_lines(self, context_menu, vetv):
        # Создание пунктов меню поворота соединительных линий.
        lang_menu = language.Other()
        input_parameters = vetv.input_parameters()

        if input_parameters.begin().value() != '0':
            rotate_line1_action = QtGui.QAction(lang_menu.graphics_view_context_menu['rotate_connection_line_1'], self)
            context_menu.addAction(rotate_line1_action)
            context_menu.connect(rotate_line1_action, QtCore.SIGNAL('triggered()'), vetv.rotate_connection_line_1)
        if input_parameters.end().value() != '0':
            rotate_line2_action = QtGui.QAction(lang_menu.graphics_view_context_menu['rotate_connection_line_2'], self)
            context_menu.addAction(rotate_line2_action)
            context_menu.connect(rotate_line2_action, QtCore.SIGNAL('triggered()'), vetv.rotate_connection_line_2)

        if vetv.gui_parameters().view().value() in prog_settings.VETVI_VIEW_3_LINES:
            if input_parameters.end_2().value() != '0':
                rotate_line3_action = QtGui.QAction(lang_menu.graphics_view_context_menu['rotate_connection_line_3'],
                                                    self)
                context_menu.addAction(rotate_line3_action)
                context_menu.connect(rotate_line3_action, QtCore.SIGNAL('triggered()'), vetv.rotate_connection_line_3)

    def contextMenuEvent(self, event):
        # Функция описывает появление контекстного меню при нажатии правой кнопки мыши.
        lang_menu = language.Other()
        context_menu = QtGui.QMenu()
        selected_items = self.selectedItems()
        if selected_items:
            if len(selected_items) == 1:
                # Контекстное меню появляется если элемент, описанный классом ViewGroup.
                if selected_items[0].name() != 'ViewGroup':
                    return
                if selected_items[0].element.gui_parameters().view().value() in prog_settings.VETVI_VIEW_SWITCHERS:
                    # Если элемент входит в список коммутационных аппаратов, то в контекстное меню добавляются
                    # пункты включить и выключить. При нажатии на этих пунктах вызываются соответствующие функции,
                    # которые описаны в классе ViewGroup.
                    self.context_menu_on_off(context_menu, selected_items)
                    context_menu.addSeparator()

                if selected_items[0].element.type() == 'vetv':
                    # Если тип выбранного элемента - ветвь, то в контекстное меню добавляются пункты выбора
                    # начального и конечного узлов
                    # и пункт поворота соединительных линий.
                    self.context_menu_select_usel(context_menu, selected_items[0].element)
                    context_menu.addSeparator()
                    self.context_menu_rotate_conn_lines(context_menu, selected_items[0].element)
                    context_menu.addSeparator()

                if selected_items[0].name_in_group:
                    # Если изображение имени сгруппировано с изображением элемента, то к контекстному меню добавляется
                    # пункт, при нажатии на который вызывается функция (класс ViewGroup), отсоединяющая изображение
                    # имени от изображения элемента для его перемещения.
                    name_move_begin_action = QtGui.QAction(lang_menu.graphics_view_context_menu['name_move_begin'],
                                                           self)
                    context_menu.addAction(name_move_begin_action)
                    context_menu.connect(name_move_begin_action, QtCore.SIGNAL('triggered()'),
                                         selected_items[0].name_move_begin)
                else:
                    # Если изображение имени не сгруппировано с изображением элемента, то к контекстному меню
                    # добавляется пункт, при нажатии на который вызывается функция (класс ViewGroup), присоединяющая
                    # изображение имени к изображению элемента, чтобы они перемещались вместе.
                    name_move_end_action = QtGui.QAction(lang_menu.graphics_view_context_menu['name_move_end'], self)
                    context_menu.addAction(name_move_end_action)
                    context_menu.connect(name_move_end_action, QtCore.SIGNAL('triggered()'),
                                         selected_items[0].name_move_end)

                context_menu.addSeparator()
            # Добавление к контекстному меню пунктов "Удалить", "Копировать".
            # Действия можно выполнять сразу над несколькими элементами.
            remove_action = QtGui.QAction(lang_menu.graphics_view_context_menu['delete'], self)
            if self.node_volt is not None:
                copy_action = QtGui.QAction(lang_menu.graphics_view_context_menu['copy'], self)
                context_menu.addAction(copy_action)
                context_menu.connect(copy_action, QtCore.SIGNAL('triggered()'), self.node_volt.copy)
            context_menu.addAction(remove_action)
            context_menu.connect(remove_action, QtCore.SIGNAL('triggered()'), self.remove_selected_elements)
        else:
            # Если не выбрано ни одного элемента, то в контекстное меню включается только пункт "Вставить".
            if self.node_volt is not None:
                if self.node_volt.vetvi_copied or self.node_volt.usli_copied:
                    past_action = QtGui.QAction(lang_menu.graphics_view_context_menu['past'], self)
                    context_menu.addAction(past_action)
                    context_menu.connect(past_action, QtCore.SIGNAL('triggered()'), self.node_volt.past)
            select_all_action = QtGui.QAction(lang_menu.graphics_view_context_menu['select_all'], self)
            context_menu.addAction(select_all_action)
            context_menu.connect(select_all_action, QtCore.SIGNAL('triggered()'), self.select_all)
        context_menu.exec_(QtGui.QCursor.pos())

    def mousePressEvent(self, event):
        if event.button() != QtCore.Qt.LeftButton:
            event.accept()
            return
        QtGui.QGraphicsScene.mousePressEvent(self, event)

    def select_begin(self, usel=''):
        # Функция, вызываемая при выборе пункта контектсного меню элемента "Выбрать начальный узел"
        selected_items = self.selectedItems()
        if len(selected_items) == 1:
            if selected_items[0].element.type() == 'vetv':
                if not usel:
                    self.parent.setCursor(QtCore.Qt.CrossCursor)
                    self.select_begin_on = selected_items[0]
                    self.select_end_on = False
                    self.select_end_2_on = False
                    self.status_bar.showMessage(language.Other().graphics_view_context_menu['select_begin'], 0)
                    return
                if usel == u'0':
                    selected_items[0].element.input_parameters().begin().set_value('0')
                    selected_items[0].update_properties_tree()
                elif usel in self.node_scheme.usli.children_list():
                    selected_items[0].element.input_parameters().begin().set_value(usel)

                self.update_connection_lines()
        self.select_begin_on = False
        self.select_end_on = False
        self.select_end_2_on = False
        self.status_bar.clearMessage()

    def select_end(self, usel='', end_2=False):
        # Функция, вызываемая при выборе пункта контектсного меню элемента "Выбрать конечный узел".
        selected_items = self.selectedItems()
        if len(selected_items) == 1:
            if selected_items[0].element.type() == 'vetv':
                if not usel:
                    self.parent.setCursor(QtCore.Qt.CrossCursor)
                    self.select_begin_on = False
                    if not end_2:
                        self.select_end_on = selected_items[0]
                        self.select_end_2_on = False
                        self.status_bar.showMessage(language.Other().graphics_view_context_menu['select_end'], 0)
                    else:
                        self.select_end_2_on = selected_items[0]
                        self.select_end_on = False
                        self.status_bar.showMessage(language.Other().graphics_view_context_menu['select_end_2'], 0)
                    return
                if not end_2:
                    end = selected_items[0].element.input_parameters().end()
                else:
                    end = selected_items[0].element.input_parameters().end_2()
                if usel == u'0':
                    end.set_value('0')
                    selected_items[0].update_properties_tree()
                elif usel in self.node_scheme.usli.children_list():
                    end.set_value(usel)

                self.update_connection_lines()
        self.select_begin_on = False
        self.select_end_on = False
        self.select_end_2_on = False
        self.status_bar.clearMessage()

    def add_new_element(self, vetv, first_time=False):
        # Добавление элемента к графической сцене.
        vetv_view = ViewGroup(vetv, self.tree_properties, self.node_scheme, self.draw_vetv, None, self)
        self.vetvi_views.append(vetv_view)
        if not first_time:
            # Если графическая сцена создается не в первый раз, то происходит обновление соединительных линий.
            self.update_connection_lines()

    def add_new_usel(self, usel, first_time=False):
        # Добавление узла к графической сцене.
        usel_view = ViewGroup(usel, self.tree_properties, self.node_scheme, self.draw_usel, None, self)
        self.usli_views.append(usel_view)
        if not first_time:
            # Если графическая сцена создается не в первый раз, то происходит обновление соединительных линий.
            self.update_connection_lines()

    def remove_selected_elements(self):
        # Удаление элементов.
        selected_elements = self.selectedItems()
        if selected_elements and self.node_scheme:
                # Проход по списку выделенных элементов.
                for selected_element in selected_elements:
                    if selected_element.name() == 'ViewGroup':
                        # Удаление элемента.
                        if selected_element.element.type() == 'vetv':
                            # Удаление элемента из списка элементов схемы.
                            position = self.node_scheme.vetvi.find_child_position(selected_element.element.name())
                            self.node_scheme.vetvi.remove_child(position)
                            # Удаление элемента с графической сцены.
                            if selected_element in self.vetvi_views:
                                del self.vetvi_views[self.vetvi_views.index(selected_element)]
                        # Удаление узла.
                        elif selected_element.element.type() == 'usel':
                            # Удаление узла из списка узлов схемы.
                            position = self.node_scheme.usli.find_child_position(selected_element.element.name())
                            self.node_scheme.usli.remove_child(position)
                            # Удаление узла из элементов.
                            self.node_scheme.rename_usel_in_vetvi(selected_element.element.name(), u'')
                            # Удаление узла с графической сцены.
                            if selected_element in self.usli_views:
                                del self.usli_views[self.usli_views.index(selected_element)]
                        # Обновление графической сцены схемы.
                        if not selected_element.name_in_group:
                            self.removeItem(selected_element.view_name)
                        self.removeItem(selected_element)
                # Обновление соединительных линий.
                self.update_connection_lines()
                # Обнуление модели дерева параметров.
                self.tree_properties.setModel(None)

    def set_scheme_model(self, node_scheme=None):
        self.draw_vetv = read_graphics_elements_file('vetv', self.module_path)
        self.draw_usel = read_graphics_elements_file('usel', self.module_path)
        # Создание новой графической сцены.
        self.key_esc()
        self.node_scheme = node_scheme
        # Удаление всех элементов на графической сцене.
        self.clear()
        self.vetvi_views = []
        self.usli_views = []
        self.connection_lines = []
        # Создание новых элементов графической сцены.
        if self.node_scheme is not None:
            # Добавление элементов на графическую сцену.
            for vetv in self.node_scheme.vetvi.children_list():
                vetv = self.node_scheme.vetvi.find_child(vetv)
                self.add_new_element(vetv, True)
            # Добавление узлов на графическую сцену.
            for usel in self.node_scheme.usli.children_list():
                usel = self.node_scheme.usli.find_child(usel)
                self.add_new_usel(usel, True)
            # Обновление соединительных линий.
            self.update_connection_lines()

    def update_connection_lines(self):
        # Функция добавления на графическую сцену соединительных линий (соединяющих узлы и элементы цепи).
        # Удаление всех соединительных линий на графической сцене.
        for connection_line in self.connection_lines:
            self.removeItem(connection_line)
        self.connection_lines = []
        # Проход по списку узлов и очистка их списков соединительных линий.
        for usel_view in self.usli_views:
            usel_view.connection_lines = []
        # Проход по списку элементов и очистка их списков соединительных линий.
        for vetv_view in self.vetvi_views:
            vetv_view.connection_lines = []
            # Создание линии, соединяющей начало элемента с определенным узлом.
            vetv_begin = vetv_view.element.input_parameters().begin().value()
            if vetv_begin:
                if vetv_begin != '0':
                    for usel_view in self.usli_views:
                        if usel_view.element.name() == vetv_begin:
                            self.connection_lines.append(ConnectionLine('begin', vetv_view, usel_view, None, self))
            # Создание линии, соединяющей первый конец элемента с определенным узлом.
            vetv_end = vetv_view.element.input_parameters().end().value()
            if vetv_end:
                if vetv_end != '0':
                    for usel_view in self.usli_views:
                        if usel_view.element.name() == vetv_end:
                            self.connection_lines.append(ConnectionLine('end_1', vetv_view, usel_view, None, self))
            # Создание линии, соединяющей второй конец элемента с определенным узлом.
            vetv_end_2 = vetv_view.element.input_parameters().end_2().value()
            if vetv_end_2:
                if vetv_end_2 != '0':
                    for usel_view in self.usli_views:
                        if usel_view.element.name() == vetv_end_2:
                            self.connection_lines.append(ConnectionLine('end_2', vetv_view, usel_view, None, self))
