# -*- coding: utf-8 -*-

from PyQt4 import QtCore, QtGui


class StyledGraphicsView(QtGui.QGraphicsView):

    def wheelEvent(self, event):
        delta = event.delta()
        scale_in = self.transform()
        delta_delta = 0.3
        if delta > 0:
            scale_out = QtGui.QTransform().fromScale(scale_in.m11() + delta_delta, scale_in.m22() + delta_delta)
            self.setTransform(scale_out)
        elif delta < 0:
            scale_out = QtGui.QTransform().fromScale(scale_in.m11() - delta_delta, scale_in.m22() - delta_delta)
            self.setTransform(scale_out)

    def mousePressEvent(self, event):
        button = event.button()
        if button == QtCore.Qt.LeftButton:
            self.setDragMode(QtGui.QGraphicsView.RubberBandDrag)
        elif button == QtCore.Qt.MiddleButton:
            self.setInteractive(False)
            self.setDragMode(QtGui.QGraphicsView.ScrollHandDrag)
            event = QtGui.QMouseEvent(QtCore.QEvent.MouseButtonPress, event.pos(), event.globalPos(),
                                      QtCore.Qt.LeftButton, event.buttons(), QtCore.Qt.NoModifier)
        QtGui.QGraphicsView.mousePressEvent(self, event)

    def mouseReleaseEvent(self, event):
        if event.button() == QtCore.Qt.MiddleButton:
            event = QtGui.QMouseEvent(QtCore.QEvent.MouseButtonRelease, event.pos(), event.globalPos(),
                                      QtCore.Qt.LeftButton, event.buttons(), QtCore.Qt.NoModifier)
            self.setInteractive(True)
        QtGui.QGraphicsView.mouseReleaseEvent(self, event)
        self.setDragMode(QtGui.QGraphicsView.RubberBandDrag)


class StyledTreeView(QtGui.QTreeView):

    def background_color(self, index):
        # Определение цвета фона строк дерева параметров.
        node = index.internalPointer()
        parent = node.parent()
        if not parent:
            return None
        if parent.name() == 'input_parameters' or node.name() == 'input_parameters':
            # Фон входных параметров.
            background_1 = '#FDFAEE'
            background_2 = '#FDFAEE'
        elif parent.name() == 'gui_parameters' or node.name() == 'gui_parameters':
            # Фон графических параметров.
            background_1 = '#FEF4F2'
            background_2 = '#FEF4F2'
        else:
            # Фон других параметров
            background_1 = '#F2EFF7'
            background_2 = '#F2EFF7'
        if index.row() % 2 != 0:
            # Фон четных строк.
            background = QtGui.QColor(background_2)
        else:
            # Фон нечетных строк.
            background = QtGui.QColor(background_1)
        return background

    def drawBranches(self, painter, rect, index):
        # Рисование отступов слева от параметров.
        background = self.background_color(index)
        # Заполнение фоном.
        painter.fillRect(rect, background)
        # Продолжение стандартной функции.
        QtGui.QTreeView.drawBranches(self, painter, rect, index)

    def drawRow(self, painter, style_option, index):
        # Рисование строк параметров.
        background = self.background_color(index)
        # Заполнение фоном.
        painter.fillRect(style_option.rect, background)
        # Изменение шрифта строки.
        style_option.font.setPointSize(8)
        style_option.palette.setColor(QtGui.QPalette.Text, QtGui.QColor('#404041'))
        # Продолжение стандартной функции.
        QtGui.QTreeView.drawRow(self, painter, style_option, index)
        # Рисование линии снизу.
        pen = QtGui.QPen()
        pen.setColor(QtGui.QColor('#58585B'))
        pen.setWidthF(0.5)
        painter.setPen(pen)
        painter.drawLine(style_option.rect.bottomLeft(), style_option.rect.bottomRight())


START_STYLE_SHEET = "QWidget {font-family: 'Open Sans Light';" \
                    "background-color: #EBE6E8;" \
                    "font-size: 12pt;}" \
                    "QPushButton {" \
                    "background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0," \
                    "stop:0 rgba(61, 56, 121, 255), stop:1 rgba(130, 56, 149, 255));" \
                    "color: rgb(255, 255, 255);" \
                    "border: 0px solid rgba(255, 255, 255, 77);}" \
                    "QPushButton::hover {" \
                    "background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 " \
                    "rgba(61, 56, 121, 200), stop:1 rgba(130, 56, 149, 200));}" \
                    "QPushButton::pressed {" \
                    "background-color: qlineargradient(spread:pad, x1:0, y1:1, x2:0, y2:0, stop:0 " \
                    "rgba(61, 56, 121, 100), stop:1 rgba(130, 56, 149, 100));}" \
                    "QListWidget {" \
                    "border: 0px solid rgba(255, 255, 255, 77);" \
                    "color: rgb(0, 0, 0);}" \
                    "QLabel {" \
                    "color: rgb(0, 0, 0);}"
MAIN_STYLE_SHEET = "QWidget {" \
                   "font-family: 'Open Sans Light';" \
                   "font-size: 9pt;}" \
                   "QScrollBar:horizontal {" \
                   "border: 0px;" \
                   "background: rgb(230, 231, 232);" \
                   "height: 7px;" \
                   "margin: 0px 20px 0px 20px;}" \
                   "QScrollBar::handle:horizontal {" \
                   "background: rgba(130, 56, 149, 255);" \
                   "min-width: 20px;}" \
                   "QScrollBar::add-line:horizontal {" \
                   "border: 0px;" \
                   "background: rgba(130, 56, 149, 255);" \
                   "width: 20px;" \
                   "height: 10px;" \
                   "subcontrol-position: right;" \
                   "subcontrol-origin: margin;}" \
                   "QScrollBar::sub-line:horizontal {" \
                   "border: 0px;" \
                   "background: rgba(130, 56, 149, 255);" \
                   "width: 20px;" \
                   "height: 10px;" \
                   "subcontrol-position: left;" \
                   "subcontrol-origin: margin;}" \
                   "QScrollBar:left-arrow:horizontal, QScrollBar::right-arrow:horizontal {" \
                   "border: 0px;" \
                   "width: 6px;" \
                   "height: 2px;" \
                   "background: rgb(230, 231, 232);}" \
                   "QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal {" \
                   "background: none;}" \
                   "QScrollBar:vertical {" \
                   "border: 0px;" \
                   "background: rgb(230, 231, 232);" \
                   "width: 7px;" \
                   "margin: 20px 0px 20px 0px;}" \
                   "QScrollBar::handle:vertical {" \
                   "background: rgba(130, 56, 149, 255);" \
                   "min-height: 20px;}" \
                   "QScrollBar::add-line:vertical {" \
                   "border: 0px;" \
                   "background: rgba(130, 56, 149, 255);" \
                   "width: 10px;" \
                   "height: 20px;" \
                   "subcontrol-position: bottom;" \
                   "subcontrol-origin: margin;}" \
                   "QScrollBar::sub-line:vertical {" \
                   "border: 0px;" \
                   "background: rgba(130, 56, 149, 255);" \
                   "width: 10px;" \
                   "height: 20px;" \
                   "subcontrol-position: top;" \
                   "subcontrol-origin: margin;}" \
                   "QScrollBar:up-arrow:vertical, QScrollBar::down-arrow:vertical {" \
                   "border: 0px;" \
                   "width: 2px;" \
                   "height: 6px;" \
                   "background: rgb(230, 231, 232);}" \
                   "QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical {" \
                   "background: none;}"
MAIN_MENU_FRAME_STYLE_SHEET = "QWidget {font-size: 10pt;}" \
                              "QFrame {" \
                              "background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, " \
                              "stop:0 rgba(61, 56, 121, 255), stop:1 rgba(130, 56, 149, 255));" \
                              "border: 0px;}"
MAIN_MENU_TAB_STYLE_SHEET = "QTabWidget::pane {" \
                            "border: 0px;}" \
                            "QTabBar::tab {" \
                            "color: rgb(255, 255, 255);" \
                            "width: 178px;" \
                            "height: 33px;" \
                            "border: 0px;}" \
                            "QTabBar::tab:selected, QTabBar::tab:hover {" \
                            "background: rgba(255, 255, 255, 51);}"
MAIN_MENU_FRAME_MAIN_STYLE_SHEET = "QFrame {" \
                                   "background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, " \
                                   "stop:0 rgba(216, 148, 152, 255), stop:1 rgba(145, 99, 161, 255));" \
                                   "border: 0px;}" \
                                   "QToolButton {" \
                                   "color: rgb(255, 255, 255);" \
                                   "border: 1px solid rgba(255, 255, 255, 77);" \
                                   "border-top: 0px;" \
                                   "border-left: 0px;" \
                                   "border-bottom: 0px;}" \
                                   "QToolButton::hover {" \
                                   "background-color: rgba(255, 255, 255, 90);}" \
                                   "QToolButton::pressed {" \
                                   "background-color: rgba(130, 56, 149, 90);}"
MAIN_MENU_FRAME_SETTINGS_STYLE_SHEET = "QFrame {" \
                                       "background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, " \
                                       "stop:0 rgba(216, 148, 152, 255), stop:1 rgba(145, 99, 161, 255));" \
                                       "border: 0px;}" \
                                       "QToolButton {" \
                                       "color: rgb(255, 255, 255);" \
                                       "border: 1px solid rgba(255, 255, 255, 77);" \
                                       "border-top: 0px;" \
                                       "border-left: 0px;" \
                                       "border-bottom: 0px;}" \
                                       "QToolButton::hover {" \
                                       "background-color: rgba(255, 255, 255, 90);}" \
                                       "QToolButton::pressed {" \
                                       "background-color: rgba(130, 56, 149, 90);}"
MAIN_FRAME_STYLE_SHEET = "QFrame {" \
                         "background-color: rgb(230, 231, 232);" \
                         "border: 0px;}"
MENU_TAB_STYLE_SHEET = "QTabWidget::pane {" \
                       "border: 0px;}" \
                       "QTabBar::tab {" \
                       "width: 155px;" \
                       "height: 32px;" \
                       "font-size: 10pt;" \
                       "color: #58585B;" \
                       "border: 0px;}" \
                       "QTabBar::tab:selected, QTabBar::tab:hover {" \
                       "background-color: qlineargradient(spread:pad, x1:1, y1:1, x2:1," \
                       "y2:0.892045, stop:0 rgba(130, 56, 149, 255)," \
                       "stop:0.994318 rgba(130, 56, 149, 255), stop:1 rgba(255, 255, 255, 0));}" \
                       "QTabBar::tab:selected {" \
                       "border-right: 1px solid rgb(88, 88, 91, 25);" \
                       "border-left: 1px solid rgb(88, 88, 91, 25);}"
TREE_PROPERTIES_STYLE_SHEET = "QTreeView::item {" \
                              "height: 23px;}" \
                              "QHeaderView {" \
                              "font-size: 8pt;" \
                              "color: #969696;}" \
                              "QHeaderView::section {" \
                              "background-color: #FFFFFF;" \
                              "padding-left: 5px;" \
                              "height: 22.25px;" \
                              "border: 1px solid #FFFFFF;}"
TAB_SCHEME_STYLE_SHEET = "background-color: rgb(230, 231, 232);"
ELEMENTS_TABLE_STYLE_SHEET = "QTableView {" \
                             "font-size: 8pt;}" \
                             "QHeaderView {" \
                             "font-size: 8pt;" \
                             "color: #969696;}" \
                             "QHeaderView::section {" \
                             "background-color: #FFFFFF;" \
                             "height: 55px;" \
                             "border: 1px solid #58585B;}" \
                             "QTableCornerButton::section {" \
                             "background-color: #FFFFFF;" \
                             "border: 1px solid #58585B;}"
NEW_ELEMENTS_LIST_STYLE_SHEET = "QListView {" \
                                "background-color: #EBE6E8;" \
                                "border: 0px;" \
                                "font-size: 10pt;}" \
                                "QListView::item {" \
                                "height: 52px}" \
                                "QListView::item:hover {" \
                                "background-color: rgba(255, 255, 255, 90);}" \
                                "QListView::item:selected {" \
                                "background-color: rgba(130, 56, 149, 90);}"
SPLITTER_STYLE_SHEET = "QSplitter::handle {" \
                       "border: 0px;" \
                       "border-left: 1px solid rgb(88, 88, 91, 25);}"
CLOSE_BTN_STYLE_SHEET = "QPushButton {" \
                        "background: none;" \
                        "color: rgb(0, 0, 0);" \
                        "border: 0px;}" \
                        "QPushButton::hover {" \
                        "background-color: rgba(255, 255, 255, 100);}" \
                        "QPushButton::pressed {" \
                        "background-color: rgba(255, 255, 255, 200);}"