# -*- coding: utf-8 -*-
""" Модуль описывает классы древовидной структуры отображаемых параметров элементов и узлов в графическом редакторе,
а также класс списка добавления новых элементов.
"""

import xml.etree.ElementTree
import xml.etree.cElementTree as ET
import classes
import language
import prog_settings


class VetvParameters(classes.Node):

    def __init__(self, node_scheme, vetv, node_name='vetv', node_type='', node_value=u'', parent=None,
                 node_description=u''):
        super(VetvParameters, self).__init__(node_name, node_type, node_value, parent, node_description)
        vetv_type = vetv.input_parameters().vetv_type().value()
        vetv_view = vetv.gui_parameters().view().value()
        input_parameters_lang = prog_settings.VetvInputParameters(vetv_type, vetv_view)
        gui_parameters_lang = prog_settings.VetvGuiParameters(vetv_view)
        additional_parameters_lang = prog_settings.VetvAdditionalParameters(vetv_view)
        optimization_parameters_lang = prog_settings.VetvOptimizationParameters(vetv_view)
        output_parameters_lang = prog_settings.VetvOutputParameters(vetv_type, vetv_view)

        self.add_child(classes.Node(node_name='input_parameters', node_value=language.Other().parameters_groups[0],
                                    parent=self))
        self.add_child(classes.Node(node_name='additional_parameters', node_value=language.Other().parameters_groups[3],
                                    parent=self))
        self.add_child(classes.Node(node_name='gui_parameters', node_value=language.Other().parameters_groups[1],
                                    parent=self))
        self.add_child(classes.Node(node_name='optimization_parameters',
                                    node_value=language.Other().parameters_groups[4], parent=self))
        self.add_child(classes.Node(node_name='output_parameters', node_value=language.Other().parameters_groups[2],
                                    parent=self))
        # Создание списка входных параметров.
        # Вставка единиц измерения в списке входных параметров.
        input_parameters_names_descr = input_parameters_lang.names_descriptions
        for p, param in enumerate(input_parameters_names_descr):
            input_parameters_names_descr[p][1] = param[1].replace('\n', ' ')
            input_parameters_names_descr[p][1] = param[1].replace('unit_voltage', node_scheme.unit_voltage)
            input_parameters_names_descr[p][1] = param[1].replace('unit_active_power', node_scheme.unit_active_power)
            input_parameters_names_descr[p][1] = param[1].replace('unit_reactive_power',
                                                                  node_scheme.unit_reactive_power)
            input_parameters_names_descr[p][1] = param[1].replace('unit_current', node_scheme.unit_current)
        input_parameters = self.child(0)
        for param in input_parameters_names_descr:
            input_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1], node_description=param[2],
                                                    parent=input_parameters))
        # Создание списка оптимизационных параметров.
        # Вставка единиц измерения в списке оптимизационных параметров.
        optimization_parameters_names_descr = optimization_parameters_lang.names_descriptions
        for p, param in enumerate(optimization_parameters_names_descr):
            optimization_parameters_names_descr[p][1] = param[1].replace('\n', ' ')
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_voltage', node_scheme.unit_voltage)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_active_power',
                                                                         node_scheme.unit_active_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_reactive_power',
                                                                         node_scheme.unit_reactive_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_full_power', node_scheme.unit_full_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_current', node_scheme.unit_current)
        optimization_parameters = self.child(3)
        for param in optimization_parameters_names_descr:
            optimization_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1],
                                                           node_description=param[1], parent=optimization_parameters))
        # Создание списка дополнительных параметров.
        additional_parameters = self.child(1)
        for param in additional_parameters_lang.names_descriptions:
            additional_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1],
                                                         node_description=param[1], parent=additional_parameters))
        # Создание списка графических параметров.
        gui_parameters = self.child(2)
        for param in gui_parameters_lang.names_descriptions:
            gui_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1],
                                                  node_description=param[1], parent=gui_parameters))
        # Создание списка выходных параметров.
        output_parameters_names_descr = output_parameters_lang.names_descriptions
        for p, param in enumerate(output_parameters_names_descr):
            output_parameters_names_descr[p][1] = param[1].replace('unit_voltage', node_scheme.unit_voltage)
            output_parameters_names_descr[p][1] = param[1].replace('unit_active_power', node_scheme.unit_active_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_reactive_power',
                                                                   node_scheme.unit_reactive_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_full_power', node_scheme.unit_full_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_current', node_scheme.unit_current)
        output_parameters = self.child(4)
        for param in output_parameters_names_descr:
            output_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1],
                                                     node_description=param[1], parent=output_parameters))


class UselParameters(classes.Node):

    def __init__(self, node_scheme, node_name='vetv', node_type='', node_value=u'', parent=None, node_description=u''):
        super(UselParameters, self).__init__(node_name, node_type, node_value, parent, node_description)
        gui_parameters_lang = prog_settings.UselGuiParameters()
        input_parameters_lang = prog_settings.UselInputParameters()
        output_parameters_lang = prog_settings.UselOutputParameters()
        optimization_parameters_lang = prog_settings.UselOptimizationParameters()

        self.add_child(classes.Node(node_name='input_parameters', node_value=language.Other().parameters_groups[0],
                                    parent=self))
        self.add_child(classes.Node(node_name='gui_parameters', node_value=language.Other().parameters_groups[1],
                                    parent=self))
        self.add_child(classes.Node(node_name='optimization_parameters',
                                    node_value=language.Other().parameters_groups[4], parent=self))
        self.add_child(classes.Node(node_name='output_parameters', node_value=language.Other().parameters_groups[2],
                                    parent=self))
        # Создание списка входных параметров.
        input_parameters = self.child(0)
        input_parameters_names_descr = input_parameters_lang.names_descriptions
        for param in input_parameters_names_descr:
            input_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1], node_description=param[2],
                                                    parent=input_parameters))
        # Создание списка графических параметров.
        gui_parameters = self.child(1)
        gui_parameters_names_descr = gui_parameters_lang.names_descriptions
        for param in gui_parameters_names_descr:
            gui_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1], node_description=param[1],
                                                  parent=gui_parameters))
        # Создание списка оптимизационных параметров.
        optimization_parameters_names_descr = optimization_parameters_lang.names_descriptions
        for p, param in enumerate(optimization_parameters_names_descr):
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_voltage', node_scheme.unit_voltage)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_active_power',
                                                                         node_scheme.unit_active_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_reactive_power',
                                                                         node_scheme.unit_reactive_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_full_power', node_scheme.unit_full_power)
            optimization_parameters_names_descr[p][1] = param[1].replace('unit_current', node_scheme.unit_current)
        optimization_parameters = self.child(2)
        for param in optimization_parameters_names_descr:
            optimization_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1],
                                                           node_description=param[1], parent=optimization_parameters))
        # Создание списка выходных параметров.
        output_parameters_names_descr = output_parameters_lang.names_descriptions
        for p, param in enumerate(output_parameters_names_descr):
            output_parameters_names_descr[p][1] = param[1].replace('unit_voltage', node_scheme.unit_voltage)
            output_parameters_names_descr[p][1] = param[1].replace('unit_active_power', node_scheme.unit_active_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_reactive_power',
                                                                   node_scheme.unit_reactive_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_full_power', node_scheme.unit_full_power)
            output_parameters_names_descr[p][1] = param[1].replace('unit_current', node_scheme.unit_current)
        output_parameters = self.child(3)
        for param in output_parameters_names_descr:
            output_parameters.add_child(classes.Node(node_name=param[0], node_value=param[1], node_description=param[1],
                                                     parent=output_parameters))


class NewElements(classes.Node):

    def __init__(self, node_name='', node_type='', node_value=u'', parent=None, node_description=u''):
        super(NewElements, self).__init__(node_name, node_type, node_value, parent, node_description)

    def read_elements_file(self, module_path):
        errors = language.Errors()
        elements_lang = language.Elements()
        errs = []
        # Открытие файла.
        try:
            if module_path:
                tree = ET.parse(u'%s/new_elements.xml' % module_path)
            else:
                tree = ET.parse(u'new_elements.xml')
        except:
            errs.append(errors.messages['error open new elements file'])
            return errs, [], []
        root = tree.getroot()
        # Поиск всех групп элементов в файле.
        elements_groups = root.findall('elements_group')
        if elements_groups is not None:
            for elements_group in elements_groups:
                elements_group_attributes = elements_group.attrib
                if 'name' in elements_group_attributes:
                    elements_group_name = elements_group_attributes['name']
                    if elements_group_name in elements_lang.elements_groups:
                        elements_group_obj = classes.Node(node_name=elements_group_name,
                                                          node_value=elements_lang.elements_groups[elements_group_name])
                        self.add_child(elements_group_obj)
                        # Поиск всех элементов в группе.
                        elements = elements_group.findall('element')
                        if elements is not None:
                            for element in elements:
                                element_name = element.find('name')
                                if element_name is not None:
                                    element_name = element_name.text
                                    if element_name in elements_lang.elements:
                                        element_obj = classes.Parameters(node_name=element_name,
                                                                         node_value=elements_lang.elements[element_name])
                                        elements_group_obj.add_child(element_obj)
                                        for parameter in element:
                                            parameter_name = ''
                                            if parameter.tag == 'image':
                                                parameter_name = 'image'
                                            elif parameter.tag == 'begin':
                                                parameter_name = 'begin'
                                            elif parameter.tag == 'end':
                                                parameter_name = 'end'
                                            elif parameter.tag == 'vetv_type':
                                                parameter_name = 'vetv_type'
                                            elif parameter.tag == 'view':
                                                parameter_name = 'view'
                                            elif parameter.tag == 'switcher_permit':
                                                parameter_name = 'switcher_permit'
                                            if parameter_name:
                                                if parameter.text and parameter.text is not None:
                                                    if parameter_name == 'image' and module_path:
                                                        parameter_text = u'%s/%s' % (module_path, parameter.text)
                                                    else:
                                                        parameter_text = parameter.text
                                                    parameter_obj = classes.Node(node_name=parameter_name,
                                                                                 node_value=parameter_text)
                                                    element_obj.add_child(parameter_obj)
        return errs, [], []