# -*- coding: utf-8 -*-
""" Модуль описывает функции и классы, добавляющие в программу поддержку разных языков.
"""

try:
    from PyQt4 import QtGui

    try:
        _encoding = QtGui.QApplication.UnicodeUTF8
        def _translate(context, text, disambig):
            return unicode(QtGui.QApplication.translate(context, text, disambig, _encoding))
    except AttributeError:
        def _translate(context, text, disambig):
            return unicode(QtGui.QApplication.translate(context, text, disambig))
except ImportError:
    def __translate(context, text, disambig):
        return unicode(text)

LANGUAGES = ['ru - Russian', 'en - English']


class Properties(object):

    def __init__(self):
        self.scheme_types = [[_translate('property', 'Simple scheme', None), ''],
                             [_translate('property', 'Three-phase balanced circuit', None), '3_phase']]
        self.unit_voltage = {1.: _translate('unit_voltage', 'V', None),
                             1E3: _translate('unit_voltage', 'kV', None),
                             1E6: _translate('unit_voltage', 'MV', None)}
        self.unit_current = {1.: _translate('unit_current', 'A', None),
                             1E3: _translate('unit_current', 'kA', None)}
        self.unit_active_power = {1.: _translate('unit_active_power', 'W', None),
                                  1E3: _translate('unit_active_power', 'kW', None),
                                  1E6: _translate('unit_active_power', 'MW', None)}
        self.unit_reactive_power = {1.: _translate('unit_reactive_power', 'VAR', None),
                                    1E3: _translate('unit_reactive_power', 'kVAR', None),
                                    1E6: _translate('unit_reactive_power', 'MVAR', None)}
        self.unit_full_power = {1.: _translate('unit_full_power', 'VA', None),
                                1E3: _translate('unit_full_power', 'kVA', None),
                                1E6: _translate('unit_full_power', 'MVA', None)}


class Other(object):

    def __init__(self):
        self.vetv_lang = _translate('name element', 'Element', None)
        self.usel_lang = _translate('name node', 'Node', None)
        self.lang_element_node_copy = _translate('word copy', 'copy', None)
        self.graphics_view_context_menu = {'on': _translate('menu', 'On', None), 'off': _translate('menu', 'Off', None),
                                           'delete': _translate('menu', 'Delete', None),
                                           'copy': _translate('menu', 'Copy', None),
                                           'past': _translate('menu', 'Past', None),
                                           'name_move_begin': _translate('menu', 'Unlock name', None),
                                           'name_move_end': _translate('menu', 'Lock name', None),
                                           'select_begin': _translate('menu', 'Select initial node', None),
                                           'select_end': _translate('menu', 'Select end node', None),
                                           'select': _translate('menu', 'Select', None),
                                           'select_end_2': _translate('menu', 'Select end 2 node', None),
                                           'rotate_connection_line_1': _translate('menu', 'Turn the connecting line to '
                                                                                          'the initial node', None),
                                           'rotate_connection_line_2': _translate('menu', 'Turn the connecting line to '
                                                                                          'the end node', None),
                                           'rotate_connection_line_3': _translate('menu', 'Turn the connecting line to '
                                                                                          'the end 2 node', None),
                                           'select_all': _translate('menu', 'Select all', None)}
        self.parameters_header = [_translate('header', 'Parameter', None), _translate('header', 'Value', None)]
        self.parameters_groups = [_translate('header', 'INPUT PARAMETERS', None),
                                  _translate('header', 'GRAPHIC PARAMETERS', None),
                                  _translate('header', 'OUTPUT PARAMETERS', None),
                                  _translate('header', 'ADDITIONAL PARAMETERS', None),
                                  _translate('header', 'OPTIMIZATION PARAMETERS', None)]
        self.optimization_table_header = [_translate('header', 'Variant', None),
                                          u'%s, unit_active_power' % _translate('header', 'Active power\nloss', None),
                                          u'%s, unit_reactive_power' % _translate('header', 'Reactive power\nloss',
                                                                                  None),
                                          u'%s' % _translate('header', 'Number of switching', None),]
        self.optimization_table_rows = {'original_scheme': _translate('header', 'Original scheme', None),
                                        'variant': _translate('header', 'Variant', None)}
        self.table_results = {'consumers powers': _translate('results', 'Consumers powers', None),
                              'generators powers': _translate('results', 'Generators powers', None),
                              'total': _translate('results', 'Total', None),
                              'power flows': _translate('results', 'Power flows in the beginnings and ends of the '
                                                                   'elements', None),
                              'begin': _translate('results', 'Begin', None),
                              'end': _translate('results', 'End', None),
                              'currents': _translate('results', 'Currents', None),
                              'module': _translate('results', 'Module', None),
                              'phase': _translate('results', 'Phase', None),
                              'elements voltages': _translate('results', 'Voltages on the elements and EMF', None),
                              'emf': _translate('results', 'EMF', None),
                              'nodes voltages': _translate('results', 'Voltages in the nodes', None)}


class Elements(object):

    def __init__(self):
        self.elements_groups = {'simple_elements': _translate('elements_group', 'Simple elements', None),
                                'generators': _translate('elements_group', 'Generators', None),
                                'load': _translate('elements_group', 'Load', None),
                                'switchers': _translate('elements_group', 'Switching and protective devices', None),
                                'transformers': _translate('elements_group', 'Transformers', None),
                                'connections': _translate('elements_group', 'Connections', None)}
        self.elements = {'ground': _translate('elements', 'Ground', None),
                         'r': _translate('elements', 'R', None),
                         'l': _translate('elements', 'L', None),
                         'c': _translate('elements', 'C', None),
                         'r_l': _translate('elements', 'RL', None),
                         'r_c': _translate('elements', 'RC', None),
                         'l_c': _translate('elements', 'LC', None),
                         'r_l_c': _translate('elements', 'RLC', None),
                         'e': _translate('elements', 'E', None),
                         'e_r': _translate('elements', 'E+R', None),
                         'e_l': _translate('elements', 'E+L', None),
                         'e_c': _translate('elements', 'E+C', None),
                         'e_r_l': _translate('elements', 'E+RL', None),
                         'e_r_c': _translate('elements', 'E+RC', None),
                         'e_r_l_c': _translate('elements', 'E+RLC', None),
                         'e_l_c': _translate('elements', 'E+LC', None),
                         'i': _translate('elements', 'I', None),
                         'dc_source': _translate('elements', 'DC source', None),
                         'ac_source': _translate('elements', 'AC source', None),
                         'load_dependent_voltage': _translate('elements', 'Load-dependent voltage', None),
                         'load_constant_active_power': _translate('elements', 'Load constant active power', None),
                         'load_constant_reactive_power': _translate('elements', 'Load constant reactive power', None),
                         'load_constant_full_power': _translate('elements', 'Load constant full power', None),
                         'circuit_breaker': _translate('elements', 'Circuit breaker', None),
                         'hv_circuit_breaker': _translate('elements', 'HV circuit breaker', None),
                         'disconnector': _translate('elements', 'Disconnector', None),
                         'fuse': _translate('elements', 'Fuse', None),
                         'two_winding_transformer': _translate('elements', 'Two-winding transformer', None),
                         'three_winding_transformer': _translate('elements', 'Three-winding transformer', None),
                         'cable_line': _translate('elements', 'Cable line', None),
                         'overhead_line': _translate('elements', 'Overhead line', None),
                         'simple_connection': _translate('elements', 'Simple connection', None)}

        self.lang_add_vetv = _translate('elements', 'Element', None)


class Parameters(object):

    def __init__(self):
        self.input_parameters = {'name': [_translate('input parameter', 'Name', None),
                                          _translate('input parameter', 'Element name (must not begin with "-" sign)',
                                                     None), 140],
                                 'vetv_type': [_translate('input parameter', 'Type', None),
                                               _translate('input parameter', 'Element type', None), 190],
                                 'note': [_translate('input parameter', 'Note', None),
                                          _translate('input parameter', 'Note', None), 140],
                                 'new_element_name': [_translate('input parameter', 'New element name', None),
                                                      _translate('input parameter', 'New element name', None), 50],
                                 'begin': [_translate('input parameter', 'Begin', None),
                                           _translate('input parameter', 'Begin node name', None), 80],
                                 'end': [_translate('input parameter', 'End', None),
                                         _translate('input parameter', 'End node name', None), 80],
                                 'end_2': [_translate('input parameter', 'End 2', None),
                                           _translate('input parameter', 'End 2 node name', None), 80],
                                 'real_voltage': [u'%s,\nunit_voltage' % _translate('input parameter', 'Ureal', None),
                                                  _translate('input parameter', 'Element real voltage', None), 70],
                                 'real_phase': [u'%s, °' % _translate('input parameter', 'Real\nphase', None),
                                                  _translate('input parameter', 'Real phase rotation', None), 70],
                                 'voltage': [u'%s, unit_voltage' % _translate('input parameter', 'Module\nUnom', None),
                                             _translate('input parameter', 'Module nominal element voltage', None), 80],
                                 'voltage_psi': [u'%s, °' % _translate('input parameter', 'Phase\nUnom', None),
                                                 _translate('input parameter',
                                                            'The initial phase of the nominal element voltage', None),
                                                 60],
                                 'active_power': [u'%s, unit_active_power' % _translate('input parameter', 'P', None),
                                                  _translate('input parameter', 'Element active power', None), 80],
                                 'reactive_power': [u'%s, unit_reactive_power' % _translate('input parameter', 'Q',
                                                                                            None),
                                                    _translate('input parameter', 'Element reactive power', None), 80],
                                 'active_resistance': [u'%s' % _translate('input parameter', 'R, Ohm', None),
                                                       _translate('input parameter', 'Element active resistance', None),
                                                       70],
                                 'reactive_resistance': [u'%s' % _translate('input parameter', 'X, Ohm', None),
                                                         _translate('input parameter',
                                                                    'Element reactive resistance (if the capacitance, '
                                                                    'then the "-")', None), 70],
                                 'active_resistance_2': [u'%s' % _translate('input parameter', 'R2, Ohm', None),
                                                         _translate('input parameter',
                                                                    'Element 2 branch active resistance', None), 70],
                                 'reactive_resistance_2': [u'%s' % _translate('input parameter', 'X2, Ohm', None),
                                                           _translate('input parameter',
                                                                      'Element 2 branch reactive resistance (if the '
                                                                      'capacitance, then the "-")', None), 70],
                                 'active_resistance_3': [u'%s' % _translate('input parameter', 'R3, Ohm', None),
                                                         _translate('input parameter',
                                                                    'Element 3 branch active resistance', None), 70],
                                 'reactive_resistance_3': [u'%s' % _translate('input parameter', 'X3, Ohm', None),
                                                           _translate('input parameter',
                                                                      'Element 3 branch reactive resistance (if the '
                                                                      'capacitance, then the "-")', None), 70],
                                 'active_conductivity_tr1': [u'%s' % _translate('input parameter', 'G1, Sm', None),
                                                             _translate('input parameter',
                                                                        'Active conductivity transverse '
                                                                        'branch in the begin node', None),
                                                             70],
                                 'active_conductivity_tr2': [u'%s' % _translate('input parameter', 'G2, Sm', None),
                                                             _translate('input parameter',
                                                                        'Active conductivity transverse '
                                                                        'branch in the end node', None), 70],
                                 'reactive_conductivity_tr1': [u'%s' % _translate('input parameter', 'B1, Sm', None),
                                                               _translate('input parameter',
                                                                          'Reactive conductivity transverse '
                                                                          'branch in the begin node', None), 70],
                                 'reactive_conductivity_tr2': [u'%s' % _translate('input parameter', 'B2, Sm', None),
                                                               _translate('input parameter',
                                                                          'Reactive conductivity transverse '
                                                                          'branch in the end node', None), 70],
                                 'eds': [u'%s, unit_voltage' % _translate('input parameter', 'Module\nE', None),
                                         _translate('input parameter', 'Module of the EMF element', None), 80],
                                 'eds_psi': [u'%s, °' % _translate('input parameter', 'Phase\nE', None),
                                             _translate('input parameter', 'The initial phase of the EMF element',
                                                        None), 60],
                                 'current': [u'%s, unit_current' % _translate('input parameter',
                                                                              'Module I\nof the current\nsource', None),
                                             _translate('input parameter',
                                                        'Module of the current of the current source element', None),
                                             80],
                                 'current_psi': [u'%s, °' % _translate('input parameter',
                                                                       'Phase I\nof the current\nsource', None),
                                                 _translate('input parameter',
                                                            'The initial phase of the current of the current source '
                                                            'element', None), 70],
                                 'voltage_addition': [u'%s, %%' % _translate('input parameter',
                                                                             'Module\nU\naddition', None),
                                                      _translate('input parameter',
                                                                 'Module voltage addition', None), 70],
                                 'voltage_addition_psi': [u'%s, °' % _translate('input parameter',
                                                                                'Phase\nU\naddition', None),
                                                          _translate('input parameter',
                                                                     'Phase voltage addition', None), 70],
                                 'transformation_coefficient': [u'%s' % _translate('input parameter',
                                                                                   'Module\nKt', None),
                                                                _translate('input parameter',
                                                                           'Module of transformation coefficient',
                                                                           None), 70],
                                 'transformation_coefficient_psi': [u'%s, °' % _translate('input parameter',
                                                                                          'Phase\nKt', None),
                                                                    _translate('input parameter',
                                                                               'Phase of transformation coefficient',
                                                                               None), 70],
                                 'transformation_coefficient_2': [u'%s' % _translate('input parameter',
                                                                                     'Module\nKt 2', None),
                                                                  _translate('input parameter',
                                                                             'Module of transformation coefficient 2',
                                                                             None), 70],
                                 'transformation_coefficient_psi_2': [u'%s, °' % _translate('input parameter',
                                                                                            'Phase\nKt 2', None),
                                                                      _translate('input parameter',
                                                                                 'Phase of transformation coefficient '
                                                                                 '2', None), 70]}

        self.gui_parameters = {'view': _translate('gui parameter', 'View', None),
                               'position_x': _translate('gui parameter', 'X coordinate', None),
                               'position_y': _translate('gui parameter', 'Y coordinate', None),
                               'rotation': _translate('gui parameter', 'Rotation', None),
                               'mirror_x': _translate('gui parameter', 'X-axis mirror', None),
                               'mirror_y': _translate('gui parameter', 'Y-axis mirror', None),
                               'rotate_line_1': _translate('gui parameter', 'Rotate 1 connecting line', None),
                               'rotate_line_2': _translate('gui parameter', 'Rotate 2 connecting line', None),
                               'rotate_line_3': _translate('gui parameter', 'Rotate 3 connecting line', None)}

        self.additional_parameters = {'nominal_temperature': _translate('additional parameter',
                                                                        'Nominal temperature', None),
                                      'operating_temperature': _translate('additional parameter',
                                                                          'Operating  temperature', None),
                                      'short_current_temperature': _translate('additional parameter',
                                                                              'Short-current temperature', None),
                                      'temperature_coefficient': _translate('additional parameter',
                                                                            'Temperature coefficient', None)}

        self.optimization_parameters = {'use_for_optimization': _translate('optimization parameter',
                                                                           'Use for optimization', None),
                                        'generator_switcher': _translate('optimization parameter',
                                                                         'Generator switcher', None),
                                        'generator_priority': _translate('optimization parameter',
                                                                         'Generator priority', None),
                                        'generator_nominal_active_power': u'%s, unit_active_power'
                                                                          % _translate('optimization parameter',
                                                                                       'Generator nominal active power',
                                                                                       None),
                                        'generator_nominal_reactive_power': u'%s, unit_reactive_power'
                                                                            % _translate('optimization parameter',
                                                                                         'Generator nominal reactive '
                                                                                         'power', None),
                                        'generator_nominal_full_power': u'%s, unit_full_power'
                                                                        % _translate('optimization parameter',
                                                                                     'Generator nominal full power',
                                                                                     None),
                                        'generator_normal_power_coefficient': _translate('optimization parameter',
                                                                                         'Generator normal power '
                                                                                         'coefficient', None),
                                        'generator_overdrive_power_coefficient': _translate('optimization parameter',
                                                                                            'Generator overdrive '
                                                                                            'power coefficient', None),
                                        'minimum_voltage': u'%s, unit_voltage' % _translate('optimization parameter',
                                                                                            'Minimum voltage', None),
                                        'maximum_voltage': u'%s, unit_voltage' % _translate('optimization parameter',
                                                                                            'Maximum voltage', None),
                                        'switcher_permit': _translate('optimization parameter',
                                                                      'Switcher permit', None),
                                        'check_direction': _translate('optimization parameter',
                                                                      'Check power flow direction', None)}

        self.output_parameters = {'real_u': u'%s, unit_voltage' % _translate('output parameter',
                                                                             'Element branch voltage', None),
                                  'real_u_2': u'%s, unit_voltage' % _translate('output parameter',
                                                                               'Element 2 branch voltage', None),
                                  'real_u_3': u'%s, unit_voltage' % _translate('output parameter',
                                                                               'Element 3 branch voltage', None),
                                  'real_e': u'%s, unit_voltage' % _translate('output parameter',
                                                                             'Element EMF', None),
                                  'real_u1': u'%s, unit_voltage' % _translate('output parameter',
                                                                              'U at the begin of the branch', None),
                                  'real_u1_2': u'%s, unit_voltage' % _translate('output parameter',
                                                                                'U at the begin of the 2 branch',
                                                                                None),
                                  'real_u1_3': u'%s, unit_voltage' % _translate('output parameter',
                                                                                'U at the begin of the 3 branch',
                                                                                None),
                                  'real_u2': u'%s, unit_voltage' % _translate('output parameter',
                                                                              'U at the end of the branch',
                                                                              None),
                                  'real_u2_2': u'%s, unit_voltage' % _translate('output parameter',
                                                                                'U at the end of the 2 branch',
                                                                                None),
                                  'real_u2_3': u'%s, unit_voltage' % _translate('output parameter',
                                                                                'U at the end of the 3 branch',
                                                                                None),
                                  'real_i': u'%s, unit_current' % _translate('output parameter',
                                                                             'I of the branch', None),
                                  'real_i_2': u'%s, unit_current' % _translate('output parameter',
                                                                               'I of the 2 branch', None),
                                  'real_i_3': u'%s, unit_current' % _translate('output parameter',
                                                                               'I of the 3 branch', None),
                                  's1': u'%s, unit_full_power' % _translate('output parameter',
                                                                            'S at the begin of the branch', None),
                                  's1_2': u'%s, unit_full_power' % _translate('output parameter',
                                                                              'S at the begin of the 2 branch', None),
                                  's1_3': u'%s, unit_full_power' % _translate('output parameter',
                                                                              'S at the begin of the 3 branch', None),
                                  's2': u'%s, unit_full_power' % _translate('output parameter',
                                                                            'S at the end of the branch', None),
                                  's2_2': u'%s, unit_full_power' % _translate('output parameter',
                                                                              'S at the end of the 2 branch', None),
                                  's2_3': u'%s, unit_full_power' % _translate('output parameter',
                                                                              'S at the end of the 3 branch', None),
                                  'ist_s': u'%s, unit_full_power' % _translate('output parameter', 'S source', None),
                                  'priem_s': u'%s, unit_full_power' % _translate('output parameter',
                                                                                 'S receiver branch', None),
                                  'priem_s_2': u'%s, unit_full_power' % _translate('output parameter',
                                                                                   'S receiver 2 branch', None),
                                  'priem_s_3': u'%s, unit_full_power' % _translate('output parameter',
                                                                                   'S receiver 3 branch', None),
                                  'real_i1_tr': u'%s, unit_current' % _translate('output parameter',
                                                                                 'I transverse 1 branch', None),
                                  'real_i2_tr': u'%s, unit_current' % _translate('output parameter',
                                                                                 'I transverse 2 branch', None),
                                  'real_i3_tr': u'%s, unit_current' % _translate('output parameter',
                                                                                 'I transverse 3 branch', None),
                                  'priem_s1_tr': u'%s, unit_full_power' % _translate('output parameter',
                                                                                     'S transverse 1 branch', None),
                                  'priem_s2_tr': u'%s, unit_full_power' % _translate('output parameter',
                                                                                     'S transverse 2 branch', None),
                                  'priem_s3_tr': u'%s, unit_full_power' % _translate('output parameter',
                                                                                     'S transverse 3 branch', None)}
        self.usli_input_parameters = {'name': [_translate('input parameter', 'Name', None),
                                               _translate('input parameter', 'Node name', None), 150],
                                      'note': [_translate('input parameter', 'Note', None),
                                               _translate('input parameter', 'Note', None), 150]}

        self.usli_output_parameters = {'real_u_usel': u'%s, unit_voltage' % _translate('output parameter',
                                                                                       'Node voltage', None)}


class Errors(object):

    def __init__(self):
        self.titles = {'error': _translate('error title', 'Error', None),
                       'export do not complete': _translate('error title', 'Export do not complete', None),
                       'open file fatal error': _translate('error title',
                                                           'A fatal errors occurred while opening the file', None),
                       'open file error': _translate('error title', 'An errors occurred while opening the file', None),
                       'no scheme': _translate('error title', 'No scheme', None),
                       'error saving': _translate('error title', 'Error saving scheme', None),
                       'error open new elements file': _translate('error title', 'Error read new elements file', None)}

        self.messages = {'invalid maximum number of iterations': _translate('error message',
                                                                            'Invalid specified maximum number of '
                                                                            'iterations', None),
                         'can continue work': _translate('error message', 'You can continue working with the file',
                                                         None),
                         'no scheme': _translate('error message', 'You need to create or open scheme', None),
                         'calculation need': _translate('error message', 'First, you need to perform a calculation',
                                                        None),
                         'error': _translate('message', 'ERROR', None),
                         'warning': _translate('message', 'WARNING', None),
                         'invalid actual element voltage': _translate('message', 'Is invalid actual element voltage',
                                                                      None),
                         'invalid actual element phase': _translate('message', 'Is invalid actual element phase',
                                                                    None),
                         'elements parameters not set': _translate('message', 'Elements branches parameters not set',
                                                                   None),
                         'incorrect base voltage': _translate('message', 'Incorrectly set the base voltage', None),
                         'invalid module EFM': _translate('message', 'An invalid module EMF element', None),
                         'invalid phase EFM': _translate('message', 'An invalid phase EMF element', None),
                         'invalid module nominal voltage': _translate('message',
                                                                      'An invalid module nominal voltage element',
                                                                      None),
                         'invalid phase nominal voltage': _translate('message',
                                                                     'An invalid phase nominal voltage element', None),
                         'invalid active power': _translate('message', 'An invalid active power element', None),
                         'invalid reactive power': _translate('message', 'An invalid reactive power element', None),
                         'invalid module current': _translate('message', 'An invalid module current element', None),
                         'invalid phase current': _translate('message', 'An invalid phase current element', None),
                         'unknown initial node': _translate('message', 'Unknown initial element node', None),
                         'unknown end node': _translate('message', 'Unknown end element node', None),
                         'invalid active resistance': _translate('message', 'An invalid active resistance element',
                                                                 None),
                         'invalid reactive resistance': _translate('message', 'An invalid reactive resistance element',
                                                                   None),
                         'not set type element': _translate('message', 'Not set the type of element', None),
                         'not set nominal voltage': _translate('message', 'Not set the nominal voltage of element',
                                                               None),
                         'not set resistance': _translate('message', 'Not set resistance of element', None),
                         'not set EFM': _translate('message', 'Not set EFM of element', None),
                         'not set current': _translate('message', 'Not set current of element', None),
                         'not set parameters': _translate('message', 'Not set parameters of element', None),
                         'complete additional calculations': _translate('message',
                                                                        'Additional calculations are completed', None),
                         'complete main calculations': _translate('message', 'Main calculation is completed', None),
                         'complete input transverse conversion': _translate('message',
                                                                            'Converting elements containing '
                                                                            'transverse branches completed', None),
                         'complete output transverse conversion': _translate('message',
                                                                             'The inverse transform elements '
                                                                             'containing transverse branches completed',
                                                                             None),
                         'complete input 3 line conversion': _translate('message',
                                                                        'Converting three-line elements completed',
                                                                        None),
                         'complete output 3 line conversion': _translate('message',
                                                                         'The inverse transform three-line elements '
                                                                         'completed', None),
                         'complete input three-phase scheme': _translate('message',
                                                                         'Transformation of the three-phase circuit '
                                                                         'has been completed', None),
                         'complete output three-phase scheme': _translate('message',
                                                                          'The inverse transform three phase circuit '
                                                                          'has been completed', None),
                         'complete units conversion': _translate('message', 'Converting units completed', None),
                         'pass check balance power': _translate('message',
                                                                'Checking on the balance of power is passed. '
                                                                'The difference:', None),
                         'fail check balance power': _translate('message',
                                                                'Checking on the balance of power is failed. '
                                                                'The difference:', None),
                         'fail check 1 Kirh': _translate('message', 'Check after 1 Kirchhoff`s law is not passed.'
                                                                    'Difference in node', None),
                         'pass check 1 Kirh': _translate('message', 'Check after 1 Kirchhoff`s law is passed in all '
                                                                    'nodes.', None),
                         'maximum number of iterations': _translate('message',
                                                                    'Has reached the maximum number of iterations',
                                                                    None),
                         'not set scheme file': _translate('message', 'Scheme file not set', None),
                         'unable to write file': _translate('message', 'Unable to write to the file', None),
                         'unable to open scheme file': _translate('message', 'Unable to open scheme file', None),
                         'failed to save scheme file': _translate('message', 'Failed to save the scheme file', None),
                         'can not read the base voltage': _translate('message', 'Can not read the base voltage', None),
                         'can not read elements parameters': _translate('message', 'Can not read the parameters of '
                                                                                   'the elements', None),
                         'can not read nodes parameters': _translate('message', 'Can not read the parameters of '
                                                                                'the nodes', None),
                         'invalid scheme type': _translate('message', 'Invalid scheme type in the scheme file', None),
                         'invalid unit of active power': _translate('message', 'Invalid specified unit of active power '
                                                                               'in the scheme file', None),
                         'invalid unit of reactive power': _translate('message', 'Invalid specified unit of reactive '
                                                                                 'power in the scheme file', None),
                         'invalid unit of full power': _translate('message', 'Invalid specified unit of full '
                                                                             'power in the scheme file', None),
                         'invalid unit of voltage': _translate('message', 'Invalid specified unit of voltage in the '
                                                                          'scheme file', None),
                         'invalid unit of current': _translate('message', 'Invalid specified unit of current in the '
                                                                          'scheme file', None),
                         'invalid unit': _translate('message', 'Invalid specified unit', None),
                         'invalid begin active conductivity': _translate('message',
                                                                         'Invalid specified active '
                                                                         'conductivity transverse branch in the '
                                                                         'initial element node', None),
                         'invalid begin reactive conductivity': _translate('message',
                                                                           'Invalid specified reactive '
                                                                           'conductivity transverse branch in the '
                                                                           'initial element node', None),
                         'invalid end active conductivity': _translate('message',
                                                                       'Invalid specified active '
                                                                       'conductivity transverse branch in the '
                                                                       'end element node', None),
                         'invalid end reactive conductivity': _translate('message',
                                                                         'Invalid specified reactive '
                                                                         'conductivity transverse branch in the '
                                                                         'end element node', None),
                         'can not adjust the source': _translate('message',
                                                                 'Can not adjust the source of the element', None),
                         'can not be achieved a given power': _translate('message',
                                                                         'Can not be achieved a given power in the '
                                                                         'element', None),
                         'maybe not enough power sources': _translate('message',
                                                                      'Maybe not enough power sources', None),
                         'deleted element': _translate('message', 'deleted element', None),
                         'its start and end nodes are the same': _translate('message',
                                                                            'its start and end nodes are the same',
                                                                            None),
                         'can not done optimization': _translate('message', 'Can not done optimization', None),
                         'not set generators and switchers': _translate('message', 'Not set generators and switchers, '
                                                                                   'that uses for optimization', None),
                         'error open new elements file': _translate('error title',
                                                                    'Error read file new_elements.xml', None),
                         'different voltages converge': _translate('error title',
                                                                   'Elements with different voltages converge at node',
                                                                   None)}


class Messages(object):

    def __init__(self):
        self.messages = {'open scheme': _translate('message', 'Open scheme file', None),
                         'save scheme': _translate('message', 'Save scheme file', None),
                         'scheme file': _translate('message', 'scheme file', None),
                         'results export': _translate('message', 'Export the results of calculation', None),
                         'opening a file': _translate('message', 'Opening a file', None),
                         'open the file': _translate('message', 'Open the file', None),
                         'saving a file': _translate('message', 'Saving a file', None),
                         'save the file': _translate('message', 'Save the file', None),
                         'calculation critical errors': _translate('message', 'A critical error in the calculation. '
                                                                              'The calculation can not be continued',
                                                                   None),
                         'complete calculation': _translate('message', 'The calculation is completed', None),
                         'iterations number': _translate('message', 'Number of iterations', None),
                         'runtime': _translate('message', 'Runtime', None),
                         'save scheme?': _translate('message', 'Do you want to save scheme file?', None),
                         'number schemes': _translate('message', 'Total number of variants schemes', None),
                         'number switchers': _translate('message', 'Total number of switchers, that are used for '
                                                                   'optimization', None),
                         'number generators': _translate('message', 'Total number of generators, that are used for '
                                                                    'optimization', None),
                         'number calculated schemes': _translate('message', 'Total number of successfully calculated '
                                                                            'schemes', None),
                         'number good voltages schemes': _translate('message', 'Total number of schemes with '
                                                                               'acceptable levels of voltages', None),
                         'complete optimization': _translate('message', 'The optimization is successfully complete',
                                                             None),
                         'optimization errors': _translate('message', 'Optimization is not complete. Report an error',
                                                           None),
                         'optimization warnings': _translate('message', 'Optimization is complete, but there are '
                                                                        'warnings', None),
                         'number normal schemes': _translate('message', 'Number of schemes with the normal operation '
                                                                        'of sources', None),
                         'number overdrive schemes': _translate('message', 'Number of schemes with an overload of '
                                                                           'sources', None)}


def vetvi_view_types():
    types = {'': _translate('view type', 'Other', None),
             'r': _translate('view type', 'R', None),
             'l': _translate('view type', 'L', None),
             'c': _translate('view type', 'C', None),
             'r_l': _translate('view type', 'RL', None),
             'r_c': _translate('view type', 'RC', None),
             'r_l_c': _translate('view type', 'RLC', None),
             'l_c': _translate('view type', 'LC', None),
             'e': _translate('view type', 'EMF', None),
             'e_r': _translate('view type', 'EMF and R', None),
             'e_l': _translate('view type', 'EMF and L', None),
             'e_c': _translate('view type', 'EMF and C', None),
             'e_r_l': _translate('view type', 'EMF and RL', None),
             'e_r_c': _translate('view type', 'EMF and RC', None),
             'e_r_l_c': _translate('view type', 'EMF and RLC', None),
             'e_l_c': _translate('view type', 'EMF and LC', None),
             'i': _translate('view type', 'Current source', None),
             'cable': _translate('view type', 'Cable line', None),
             'line': _translate('view type', 'Line', None),
             'hv_switch': _translate('view type', 'HV circuit breaker', None),
             'lv_switch': _translate('view type', 'Circuit breaker', None),
             'breaker': _translate('view type', 'Disconnector', None),
             'fuse': _translate('view type', 'Fuse', None),
             'ac_source': _translate('view type', 'AC source', None),
             'dc_source': _translate('view type', 'DC source', None),
             'load': _translate('view type', 'Load', None),
             'asynchronous_motor': _translate('view type', 'Induction motor', None),
             'two_windings_transformer': _translate('view type', 'Two winding transformer', None),
             'three_windings_transformer': _translate('view type', 'Three winding transformer', None),
             'ground': _translate('view type', 'Ground', None)}
    return types


def v_types():
    return {'RX_post': _translate('vetv type', 'RX resistance', None),
            'ground': _translate('vetv type', 'Ground', None),
            'P_post': _translate('vetv type', 'Load constant P', None),
            'Q_post': _translate('vetv type', 'Load constant Q', None),
            'PQ_post': _translate('vetv type', 'Load constant P and Q', None),
            'PQ_perem': _translate('vetv type', 'Load-dependent voltage', None),
            'nagr_EDS': _translate('vetv type', 'Load with EMF', None),
            'nagr_EDS_P_post': _translate('vetv type', 'Load with EMF constant P', None),
            'nagr_EDS_Q_post': _translate('vetv type', 'Load with EMF constant Q', None),
            'nagr_EDS_PQ_post': _translate('vetv type', 'Load with EMF constant P and Q', None),
            'EDS_PU_post': _translate('vetv type', 'EMF set P and U', None),
            'EDS_QU_post': _translate('vetv type', 'EMF set Q and U', None),
            'EDS_PQ_post': _translate('vetv type', 'EMF set P and Q', None),
            'EDS_PQ_perem': _translate('vetv type', 'EMF', None),
            'ist_toka': _translate('vetv type', 'Current source', None),
            'EDS_U_post': _translate('vetv type', 'EMF set U', None),
            'ist_toka_U_post': _translate('vetv type', 'Current source set U', None),
            'RX_voltage_addition': _translate('vetv type', 'RX resistance and voltage addition', None)}


def usli_view_types():
    return {'': _translate('node view type', 'Other', None)}