# -*- coding: utf-8 -*-
"""В модуле содержатся классы предназначенные для работы системы модель-представление в графическом интерфейсе PyQt.
"""

from PyQt4 import QtGui, QtCore
import classes
import gui_classes
import language
import prog_settings


class ComboBoxVetvType(QtGui.QItemDelegate):
    """Делегат для выбора типа ветви.
    """

    def createEditor(self, parent, option, index):
        # Создание выпадающего списка.
        combobox = QtGui.QComboBox(parent)
        combobox.setStyleSheet('font-size: 8pt;')
        combobox.addItems(prog_settings.v_types_list())
        return combobox

    def setEditorData(self, editor, index):
        # Выбор нужного элемента в выпадающем списке.
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_str = unicode(item_var.toString())
        if item_str in prog_settings.v_types_list():
            editor.setCurrentIndex(prog_settings.v_types_list().index(item_str))

    def setModelData(self, editor, table_model, index):
        # Запись выбранного элемента выпадающего списка в модель.
        data_int = editor.currentIndex()
        data_str = prog_settings.v_types_list()[data_int]
        data_key = ''
        for key, val in language.v_types().items():
            if val == data_str:
                data_key = key
                break
        data_var = QtCore.QVariant(data_key)
        table_model.setData(index, data_var)


class ComboBoxViewType(QtGui.QItemDelegate):
    """Делегат для выбора графического вида ветви.
    """

    def createEditor(self, parent, option, index):
        # Создание выпадающего списка.
        element_type = index.model().view.element.type()
        combobox = QtGui.QComboBox(parent)
        combobox.setStyleSheet('font-size: 8pt;')
        if element_type == 'vetv':
            v_type = index.model().view.element.input_parameters().vetv_type().value()
            combobox.addItems(prog_settings.vetvi_view(v_type)[1])
        elif element_type == 'usel':
            combobox.addItems(prog_settings.usli_view_types_list())
        return combobox

    def setEditorData(self, editor, index):
        # Выбор нужного элемента в выпадающем списке.
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_str = unicode(item_var.toString())

        element_type = index.model().view.element.type()
        if element_type == 'vetv':
            v_type = index.model().view.element.input_parameters().vetv_type().value()
            element_list = prog_settings.vetvi_view(v_type)[1]
        elif element_type == 'usel':
            element_list = prog_settings.usli_view_types_list()
        else:
            element_list = {}

        if element_type == 'vetv':
            if item_str in element_list:
                editor.setCurrentIndex(element_list.index(item_str))

    def setModelData(self, editor, table_model, index):
        # Запись выбранного элемента выпадающего списка в модель.
        data_int = editor.currentIndex()
        v_type = index.model().view.element.input_parameters().vetv_type().value()
        data_str = prog_settings.vetvi_view(v_type)[1][data_int]
        data_key = ''

        element_type = index.model().view.element.type()
        if element_type == 'vetv':
            element_values = prog_settings.vetvi_view(v_type)[0]
        elif element_type == 'usel':
            element_values = language.usli_view_types()
        else:
            element_values = {}

        for key, val in element_values.items():
            if val == data_str:
                data_key = key
                break

        data_var = QtCore.QVariant(data_key)
        table_model.setData(index, data_var)


class ComboBoxUsli(QtGui.QItemDelegate):
    """Делегат для выбора начального или конечного узла.
    """

    def __init__(self, usli=classes.Parameters('usli'), parent=None):
        QtGui.QItemDelegate.__init__(self, parent)
        self.usli = usli

    def createEditor(self, parent, option, index):
        # Создание выпадающего списка.
        combobox = QtGui.QComboBox(parent)
        combobox.setStyleSheet('font-size: 8pt;')
        combobox.addItems([u'0'] + self.usli.children_list())
        return combobox

    def setEditorData(self, editor, index):
        # Выбор нужного элемента в выпадающем списке.
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_str = unicode(item_var.toString())
        if item_str in self.usli.children_list():
            editor.setCurrentIndex(self.usli.children_list().index(item_str) + 1)
        elif item_str == u'0':
            editor.setCurrentIndex(0)

    def setModelData(self, editor, table_model, index):
        # Запись выбранного элемента выпадающего списка в модель.
        data_int = editor.currentIndex() - 1
        data_str = u''
        if data_int in range(self.usli.child_count()):
            data_str = self.usli.children_list()[data_int]
        elif data_int == -1:
            data_str = u'0'
        data_var = QtCore.QVariant(data_str)
        table_model.setData(index, data_var)


class ComboBoxVetvi(QtGui.QItemDelegate):
    """Делегат для выбора элемента.
    """

    def __init__(self, vetvi=classes.Parameters('vetvi'), parent=None):
        QtGui.QItemDelegate.__init__(self, parent)
        self.vetvi = vetvi

    def createEditor(self, parent, option, index):
        # Создание выпадающего списка.
        combobox = QtGui.QComboBox(parent)
        combobox.setStyleSheet('font-size: 8pt;')
        combobox.addItems([u''] + self.vetvi.children_list())
        return combobox

    def setEditorData(self, editor, index):
        # Выбор нужного элемента в выпадающем списке.
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_str = unicode(item_var.toString())
        if item_str in self.vetvi.children_list():
            editor.setCurrentIndex(self.vetvi.children_list().index(item_str) + 1)
        elif item_str == u'':
            editor.setCurrentIndex(0)

    def setModelData(self, editor, table_model, index):
        # Запись выбранного элемента выпадающего списка в модель.
        data_int = editor.currentIndex() - 1
        data_str = u''
        if data_int in range(self.vetvi.child_count()):
            data_str = self.vetvi.children_list()[data_int]
        elif data_int == -1:
            data_str = u''
        data_var = QtCore.QVariant(data_str)
        table_model.setData(index, data_var)


class LineEdit(QtGui.QItemDelegate):

    def createEditor(self, parent, option, index):
        lineedit = QtGui.QLineEdit(parent)
        lineedit.setStyleSheet('font-size: 8pt;')
        return lineedit

    def setEditorData(self, editor, index):
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_str = unicode(item_var.toString())
        editor.setText(item_str)

    def setModelData(self, editor, table_model, index):
        data_str = unicode(editor.text())
        data_var = QtCore.QVariant(data_str)
        table_model.setData(index, data_var)


class CheckBox(QtGui.QItemDelegate):

    def createEditor(self, parent, option, index):
        check_box = QtGui.QCheckBox(parent)
        return check_box

    def setEditorData(self, editor, index):
        item_var = index.data(QtCore.Qt.DisplayRole)
        item_bool = item_var.toBool()
        if item_bool:
            editor.setCheckState(2)
        else:
            editor.setCheckState(0)

    def setModelData(self, editor, table_model, index):
        data_str = unicode(editor.checkState())
        data_var = QtCore.QVariant(data_str)
        table_model.setData(index, data_var)


class PropertiesDelegate(QtGui.QStyledItemDelegate):
    """Общий делегат для таблицы параметров, который создает нужный делегат для конкретного параметра.
    """

    def __init__(self, usli=classes.Parameters('usli'), vetvi=classes.Parameters('vetvi'), parent=None):
        QtGui.QStyledItemDelegate.__init__(self, parent)
        self.usli = usli
        self.vetvi = vetvi
        self._parent = parent

    def delegate(self, node):
        if node.name() in ('begin', 'end', 'end_2'):
            new_delegate = ComboBoxUsli(self.usli, self._parent)
        elif node.name() == 'vetv_type':
            new_delegate = ComboBoxVetvType(self._parent)
        elif node.name() == 'generator_switcher':
            new_delegate = ComboBoxVetvi(self.vetvi, self._parent)
        elif node.name() == 'view':
            new_delegate = ComboBoxViewType(self._parent)
        elif node.name() in classes.BOOL_PARAMETERS:
            new_delegate = CheckBox(self._parent)
        else:
            new_delegate = LineEdit(self._parent)
        return new_delegate

    def createEditor(self, parent, option, index):
        node = index.internalPointer()
        new_delegate = self.delegate(node)
        return new_delegate.createEditor(parent, option, index)

    def setEditorData(self, editor, index):
        node = index.internalPointer()
        new_delegate = self.delegate(node)
        new_delegate.setEditorData(editor, index)

    def setModelData(self, editor, table_model, index):
        node = index.internalPointer()
        new_delegate = self.delegate(node)
        new_delegate.setModelData(editor, table_model, index)

    def updateEditorGeometry(self, widget, option, index):
        widget.setGeometry(option.rect)


class VetviModel(QtCore.QAbstractTableModel):
    """Модель таблицы ветвей.
    """
    
    def __init__(self, node_scheme, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self.parent = parent
        self.node_scheme = node_scheme
        input_parameters = prog_settings.VetvInputParameters()
        # Создание делегата для всех ячеек.
        self.parent.setItemDelegate(LineEdit())
        # Создание делегата для столбца типа схемы.
        self.combo_box_vetv_type = ComboBoxVetvType()
        if 'vetv_type' in input_parameters.keys:
            self.parent.setItemDelegateForColumn(input_parameters.keys.index('vetv_type'),
                                                 self.combo_box_vetv_type)
        # Создание делегата для столбцов начального и конечных узлов.
        self.combo_box_usli = ComboBoxUsli(self.node_scheme.usli)
        self.parent.setItemDelegateForColumn(input_parameters.keys.index('begin'), self.combo_box_usli)
        self.parent.setItemDelegateForColumn(input_parameters.keys.index('end'), self.combo_box_usli)
        self.parent.setItemDelegateForColumn(input_parameters.keys.index('end_2'), self.combo_box_usli)
        # Создание заголовка
        self.column_names = input_parameters.names_descriptions
        # Вставка единиц измерения в заголовке
        for column in range(len(self.column_names)):
            self.column_names[column][1] = \
                self.column_names[column][1].replace('unit_voltage', self.node_scheme.unit_voltage)
            self.column_names[column][1] = \
                self.column_names[column][1].replace('unit_active_power', self.node_scheme.unit_active_power)
            self.column_names[column][1] = \
                self.column_names[column][1].replace('unit_reactive_power', self.node_scheme.unit_reactive_power)
            self.column_names[column][1] = \
                self.column_names[column][1].replace('unit_full_power', self.node_scheme.unit_full_power)
            self.column_names[column][1] = \
                self.column_names[column][1].replace('unit_current', self.node_scheme.unit_current)
        if self.node_scheme.scheme_type == '3_phase':
            pass
        
    def rowCount(self, parent):
        # Количество строк в таблице ветвей.
        return self.node_scheme.vetvi.child_count()
        
    def columnCount(self, parent):
        # Количество столбцов в таблице ветвей.
        return len(self.column_names)
    
    def headerData(self, section, orientation, role):
        # Заголовок таблицы ветвей.
        if role == QtCore.Qt.DisplayRole:
            if orientation == QtCore.Qt.Horizontal:
                return self.column_names[section][1]
            else:
                return QtCore.QString("%1").arg(section)
        if role == QtCore.Qt.ToolTipRole:
            if orientation == QtCore.Qt.Horizontal:
                return self.column_names[section][2]
    
    def data(self, index, role):
        row = index.row()
        column = index.column()
        column_name = self.column_names[column][0]
            
        if role in (QtCore.Qt.DisplayRole, QtCore.Qt.EditRole, QtCore.Qt.ToolTipRole):
            # Содержание ячейки.
            if column_name == 'name':
                # Отображение имени ветви.
                item = self.node_scheme.vetvi.child(row).name()
            elif column_name == 'vetv_type':
                # Отображение типа ветви.
                v_type = self.node_scheme.vetvi.child(row).input_parameters().vetv_type().value()
                if v_type in language.v_types():
                    if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
                        item = language.v_types()[v_type]
                    else:
                        item = v_type
                else:
                    item = ''
            else:
                # Отображение остальных столбцов.
                param = self.node_scheme.vetvi.child(row).input_parameters().find_child(column_name)
                if param:
                    item = param.value()
                else:
                    item = ''
            return item
        elif role == QtCore.Qt.BackgroundRole:
            # Фон ячейки.
            background = QtGui.QBrush()
            background.setStyle(QtCore.Qt.SolidPattern)
            if index.row() % 2 != 0:
                background.setColor(QtGui.QColor('#E3E1D6'))
            else:
                background.setColor(QtGui.QColor('#FDFBF3'))
            return background
        elif role == QtCore.Qt.TextAlignmentRole:
            # Выравнивание текста в ячейке.
            if column_name not in ('name', 'note', 'vetv_type'):
                return QtCore.Qt.AlignCenter
            else:
                return QtCore.Qt.AlignVCenter
        
    def flags(self, index):
        return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
    
    def setData(self, index, value, role=QtCore.Qt.EditRole):
        row = index.row()
        column = index.column()
        column_name = self.column_names[column][0]
        
        if role == QtCore.Qt.EditRole:
            vetv = self.node_scheme.vetvi.child(row)
            if column_name == 'name':
                # Редактирование имени ветви.
                # Новое имя ветви.
                vetv_new_name = unicode(value.toString())
                if vetv_new_name != '':
                    if vetv_new_name[0] != '-':
                        # Если новое и старое имена не совпадают.
                        if vetv_new_name != vetv.name():
                            vetv.set_name(vetv_new_name)
            else:
                # Остальные столбцы.
                new_value = unicode(value.toString())
                param = vetv.input_parameters().find_child(column_name)
                if not param:
                    input_parameters = vetv.input_parameters()

                    if column_name in classes.UNICODE_PARAMETERS:
                        param = input_parameters.parameter(column_name, 'unicode')
                        param.set_value(unicode(value.toString()))
                    elif column_name in classes.FLOAT_PARAMETERS:
                        param = input_parameters.parameter(column_name, 'float')
                        param.set_value(unicode(value.toString()))
                    elif column_name in classes.BOOL_PARAMETERS:
                        param = input_parameters.parameter(column_name, 'bool')
                        param.set_value(value.toBool())
                    else:
                        return False
                else:
                    param.set_value(new_value)
            return True
        return False
    
    def insertRow(self, position, parent=QtCore.QModelIndex()):
        self.beginInsertRows(parent, position, position)
        self.node_scheme.insert_element(position=position)
        self.endInsertRows()
        return True
    
    def removeRow(self, position, parent=QtCore.QModelIndex()):
        # Удаление ветви.
        self.beginRemoveRows(parent, position, position)
        self.node_scheme.vetvi.remove_child(position)
        self.endRemoveRows()
        return True


class UsliModel(QtCore.QAbstractTableModel):
    """Модель таблицы узлов.
    """

    def __init__(self, node_scheme, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self.parent = parent
        self.node_scheme = node_scheme
        # Создание делегата для всех ячеек.
        self.parent.setItemDelegate(LineEdit())
        # Создание заголовка.
        self.column_names = prog_settings.UselInputParameters().names_descriptions

    def rowCount(self, parent):
        # Количество строк в таблице узлов.
        return self.node_scheme.usli.child_count()

    def columnCount(self, parent):
        # Количество столбцов в таблице узлов.
        return len(self.column_names)

    def headerData(self, section, orientation, role):
        # Заголовок таблицы узлов.
        if role == QtCore.Qt.DisplayRole:
            if orientation == QtCore.Qt.Horizontal:
                return self.column_names[section][1]
            else:
                return QtCore.QString("%1").arg(section)
        elif role == QtCore.Qt.ToolTipRole:
            if orientation == QtCore.Qt.Horizontal:
                return self.column_names[section][2]

    def data(self, index, role):
        row = index.row()
        column = index.column()
        column_name = self.column_names[column][0]

        if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole, QtCore.Qt.EditRole):
            # Отображение имени узла.
            if column_name == 'name':
                item = self.node_scheme.usli.child(row).name()
            # Отображение остальных столбцов.
            else:
                param = self.node_scheme.usli.child(row).input_parameters().find_child(column_name)
                if param:
                    item = param.value()
                else:
                    item = ''
            return item
        elif role == QtCore.Qt.BackgroundRole:
            background = QtGui.QBrush()
            background.setStyle(QtCore.Qt.SolidPattern)
            if index.row() % 2 != 0:
                background.setColor(QtGui.QColor('#E3E1D6'))
            else:
                background.setColor(QtGui.QColor('#FDFBF3'))
            return background
        elif role == QtCore.Qt.TextAlignmentRole:
            if column_name in ('name', 'note'):
                return QtCore.Qt.AlignVCenter
            else:
                return QtCore.Qt.AlignCenter

    def flags(self, index):
        return QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        row = index.row()
        column = index.column()
        column_name = self.column_names[column][0]

        if role == QtCore.Qt.EditRole:
            usel = self.node_scheme.usli.child(row)
            # Редактирование имени узла.
            if column_name == 'name':
                # Новое имя узла.
                usel_new_name = unicode(value.toString())
                if usel_new_name != '':
                    if usel_new_name != '0' and usel_new_name[0] != '-':
                        usel_old_name = usel.name()
                        # Если новое и старое имена не совпадают.
                        if usel_new_name != usel_old_name:
                            if usel.set_name(usel_new_name):
                                self.node_scheme.rename_usel_in_vetvi(usel_old_name, usel_new_name)
            # Остальные столбцы.
            else:
                new_value = unicode(value.toString())
                param = usel.input_parameters().find_child(column_name)
                if not param:
                    input_parameters = usel.input_parameters()
                    input_parameters.add_child(classes.Parameters(column_name, 'unicode', new_value, input_parameters))
                else:
                    param.set_value(new_value)
            return True
        return False

    def insertRow(self, position, parent=QtCore.QModelIndex()):
        self.beginInsertRows(parent, position, position)
        self.node_scheme.insert_usel(position=position)
        self.endInsertRows()
        return True

    def removeRow(self, position, parent=QtCore.QModelIndex()):
        # Удаление узла.
        self.beginRemoveRows(parent, position, position)
        self.node_scheme.rename_usel_in_vetvi(self.node_scheme.usli.child(position).name(), u'')
        self.node_scheme.usli.remove_child(position)
        self.endRemoveRows()
        return True


class OptimizationTableModel(QtCore.QAbstractTableModel):
    """Модель таблицы вариантов схемы, полученных при оптимизации.
    """

    def __init__(self, list_node_schemes, node_scheme, parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self.parent = parent
        self.list_node_schemes = list_node_schemes
        # Создание заголовка.
        self.column_names = language.Other().optimization_table_header
        for column in range(len(self.column_names)):
            self.column_names[column] = \
                self.column_names[column].replace('unit_active_power', node_scheme.unit_active_power)
            self.column_names[column] = \
                self.column_names[column].replace('unit_reactive_power', node_scheme.unit_reactive_power)

    def rowCount(self, parent):
        # Количество строк.
        return len(self.list_node_schemes)

    def columnCount(self, parent):
        # Количество столбцов.
        return len(self.column_names)

    def headerData(self, section, orientation, role):
        # Заголовок таблицы.
        if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
            if orientation == QtCore.Qt.Horizontal:
                return self.column_names[section]
            else:
                return QtCore.QString("%1").arg(section)
        return

    def data(self, index, role):
        row = index.row()
        column = index.column()

        if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
            if column == 0:
                return self.list_node_schemes[row][0]
            elif column in (1, 2):
                node_scheme = self.list_node_schemes[row][1]
                round_n = node_scheme.settings.round_n
                if column == 1:
                    sum_power_loss = self.list_node_schemes[row][2]
                else:
                    sum_power_loss = self.list_node_schemes[row][3]
                sum_power_loss = round(sum_power_loss, round_n)
                return unicode(sum_power_loss)
            elif column == 3:
                return self.list_node_schemes[row][4]
            else:
                return
        elif role == QtCore.Qt.BackgroundRole:
            background = QtGui.QBrush()
            background.setStyle(QtCore.Qt.SolidPattern)
            if index.row() % 2 != 0:
                background.setColor(QtGui.QColor('#E3E1D6'))
            else:
                background.setColor(QtGui.QColor('#FDFBF3'))
            return background
        elif role == QtCore.Qt.TextAlignmentRole:
            if column == 0:
                return QtCore.Qt.AlignVCenter
            else:
                return QtCore.Qt.AlignCenter
        return

    def flags(self, index):
        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable


class NewElementsTableModel(QtCore.QAbstractListModel):
    """Модель таблицы добавления новых элементов.
    """

    def __init__(self, element_group_obj, parent=None):
        QtCore.QAbstractListModel.__init__(self, parent)
        self.parent = parent
        self.element_group_obj = element_group_obj

    def rowCount(self, parent):
        # Количество строк.
        return self.element_group_obj.child_count()

    def data(self, index, role):
        row = index.row()

        if role == QtCore.Qt.DecorationRole:
            icon = None
            element = self.element_group_obj.child(row)
            image_path = element.find_child('image')
            if image_path:
                if image_path.value():
                    image = QtGui.QPixmap(image_path.value())
                    icon = QtGui.QIcon(image)
            return icon
        elif role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
            element = self.element_group_obj.child(row)
            element_name = element.value()
            return element_name
        elif role == QtCore.Qt.TextAlignmentRole:
            return QtCore.Qt.AlignVCenter
        return

    def flags(self, index):
        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable


class TableResultsModel(QtCore.QAbstractTableModel):
    """Модель таблицы результатов расчета.
    """
    
    def __init__(self, results_table=[[]], parent=None):
        QtCore.QAbstractTableModel.__init__(self, parent)
        self.results_table = results_table
        
    def rowCount(self, parent):
        # Количество строк таблицы.
        return len(self.results_table)
        
    def columnCount(self, parent):
        # Количество столбцов таблицы.
        col_count = 0
        for row in self.results_table:
            if len(row) > col_count:
                col_count = len(row)
        return col_count
        
    def data(self, index, role):
        if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
            # Содержание ячейки.
            try:
                item = self.results_table[index.row()][index.column()]
            except IndexError:
                item = ''
            return item
        elif role == QtCore.Qt.BackgroundRole:
            # Фон ячейки.
            row = index.row()
            background = QtGui.QBrush()
            background.setStyle(QtCore.Qt.SolidPattern)
            if len(self.results_table[row]) == 1:
                background.setColor(QtGui.QColor('#E3E1D6'))
            else:
                background.setColor(QtGui.QColor('#FDFBF3'))
            return background
        elif role == QtCore.Qt.TextAlignmentRole:
            # Выравнивание текста в ячейке.
            column = index.column()
            if column != 0:
                return QtCore.Qt.AlignCenter
            else:
                return QtCore.Qt.AlignVCenter

    def flags(self, index):
        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable


class PropertiesModel(QtCore.QAbstractItemModel):
    """Модель таблицы свойств элемента.
    """

    def __init__(self, view, node_scheme, parent=None):
        super(PropertiesModel, self).__init__(parent)
        self.view = view
        self.node_scheme = node_scheme
        self.element = view.element
        if self.view.element.type() == 'usel':
            self._rootNode = gui_classes.UselParameters(node_scheme)
        else:
            self._rootNode = gui_classes.VetvParameters(node_scheme, self.element)
        # Установка делегатов.
        self.properties_delegate = PropertiesDelegate(self.node_scheme.usli, self.node_scheme.vetvi)
        parent.setItemDelegateForColumn(1, self.properties_delegate)

    def rowCount(self, parent):
        # Количество строк таблицы свойств.
        if not parent.isValid():
            parent_node = self._rootNode
        else:
            parent_node = parent.internalPointer()

        return parent_node.child_count()

    def columnCount(self, parent):
        # Количество столбцов таблицы свойств.
        return 2

    def data(self, index, role):
        if not index.isValid():
            return None
        node = index.internalPointer()
        column = index.column()
        if role == QtCore.Qt.ToolTipRole:
            # Отображение всплывающей подсказки.
            if column == 0:
                return node.description()
            else:
                return self.return_data(role, node)
        elif role in (QtCore.Qt.DisplayRole, QtCore.Qt.EditRole):
            # Отображение значения ячейки.
            if column == 0:
                # Отображение имени параметра.
                return node.value()
            else:
                # Отображение значения параметра.
                return self.return_data(role, node)

        return None

    def return_data(self, role, node):
        # Отображение значения параметра.
        if node.name() == 'name':
            # Отображение имени элеметра (элемента или узла).
            return self.element.name()
        elif node.name() == 'vetv_type':
            # Отображение типа элеметра (элемента или узла).
            v_type = self.element.input_parameters().vetv_type().value()
            if v_type in language.v_types():
                if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
                    item = language.v_types()[v_type]
                else:
                    item = v_type
            else:
                item = ''
            return item
        elif node.name() == 'view':
            # Отображение вида элеметра (элемента или узла).
            view_type = self.element.gui_parameters().view().value()
            if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
                if self.view.element.type() == 'usel':
                    item = language.usli_view_types().get(view_type, '')
                else:
                    item = prog_settings.vetvi_view()[0].get(view_type, '')
            else:
                item = view_type
            return item
        else:
            # Отображение других параметров элемента (ветви или узла).
            parent = node.parent()
            if not parent:
                return None
            parameters_group = self.element.find_child(parent.name())
            if parameters_group:
                param = parameters_group.find_child(node.name())
                if param:
                    param = param.value()
                    param_val = param
                    if isinstance(param, complex):
                        # Отображение значения параметра, являющего комплексным числом.
                        if parameters_group.name() == 'output_parameters':
                            param_real = round(param.real, self.node_scheme.settings.round_n)
                            param_imag = round(param.imag, self.node_scheme.settings.round_n)
                            param_val = param_real + param_imag*1j
                    elif isinstance(param, float):
                        # Отображение значения параметра, являющего числом с плавающий точкой.
                        if parameters_group.name() == 'output_parameters':
                            param_val = round(param, self.node_scheme.settings.round_n)
                    elif isinstance(param, list):
                        # Отображение значения параметра, являющегося списком.
                        param_val = u''
                        for i in range(len(param)):
                            i_val = param[i]
                            if isinstance(i_val, complex):
                                if parameters_group.name() == 'output_parameters':
                                    i_real = round(i_val.real, self.node_scheme.settings.round_n)
                                    i_imag = round(i_val.imag, self.node_scheme.settings.round_n)
                                    i_val = i_real + i_imag*1j
                            elif isinstance(i_val, float):
                                if parameters_group.name() == 'output_parameters':
                                    i_val = round(i_val, self.node_scheme.settings.round_n)
                            param_val += unicode(i_val)
                            if i != len(param)-1:
                                param_val += u'; \n'
                    return unicode(param_val)
        return None

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        if not index.isValid():
            return False
        column = index.column()
        if column != 1 and role != QtCore.Qt.EditRole:
            return False

        node = index.internalPointer()
        if node.name() == 'name':
            # Если изменено название элемента.
            old_name = self.element.name()
            new_name = unicode(value.toString())
            if new_name != '':
                if new_name[0] != '-':
                    # Название не должно начинаться со знака "-".
                    if old_name != new_name:
                        if self.element.set_name(new_name):
                            self.view.update_group()
                            if self.view.element.type() == 'usel':
                                self.node_scheme.rename_usel_in_vetvi(old_name, new_name)
        else:
            # Если изменен другой параметр.
            parent = node.parent()
            if not parent:
                return False
            parameters_group = self.element.find_child(parent.name())
            if not parameters_group:
                return False

            if node.name() in classes.UNICODE_PARAMETERS:
                param = parameters_group.parameter(node.name(), 'unicode')
                param.set_value(unicode(value.toString()))
            elif node.name() in classes.FLOAT_PARAMETERS:
                param = parameters_group.parameter(node.name(), 'float')
                param.set_value(unicode(value.toString()))
            elif node.name() in classes.BOOL_PARAMETERS:
                param = parameters_group.parameter(node.name(), 'bool')
                param.set_value(value.toBool())
            else:
                return False

            if node.name() in ('begin', 'end', 'end_2'):
                # Если изменено начало или конец ветви.
                self.view.scene().update_connection_lines()
            else:
                # Если изменен любой другой входной параметр.
                self.view.update_group()
        return True

    def headerData(self, section, orientation, role):
        if role == QtCore.Qt.DisplayRole:
            if section == 0:
                return language.Other().parameters_header[0]
            else:
                return language.Other().parameters_header[1]

    def flags(self, index):
        column = index.column()
        if column == 0:
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
        else:
            node = self.get_node(index)
            if node.parent():
                if node.parent().name() in ('input_parameters', 'gui_parameters', 'additional_parameters',
                                            'optimization_parameters'):
                    return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEditable
                elif node.parent().name() == 'output_parameters':
                    return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
            return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable

    def parent(self, index):
        node = self.get_node(index)
        parent_node = node.parent()

        if parent_node == self._rootNode:
            return QtCore.QModelIndex()
        return self.createIndex(parent_node.row(), 0, parent_node)

    def index(self, row, column, parent):
        parent_node = self.get_node(parent)
        child_item = parent_node.child(row)

        if child_item:
            return self.createIndex(row, column, child_item)
        else:
            return QtCore.QModelIndex()

    def get_node(self, index):
        if index.isValid():
            node = index.internalPointer()
            if node:
                return node

        return self._rootNode