# -*- coding: utf-8 -*-
""" Модуль описывает функции и переменные, которые используются при выводе текстовой информации в графическом
интерфейсе программы.
"""

import language

VETVI_VIEW_SWITCHERS = ['lv_switch', 'hv_switch', 'breaker']
VETVI_VIEW_3_LINES = ['three_windings_transformer']
VETVI_VIEW_TR = ['cable', 'line', 'two_windings_transformer', 'three_windings_transformer']
VETVI_VIEW_LINES = ['cable', 'line']
VETVI_VIEW_GENERATORS = ['dc_source', 'ac_source']
VETVI_VIEW_LOAD = ['load']
VETVI_VIEW_TRANSFORMERS = ['two_windings_transformer', 'three_windings_transformer']

RX = ['RX_post', 'RX_voltage_addition']
NAGR = ['P_post', 'Q_post', 'PQ_post', 'PQ_perem']
NAGR_EDS = ['nagr_EDS', 'nagr_EDS_P_post', 'nagr_EDS_Q_post', 'nagr_EDS_PQ_post']
EDS = ['EDS_PU_post', 'EDS_QU_post', 'EDS_PQ_post', 'EDS_PQ_perem', 'EDS_U_post']
II = ['ist_toka', 'ist_toka_U_post']


class VetvInputParameters(object):

    def __init__(self, vetv_type='', vetv_view=''):
        # Входные параметры ветвей, список которых зависит от типа ветви.
        self.keys = ['name', 'note', 'begin', 'end']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['end_2']
        self.keys += ['real_voltage', 'real_phase']
        if vetv_type in RX:
            self.keys += ['active_resistance', 'reactive_resistance']
            if vetv_view in VETVI_VIEW_3_LINES:
                self.keys += ['active_resistance_2', 'reactive_resistance_2', 'active_resistance_3',
                              'reactive_resistance_3']
            if vetv_view in VETVI_VIEW_TR:
                self.keys += ['active_conductivity_tr1', 'reactive_conductivity_tr1', 'active_conductivity_tr2',
                              'reactive_conductivity_tr2']
            if vetv_view in VETVI_VIEW_TRANSFORMERS:
                self.keys += ['voltage', 'transformation_coefficient', 'transformation_coefficient_psi']
                if vetv_view in VETVI_VIEW_3_LINES:
                    self.keys += ['transformation_coefficient_2', 'transformation_coefficient_psi_2']
        elif vetv_type == 'ground':
            self.keys += []
        elif vetv_type in NAGR:
            self.keys += ['voltage', 'active_power', 'reactive_power']
        elif vetv_type in NAGR_EDS:
            self.keys += ['voltage', 'active_power', 'reactive_power', 'active_resistance',
                          'reactive_resistance']
        elif vetv_type == 'EDS_PU_post':
            self.keys += ['voltage', 'voltage_psi', 'active_power', 'active_resistance',
                          'reactive_resistance']
        elif vetv_type == 'EDS_QU_post':
            self.keys += ['voltage', 'voltage_psi', 'reactive_power', 'active_resistance',
                          'reactive_resistance']
        elif vetv_type == 'EDS_PQ_post':
            self.keys += ['voltage', 'voltage_psi', 'active_power', 'reactive_power',
                          'active_resistance', 'reactive_resistance']
        elif vetv_type == 'EDS_PQ_perem':
            if vetv_view in VETVI_VIEW_GENERATORS:
                self.keys += ['voltage', 'voltage_psi', 'active_resistance', 'reactive_resistance']
            else:
                self.keys += ['active_resistance', 'reactive_resistance', 'eds', 'eds_psi']
        elif vetv_type == 'EDS_U_post':
            self.keys += ['voltage', 'voltage_psi', 'active_resistance', 'reactive_resistance']
        elif vetv_type == 'ist_toka':
            self.keys += ['current', 'current_psi']
        elif vetv_type == 'ist_toka_U_post':
            self.keys += ['voltage', 'voltage_psi']
        else:
            self.keys = ['name', 'note', 'begin', 'end', 'end_2', 'real_voltage', 'real_phase', 'voltage',
                         'voltage_psi', 'active_power', 'reactive_power', 'active_resistance', 'reactive_resistance',
                         'active_resistance_2', 'reactive_resistance_2', 'active_resistance_3', 'reactive_resistance_3',
                         'eds', 'eds_psi', 'current', 'current_psi', 'active_conductivity_tr1',
                         'reactive_conductivity_tr1', 'active_conductivity_tr2', 'reactive_conductivity_tr2',
                         'voltage_addition', 'voltage_addition_psi', 'transformation_coefficient',
                         'transformation_coefficient_psi', 'transformation_coefficient_2',
                         'transformation_coefficient_psi_2']
        self.parameters = language.Parameters().input_parameters
        self.names_descriptions = [[name, self.parameters[name][0],
                                    self.parameters[name][1]] for name in self.keys]
        self.names = [row[1] for row in self.names_descriptions]
        self.widths = [self.parameters[name][2] for name in self.keys]


class VetvGuiParameters(object):

    def __init__(self, vetv_view=''):
        # Гарфические параметры ветвей.
        self.keys = ['position_x', 'position_y', 'rotation']
        self.parameters = language.Parameters().gui_parameters
        self.names_descriptions = [[name, self.parameters[name]] for name in self.keys]
        self.names = [self.parameters[key] for key in self.keys]


class VetvAdditionalParameters(object):

    def __init__(self, vetv_view=''):
        # Дополнительные параметры ветвей.
        self.keys = []
        if vetv_view in VETVI_VIEW_LINES:
            self.keys += ['nominal_temperature', 'operating_temperature', 'short_current_temperature',
                          'temperature_coefficient']
        self.parameters = language.Parameters().additional_parameters
        self.names_descriptions = [[name, self.parameters[name]] for name in self.keys]
        self.names = [self.parameters[key] for key in self.keys]


class VetvOptimizationParameters(object):

    def __init__(self, vetv_view=u''):
        # Оптимизационные параметры ветвей.
        self.keys = []
        if vetv_view in VETVI_VIEW_GENERATORS:
            self.keys += ['use_for_optimization', 'generator_switcher', 'switcher_permit', 'generator_priority',
                          'generator_nominal_active_power', 'generator_nominal_reactive_power',
                          'generator_nominal_full_power', 'generator_normal_power_coefficient',
                          'generator_overdrive_power_coefficient']
        elif vetv_view in VETVI_VIEW_LOAD:
            self.keys += ['use_for_optimization', 'minimum_voltage', 'maximum_voltage']
        elif vetv_view in VETVI_VIEW_SWITCHERS:
            self.keys += ['use_for_optimization', 'switcher_permit', 'check_direction']
        self.parameters = language.Parameters().optimization_parameters
        self.names_descriptions = [[name, self.parameters[name]] for name in self.keys]
        self.names = [self.parameters[key] for key in self.keys]


class VetvOutputParameters(object):

    def __init__(self, vetv_type='', vetv_view=''):
        # Выходные параметры ветвей.
        self.keys = ['real_u']
        if vetv_type in (EDS + NAGR_EDS):
            self.keys += ['real_e']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['real_u_2', 'real_u_3']
        self.keys += ['real_u1']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['real_u1_2', 'real_u1_3']
        self.keys += ['real_u2']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['real_u2_2', 'real_u2_3']
        self.keys += ['real_i']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['real_i_2', 'real_i_3']
        self.keys += ['s1']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['s1_2', 's1_3']
        self.keys += ['s2']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['s2_2', 's2_3']
        if vetv_type in (EDS + II):
            self.keys += ['ist_s']
        self.keys += ['priem_s']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['priem_s_2', 'priem_s_3']
        if vetv_view in VETVI_VIEW_TR:
            self.keys += ['real_i1_tr', 'real_i2_tr']
        if vetv_view in VETVI_VIEW_3_LINES:
            self.keys += ['real_i3_tr']
        if vetv_view in VETVI_VIEW_TR:
            self.keys += ['priem_s1_tr', 'priem_s2_tr']
            if vetv_view in VETVI_VIEW_3_LINES:
                self.keys += ['priem_s3_tr']
        if not vetv_type and not vetv_view:
            self.keys = ['real_u', 'real_e', 'real_u_2', 'real_u_3', 'real_u1', 'real_u1_2', 'real_u1_3', 'real_u2',
                         'real_u2_2', 'real_u2_3', 'real_i', 'real_i_2', 'real_i_3', 's1', 's1_2', 's1_3', 's2',
                         's2_2', 's2_3', 'ist_s', 'priem_s', 'priem_s_2', 'priem_s_3', 'real_i1_tr', 'real_i2_tr',
                         'real_i3_tr', 'priem_s1_tr', 'priem_s2_tr', 'priem_s3_tr']
        self.parameters = language.Parameters().output_parameters
        self.names_descriptions = [[name, self.parameters[name]] for name in self.keys]
        self.names = [self.parameters[key] for key in self.keys]


class UselInputParameters(object):

    def __init__(self):
        parameters = language.Parameters().usli_input_parameters
        self.keys = ['name', 'note']
        self.names_descriptions = [[name, parameters[name][0], parameters[name][1]] for name in self.keys]
        self.names = [row[1] for row in self.names_descriptions]
        self.widths = [parameters[name][2] for name in self.keys]


class UselOutputParameters(object):

    def __init__(self):
        parameters = language.Parameters().usli_output_parameters
        self.keys = ['real_u_usel']
        self.names_descriptions = [[name, parameters[name]] for name in self.keys]
        self.names = [parameters[key] for key in self.keys]


class UselOptimizationParameters(object):

    def __init__(self):
        # Оптимизационные параметры узлов.
        parameters = language.Parameters().optimization_parameters
        self.keys = ['use_for_optimization', 'minimum_voltage', 'maximum_voltage']
        self.names_descriptions = [[name, parameters[name]] for name in self.keys]
        self.names = [parameters[key] for key in self.keys]


class UselGuiParameters(object):

    def __init__(self):
        parameters = language.Parameters().gui_parameters
        self.keys = ['position_x', 'position_y', 'rotation']
        self.names_descriptions = [[name, parameters[name]] for name in self.keys]
        self.names = [parameters[key] for key in self.keys]


def vetvi_view(vetv_type=''):
    # Графические изображения ветвей, список которых зависит от типа ветви.
    # Функция возвращает словарь с именами и назавниями изображений и список названий изображений.
    if vetv_type in RX:
        vetvi_types_names = ['', 'r', 'l', 'c', 'r_l', 'r_c', 'r_l_c', 'l_c', 'cable', 'line', 'hv_switch',
                             'lv_switch', 'breaker', 'fuse',
                             'two_windings_transformer', 'three_windings_transformer']
    elif vetv_type == 'ground':
        vetvi_types_names = ['', 'ground']
    elif vetv_type in EDS:
        vetvi_types_names = ['', 'e', 'e_r', 'e_l', 'e_c', 'e_r_l', 'e_r_c', 'e_r_l_c', 'e_l_c', 'ac_source',
                             'dc_source']
    elif vetv_type in (NAGR + NAGR_EDS):
        vetvi_types_names = ['', 'r', 'l', 'c', 'r_l', 'r_c', 'r_l_c', 'l_c', 'load', 'asynchronous_motor']
    else:
        vetvi_types_names = ['', 'r', 'l', 'c', 'r_l', 'r_c', 'r_l_c', 'l_c', 'e', 'e_r', 'e_l', 'e_c', 'e_r_l',
                             'e_r_c', 'e_r_l_c', 'e_l_c', 'i', 'cable', 'line', 'hv_switch', 'lv_switch', 'breaker',
                             'fuse', 'ac_source', 'dc_source', 'load', 'asynchronous_motor',
                             'two_windings_transformer', 'three_windings_transformer', 'ground']
    return ({view_name: language.vetvi_view_types()[view_name] for view_name in vetvi_types_names},
            [language.vetvi_view_types()[view_name] for view_name in vetvi_types_names])


def usli_view_types_list():
    return sorted(language.usli_view_types().values())


def v_types_list():
    return sorted(language.v_types().values())